/*
 * CharsetWidgets.js
 * JavaScript widgets for specifying alternate and fallback character sets.
 *
 * Copyright (C) 2009 McAfee, Inc. All rights reserved.
 */

// Non-breaking space
var NBSP = 160;

/**
 * Constructor for the CharsetMapper object
 * @param divId - the ID of a div that will be used to contain the control.
 * @param objectName - the name by which the object is declared; defaults to
 * "csMapper"
 * @param encodings a list of valid encodings
 * @param changeNotifyFn - a function to call when the widget changes
 */
function CharsetMapper(divId, objectName, encodings, changeNotifyFn) {
    this.encodings = encodings;
    // Input values take the form " x = y " with any amount of whitespace around
    // x and y handled
    this.splitPattern = /\s*(\S+)\s*=\s*(\S+)\s*/;
    this.charsetMap = {};
    this.divId = divId;
    this.objectName = objectName ? objectName : "csMapper";
    this.enabled = true;
    this.browserInfo = IdentifyBrowser();
    this.uniquifier = Math.random();
    this.currCS = null;
    this.changeNotifyFn = changeNotifyFn ?
        ('function' == typeof(changeNotifyFn) ? changeNotifyFn : new Function(changeNotifyFn)) : null;
}

// Splits and trims an input
CharsetMapper.prototype.parseInput = function(str) {
    var ret = null;
    // Remove any whitespace around the list separators
    var pat = /\s*,\s*/g;
    var match = this.splitPattern.exec(str);
    if (match) {
        ret = [match[1], match[2].replace(pat, ",")];
    }
    return ret;
}

// Sets the value of the object
CharsetMapper.prototype.setValue = function(fixedMappings, fallbacks) {
    this._setValue(fixedMappings, true);
    this._setValue(fallbacks, false);
}

// Value set helper function
CharsetMapper.prototype._setValue = function(values, isFixed) {
    for (var i = 0; i < values.length; i++) {
        var parts = this.parseInput(values[i]);
        if (null == parts)
            continue;
        this.charsetMap[parts[0]] = {value:parts[1], fixed:isFixed};
    }
}

// Gets the value of the object (1)
CharsetMapper.prototype.getFixedMappings = function() {
    var ret = [];
    var mappingList = this.keys();

    for (var i = 0; i < mappingList.length; i++) {
        var cs = mappingList[i];
        var mapping = this.charsetMap[cs];
        if (mapping.fixed)
            ret.push(cs + "=" + mapping.value);
    }
    return ret;
}

// Gets the value of the object (2)
CharsetMapper.prototype.getFallbacks = function() {
    var ret = [];
    var mappingList = this.keys();

    for (var i = 0; i < mappingList.length; i++) {
        var cs = mappingList[i];
        var mapping = this.charsetMap[cs];
        if (mapping.fixed)
            continue;
        ret.push(cs + "=" + mapping.value);
    }
    return ret;
}

// Gets the sorted list of primary character sets
CharsetMapper.prototype.keys = function() {
    var mappingList = [];
    for (var mapped in this.charsetMap) {
        mappingList.push(mapped);
    }
    mappingList.sort();
    return mappingList;
}

/**
 * Draws the CharsetMapper control
 * @param enabled true if the control is to be enabled, false otherwise
 */
CharsetMapper.prototype.draw = function(enabled) {
    this.enabled = enabled;
    var div = document.getElementById(this.divId);
    if (!div)
        return;
    while(null != div.firstChild) {
        div.removeChild(div.firstChild);
    }
    div.appendChild(this.make(enabled));
}

// Creates the elements that go to make up the control
CharsetMapper.prototype.make = function(enabled) {
    var table = document.createElement("table");
    table.id = this.divId + "_tbl";

    // Create the header
    var thead = document.createElement("thead");
    table.appendChild(thead);
    var row = thead.insertRow(0);
    this.makeHeader(row);

    var tbody = document.createElement("tbody");
    table.appendChild(tbody);
    var keys = this.keys();
    if (0 == keys.length && !this.enabled) {
        row = tbody.insertRow(0);
        var cell = document.createElement("td");
        cell.colSpan = 3;
        cell.style.textAlign = "center";
        cell.style.fontWeight = "bold";
        cell.appendChild(document.createTextNode(resolveString(cse_pp_st, "__nothing_to_display")));
        row.appendChild(cell);
    }
    else {
        row = tbody.insertRow(0);
        this.makeInputRow(row);
        for (var i = keys.length - 1; i >= 0; i--) {
            row = tbody.insertRow(0);
            this.makeRow(row, keys[i]);
        }
    }
    this.fixTable(table);
    return table;
}

CharsetMapper.prototype.makeHeader = function(row) {
    var cell = null;
    var hdrs = ["__charset", "__maps_to", "__fixed"];
    for (var i = 0; i < hdrs.length; i++) {
        cell = document.createElement("th");
        cell.appendChild(document.createTextNode(resolveString(cse_pp_st, hdrs[i])));
        row.appendChild(cell);
    }
    row.cells[row.cells.length - 1].style.textAlign = "center";

    if (this.enabled) {
        cell = document.createElement("th");
        cell.appendChild(document.createTextNode(resolveString(cse_pp_st, "__actions")));
        row.appendChild(cell);
    }
}

// Creates the input row
CharsetMapper.prototype.makeInputRow = function(row) {
    var elems = this.makeInputElements();

    for (var i = 0; i < elems.length; i++) {
        var td = document.createElement("td");
        td.vAlign = "top";
        td.appendChild(elems[i]);
        row.appendChild(td);
    }
    row.cells[2].style.textAlign = "center";
}

// Creates the contents of an input row
CharsetMapper.prototype.makeInputElements = function(cs) {
    if (null == cs)
        cs = "";
    var mapInfo = null;
    if (cs) {
        mapInfo = this.charsetMap[cs];
    }
    if (!mapInfo)
        mapInfo = {fixed:false};
    var elems = [];

    var select = document.createElement("select");
    select.id = this.uniquifier + cs + "selP";
    select.className = "mw";
    this.populateP(select, cs);
    elems.push(select);

    select = document.createElement("select");
    select.id = this.uniquifier + cs + "selS";
    select.style.width = "250px";
    select.className = "mw";
    select.size = 5;
    this.populateS(select, cs, mapInfo.fixed);
    elems.push(select);

    var cb = document.createElement("input");
    cb.type = "checkbox";
    cb.id = this.uniquifier + cs + "cbF";
    cb.className = "checkbox";
    cb.checked = mapInfo.fixed;
    var fnStr = this.objectName + ".toggleFixed(this.checked,'" + cs + "');";
    cb.onclick = new Function(fnStr);
    elems.push(cb);

    var span = document.createElement("span");
    if (cs) {
        var link = document.createElement("a");
        link.className = "blueLink";
        link.href = "javascript:" + this.objectName + ".save('" + cs + "');";
        link.appendChild(document.createTextNode(resolveString(cse_pp_st, "__save")));
        span.appendChild(link);
        span.appendChild(document.createTextNode(" | "));
        link = document.createElement("a");
        link.className = "blueLink";
        link.href = "javascript:" + this.objectName + ".cancelEdit();";
        link.appendChild(document.createTextNode(resolveString(cse_pp_st, "__cancel")));
        span.appendChild(link);
    }
    else {
        var link = document.createElement("a");
        link.className = "blueLink";
        link.href = "javascript:" + this.objectName + ".add();";
        link.appendChild(document.createTextNode(resolveString(cse_pp_st, "__add")));
        span.appendChild(link);
    }
    elems.push(span);

    return elems;
}

// Creates a row in the table
CharsetMapper.prototype.makeRow = function(row, key) {
    var rowContents = this.makeRowContents(key);
    for (var i = 0; i < rowContents.length; i++) {
        var cell = document.createElement("td");
        cell.vAlign = "top";
        cell.appendChild(rowContents[i]);
        row.appendChild(cell);
    }
    if (rowContents.length >= 3)
        row.cells[2].style.textAlign = "center";

}

// Creates the elements that make up the contents of the row
CharsetMapper.prototype.makeRowContents = function(key) {
    var elems = [];

    elems.push(document.createTextNode(resolveString(cse_pp_st, key)));

    var mDisplay = [];
    var mapList = this.charsetMap[key].value.split(",");
    for (var i = 0; i < mapList.length; i++) {
        mDisplay.push(resolveString(cse_pp_st, mapList[i]));
    }
    elems.push(document.createTextNode(mDisplay.join(", ")));

    var img = document.createElement("img");
    img.height = img.width = 13;
    img.align = "middle";
    img.src = "../../images/icons/" + (this.charsetMap[key].fixed ?
        "checkbox.gif" : "checkbox_unchecked.gif");
    elems.push(img);

    if (this.enabled) {
        var span = document.createElement("span");
        var link = document.createElement("a");
        link.className = "blueLink";
        link.href = "javascript:" + this.objectName + ".editMapping('" + key + "');";
        link.appendChild(document.createTextNode(resolveString(cse_pp_st, "__edit")));
        span.appendChild(link);
        span.appendChild(document.createTextNode(" | "));
        link = document.createElement("a");
        link.className = "blueLink";
        link.href = "javascript:" + this.objectName + ".deleteMapping('" + key + "');";
        link.appendChild(document.createTextNode(resolveString(cse_pp_st, "__delete")));
        span.appendChild(link);
        elems.push(span);
    }

    return elems;
}

// Fixes styles on the main table
CharsetMapper.prototype.fixTable = function(table) {
    table.className = "dataSubTable";
    table.width = "100%";
    table.border = table.cellPadding = table.cellSpacing = 0;
    var rows = table.rows;
    var nRows = rows.length;
    if (nRows) {
        //rows[0].className = addClass(rows[0].className, "header");
        var cells = rows[0].cells;
        for (var i = 0; i < cells.length; i++) {
            cells[i].className = addClass(cells[i].className, "dst");
            cells[cells.length - 1].className = addClass(
                cells[cells.length - 1].className, "eor");
        }
        for (var i = 1; i < nRows; i++) {
            rows[i].className = addClass(rows[i].className, "row0");
            var cells = rows[i].cells;
            var nCells = cells.length;
            for (var j = 0; j < nCells; j++) {
                cells[j].className = addClass(cells[j].className, "dst");
            }
            var cell = cells[nCells - 1];
            cell.className = addClass(cell.className, "eor");
        }
        var row = rows[nRows - 1];
        row.className = addClass(row.className, "lastRow");
    }
}

// Deletes a character set mapping
CharsetMapper.prototype.deleteMapping = function(cs) {
    if (confirm(resolveString(cse_pp_st, "__del_map_prompt")) && this.charsetMap[cs]) {
        delete this.charsetMap[cs];
        if (this.changeNotifyFn)
            this.changeNotifyFn();
        this.draw(true);
    }
}

// Called in response to an Edit command
CharsetMapper.prototype.editMapping = function(cs) {
    var formElements = this.makeInputElements(cs);
    if (this.currCS)
        this.cancelEdit();
    this.fillRow(cs, formElements);
    this.currCS = cs;
    if (this.browserInfo.isIE) {
        this.setSelect(cs);
    }
}

// Called in response to a cancel edit command
CharsetMapper.prototype.cancelEdit = function() {
    var cs = this.currCS;
    if (cs) {
        var elements = this.makeRowContents(cs);
        this.fillRow(cs, elements);
        this.currCS = null;
    }
}

// Dynamically fills in the elements of a row; used in "Edit" and "Cancel"
CharsetMapper.prototype.fillRow = function(cs, elements) {
    var rowNo = this.getOrder(cs);
    if (rowNo < 0)
        return;
    var tbody = this.getTBody();
    if (!tbody)
        return;
    var rows = tbody.rows;
    if (rowNo >= rows.length)
        return;
    var row = rows[rowNo];

    var cells = row.cells;
    for (var i = 0; i < cells.length; i++) {
        var cell = cells[i];
        while(cell.firstChild)
            cell.removeChild(cell.firstChild);
        cell.appendChild(elements[i]);
    }
}


// Gets the TBODY element that contains the tabular data
CharsetMapper.prototype.getTBody = function() {
    var ret = null;
    var tableId = this.divId + "_tbl";
    var table = document.getElementById(tableId);
    if (table) {
        var tbArray = table.getElementsByTagName("tbody");
        if (tbArray && tbArray.length > 0)
            ret = tbArray[0];
    }
    return ret;
}

// Gets a numerical order for a selected character set
CharsetMapper.prototype.getOrder = function(cs) {
    var ret = -1;
    var keys = this.keys();
    for (var i = keys.length - 1; i >= 0; i--) {
        if (cs == keys[i]) {
            ret = i;
            break;
        }
    }
    return ret;
}

// Toggles between fixed and fallback mappings
CharsetMapper.prototype.toggleFixed = function(isFixed, currSel) {
    var eKey = this.uniquifier;
    if (currSel)
        eKey += currSel;
    var selS = document.getElementById(eKey + "selS");
    if (selS) {
        if (selS.length > 0) {
            var stKey = isFixed ? "__selectOne" : "__selectMulti";
            selS.options[0].text = resolveString(cse_pp_st, stKey);
        }
        selS.multiple = !isFixed;
    }
}

// Adds a new character set mapping
CharsetMapper.prototype.add = function() {
    this.save(null);
}

// Saves an edited mapping
CharsetMapper.prototype.save = function(cs) {
    if (!cs)
        cs = "";
    var selP = document.getElementById(this.uniquifier + cs + "selP");
    var selS = document.getElementById(this.uniquifier + cs + "selS");
    var selCB = document.getElementById(this.uniquifier + cs + "cbF");
    if (selP && selS) {
        var pVal = selP.value;
        var sVals = [];
        var opts = selS.options;
        for (var i = 0; i < opts.length; i++) {
            var option = opts[i];
            if (option.selected) {
                if (option.value && option.value != pVal)
                    sVals.push(option.value);
                if (selCB.checked)
                    break;
            }
        }
        var mapped = sVals.join(",");
        if (pVal && mapped) {
            if (cs)
                delete this.charsetMap[cs];
            this.charsetMap[pVal] = {value:mapped, fixed:selCB.checked};
            this.currCS = null;
            if (this.changeNotifyFn)
                this.changeNotifyFn();
            this.draw(true);
        }
    }
}

// Populates the primary character set selector
CharsetMapper.prototype.populateP = function(selCtl, currSel) {
    var option = null;
    if (0 == selCtl.length) {
        option = document.createElement("option");
        option.value = "";
        option.text = resolveString(cse_pp_st, "__selectOne");
        this.addOption(selCtl, option);
    }
    // Clear any values from the selector
    while (selCtl.length > 1) {
        selCtl.remove(1);
    }

    var opts = this.encodings;
    nOpts = opts.length;
    var found = false;
    for (var i = 0; i < nOpts; i++) {
        var encoding = opts[i];
        // If the character set has already been mapped we cannot
        // map it again; however, if there is a current selection,
        // we need to include it in the option list
        if (null != this.charsetMap[encoding] && encoding != currSel) {
            continue;
        }
        option = document.createElement("option");
        option.value = encoding;
        if (encoding == currSel) {
            option.selected = true;
            found = true;
        }
        option.text = resolveString(cse_pp_st, encoding);
        this.addOption(selCtl, option);
    }
    if (currSel && !found) {
        // Obsolete character set from older version; add it explicitly to the list
        option = document.createElement("option");
        option.value = currSel;
        option.text = currSel;
        option.selected = true;
        this.addOption(selCtl, option);
    }
}

// Populates the secondary character set selector
CharsetMapper.prototype.populateS = function(selCtl, currSel, isFixed) {
    var selectedS = {};
    var mapInfo = this.charsetMap[currSel];
    if (mapInfo) {
        var mList = mapInfo.value.split(",");
        for (var i = 0; i < mList.length; i++) {
            selectedS[mList[i]] = 1;
        }
    }

    var foundS = {};
    for (var s in selectedS) {
        foundS[s] = 0;
    }

    selCtl.multiple = !isFixed;

    // Clear existing options
    while (selCtl.length > 0) {
        selCtl.remove(0);
    }
    var foKey = isFixed ? "__selectOne" : "__selectMulti";
    var option = document.createElement("option");
    option.value = "";
    option.text = resolveString(cse_pp_st, foKey);
    this.addOption(selCtl, option);

    var opts = this.encodings;
    var nOpts = opts.length;
    for (var i = 0; i < nOpts; i++) {
        var encoding = opts[i];
        option = document.createElement("option");
        option.value = encoding;
        if (selectedS[encoding]) {
            option.selected = true;
            foundS[encoding] = 1;
        }
        option.text = resolveString(cse_pp_st, encoding);
        this.addOption(selCtl, option);
    }

    for (var s in foundS) {
        if (!foundS[s]) {
            // Obsolete character set from older version; add it explicitly to the list
            option = document.createElement("option");
            option.value = s;
            option.text = s;
            option.selected = true;
            this.addOption(selCtl, option);
        }
    }
}

// Adds an option to a select control (browser-specific)
CharsetMapper.prototype.addOption = function(sel, option) {
    if (this.browserInfo.isIE) {
        sel.add(option);
    }
    else {
        sel.add(option, null);
    }
}

// Sets selection in IE
CharsetMapper.prototype.setSelect = function(cs) {
    if (null == cs)
        cs = "";
    var mapInfo = this.charsetMap[cs];

    var selP = document.getElementById(this.uniquifier + cs + "selP");
    if (selP) {
        selP.selectedIndex = -1;
        var options = selP.options;
        var nOpts = selP.length;
        for (var i = 0; i < nOpts; i++) {
            if (options[i].value == cs) {
                selP.selectedIndex = i;
                break;
            }
        }
    }
    var selId = this.uniquifier + cs + "selS";
    var selS = document.getElementById(selId);
    if (selS) {
        var selectedS = {};
        if (mapInfo) {
            var mList = mapInfo.value.split(",");
            for (var i = 0; i < mList.length; i++) {
                selectedS[mList[i]] = 1;
            }
        }
        selS.selectedIndex = -1;
        var options = selS.options;
        var nOpts = selS.length;
        for (var i = 0; i < nOpts; i++) {
            if (selectedS[options[i].value]) {
                if (!selS.multiple) {
                    options[i].selected = true;
                    break;
                }
                var fnStr = "document.getElementById('" + selId + "').options[" +
                    i + "].selected = true";
                setTimeout(fnStr, 10);
            }
        }
    }

    var cb = document.getElementById(this.uniquifier + cs + "cbF");
    if (cb && mapInfo) {
        cb.checked = mapInfo.fixed;
    }
}

// Data binding class for the CharsetMapper widget
function CSMDataBinding(csmObj, defFixed, defFallback) {
    this.csmObj = csmObj;
    this.initialValue = null;
    this.defFixed = defFixed ? defFixed : [];
    this.defFallback = defFallback ? defFallback : [];
}

CSMDataBinding.prototype = new DataBinding("", "", "");

CSMDataBinding.prototype.set = function(pp, pbIn) {
    var fixedList = [], fallbackList = [];

    var ppT = new PropertyPath(pp.toString() + ".CharSetMap");
    for(ppT.BeginEnum(); null != pbIn.GetProperty(ppT); ppT.IncrementEnum()) {
        fixedList.push(pbIn.GetProperty(ppT));
    }
    ppT = new PropertyPath(pp.toString() + ".FallbackCharset");
    for(ppT.BeginEnum(); null != pbIn.GetProperty(ppT); ppT.IncrementEnum()) {
        fallbackList.push(pbIn.GetProperty(ppT));
    }

    if (0 == fixedList.length)
        fixedList = this.defFixed;
    if (0 == fallbackList.length)
        fallbackList = this.defFallback;

    this.csmObj.setValue(fixedList, fallbackList);

    // Get the lists in sorted, canonical form to create the initial value
    var list1 = this.csmObj.getFixedMappings();
    var list2 = this.csmObj.getFallbacks();
    this.initialValue = list1.join() + ":" + list2.join();
}

CSMDataBinding.prototype.get = function(pp, pbOut) {
    var changed = false;
    var list1 = this.csmObj.getFixedMappings();
    var list2 = this.csmObj.getFallbacks();
    var testVal = list1.join() + ":" + list2.join();
    if (testVal != this.initialValue)
        changed = true;

    var newPP = new PropertyPath(pp.toString() + ".CharSetMap");
    pbOut.DeleteList(newPP);
    var nVals = list1.length;
    newPP.BeginEnum();
    for (var i = 0; i < nVals; i++, newPP.IncrementEnum()) {
        pbOut.SetProperty(newPP, list1[i]);
    }
    newPP = new PropertyPath(pp.toString() + ".FallbackCharset");
    pbOut.DeleteList(newPP);
    var nVals = list2.length;
    newPP.BeginEnum();
    for (var i = 0; i < nVals; i++, newPP.IncrementEnum()) {
        pbOut.SetProperty(newPP, list2[i]);
    }
    return changed;
}
