/*
 * DataBindings.js
 * Functions to populate form fields from a PropertBag object and read them
 * back.
 *
 * Copyright (C) 2009 McAfee, Inc. All rights reserved.
 */

includeJS("../../js/PropertyBag.js");

/**
 * Base class constructor
 * @param form the name of the form on which a control is located
 * @param element the name of the form element
 * @param def the default value
 */
function DataBinding(form, element, def) {
    this.form = form;
    this.element = element;
    this.initialValue = null;
    this.def = def;
}

/**
 * Sets the field value from a PropertyBag object
 * @param pp the base property path to use
 * @param pbIn the property bag to use
 */
DataBinding.prototype.set = function(pp, pbIn) {
    var f = document.forms[this.form];
    if (f) {
        var e = f.elements[this.element];
        if (e) {
            pp.SetTempLeaf(this.element);
            this.initialValue = pbIn.GetPropertyDef(pp, this.def);
            
            //In case the value is undefined set it to a empty string.
            if( this.initialValue == McUndefined )
				e.value = "";
			else
	            e.value = this.initialValue;
            pp.SetTempLeaf("");
        }
    }
}

/**
 * Gets the field value and places it in a PropertyBag object
 * @param pp the base property path to use
 * @param pbOut the output property bag
 * @return true if the value has changed, false otherwise
 */
DataBinding.prototype.get = function(pp, pbOut) {
    var ret = false;
    var f = document.forms[this.form];
    if (f) {
        var e = f.elements[this.element];
        if (e) {
            var val = e.value;
            if (val != this.initialValue) {
                ret = true;
            }
            pp.SetTempLeaf(this.element);
            pbOut.SetProperty(pp, val);
            pp.SetTempLeaf("");
        }
    }
    return ret;
}

/**
 * Data binding for calculated fields
 * @param form the name of the form on which a control is located
 * @param element the name of the form element
 * @param def the default value
 * @param olFn a function to transform the data value when loading
 * @param osFn a function to transform the data value back when saving
 */
function CFDataBinding(form, element, def, olFn, osFn) {
    this.form = form;
    this.element = element;
    this.initialValue = null;
    this.def = def;
    this.olFn = olFn;
    this.osFn = osFn;
}

CFDataBinding.prototype = new DataBinding("", "", "");

/**
 * Sets the field value from a PropertyBag object
 * @param pp the base property path to use
 * @param pbIn the property bag to use
 */
CFDataBinding.prototype.set = function(pp, pbIn) {
    var f = document.forms[this.form];
    if (f) {
        var e = f.elements[this.element];
        if (e) {
            pp.SetTempLeaf(this.element);
            this.initialValue = pbIn.GetPropertyDef(pp, this.def);
            e.value = this.olFn ? this.olFn(this.initialValue) : this.initialValue;
            pp.SetTempLeaf("");
        }
    }
}

/**
 * Gets the field value and places it in a PropertyBag object
 * @param pp the base property path to use
 * @param pbOut the output property bag
 * @return true if the value has changed, false otherwise
 */
CFDataBinding.prototype.get = function(pp, pbOut) {
    var ret = false;
    var f = document.forms[this.form];
    if (f) {
        var e = f.elements[this.element];
        if (e) {
			var val;
			if(e.name == "DisclaimerText" && e.value.length <= 230 && gEPOMode)
			{
				val = e.value;
				//val = s.replace(/\r\n/g,"~e")
			}
			else
			{
				val = this.osFn ? this.osFn(e.value) : e.value;
			}
            if (val != this.initialValue) {
                ret = true;
            }
            pp.SetTempLeaf(this.element);
            pbOut.SetProperty(pp, val);
            pp.SetTempLeaf("");
        }
    }
    return ret;
}

/**
 * Data binding for radio button controls
 * @param form the name of the form on which a control is located
 * @param element the name of the form element
 * @param def the def value
 */
function RadioDataBinding(form, element, def) {
    this.form = form;
    this.element = element;
    this.initialValue = null;
    this.def = def;
}

RadioDataBinding.prototype = new DataBinding("", "", "");

/**
 * Sets the field value from a properties object
 * @param pp the base property path to use
 * @param pbIn the property bag to use
 */
RadioDataBinding.prototype.set = function(pp, pbIn) {
    var f = document.forms[this.form];
    if (f) {
        var elems = f.elements;
        pp.SetTempLeaf(this.element);
        var val = pbIn.GetPropertyDef(pp, this.def);
        pp.SetTempLeaf("");
        for (var i = 0; i < elems.length; i++) {
            var e = elems[i];
            if (e.type == "radio" && e.name == this.element &&
            e.value == val) {
            e.checked = true;
            break;
            }
        }
        this.initialValue = val;
    }
}

/**
 * Reads the field value into a PropertyBag
 * @param pp the base property path to use
 * @param pbOut the output property bag
 * @return true if the value has changed, false otherwise
 */
RadioDataBinding.prototype.get = function(pp, pbOut) {
    var ret = false;
    var f = document.forms[this.form];
    if (f) {
        var elems = f.elements;
        for (var i = 0; i < elems.length; i++) {
            var e = elems[i];
            if (e.type == "radio" && e.name == this.element && e.checked) {
                pp.SetTempLeaf(this.element);
                var val = e.value;
                if (val != this.initialValue) {
                    ret = true;
                }
                pbOut.SetProperty(pp, val);
                pp.SetTempLeaf("");
                break;
            }
        }
    }
    return ret;
}

/**
 * Data binding for unique checkbox controls (i.e. not grouped)
 * @param form the name of the form on which a control is located
 * @param element the name of the form element
 * @param def the default value
 * @param tVal true value; defaults to "1"
 * @param fVal false value; defaults to "0"
 */
function CheckboxDataBinding(form, element, def, tVal, fVal) {
    this.form = form;
    this.element = element;
    this.initialValue = null;
    this.def = def;
    this.tVal = (null == tVal) ? "1" : tVal;
    this.fVal = (null == fVal) ? "0" : fVal;
}

CheckboxDataBinding.prototype = new DataBinding("", "", "");

/**
 * Checks a box according to whether a corresponding value in a
 * properties object has a "true" value.
 * @param pp the base property path to use
 * @param pbIn the property bag to use
 */
CheckboxDataBinding.prototype.set = function(pp, pbIn) {
    var f = document.forms[this.form];
    if (f) {
        var e = f.elements[this.element];
        if (e) {
            var checked = false;
            pp.SetTempLeaf(this.element);
           
			if (this.tVal == pbIn.GetPropertyDef(pp, this.def)) 
			{
                checked = true;
            }
			//This case brings more than 2 value which are "-1","0","1","2","3","4"
			if(e.id == "chkbox_ArtemisSensitivityLevel")
			{
				if(pbIn.GetPropertyDef(pp, this.def) != "-1")
					checked = true;
            }
			pp.SetTempLeaf("");
            e.checked = checked;
            this.initialValue = checked ? this.tVal : this.fVal;
        }
    }
}

/**
 * Sets a value in a PropertyBag to 1 or 0, depending
 * on whether the corresponding box is checked.
 * @param pp the base property path to use
 * @param pbOut the output property bag
 * @return true if the value has changed, false otherwise
 */
CheckboxDataBinding.prototype.get = function(pp, pbOut) {
    var ret = false;
    var f = document.forms[this.form];
    if (f) {
        var e = f.elements[this.element];
        if (e) 
		{
			//artemis check
			var tmpVal = ""
			if(e.id	== "chkbox_ArtemisSensitivityLevel") {
				if(e.checked == false)
					tmpVal = "-1"

				val = tmpVal;
				if (val != this.initialValue) {
					ret = true;
				}
			}
			else
			{
				var val = e.checked ? this.tVal : this.fVal;
				if (val != this.initialValue) {
					ret = true;
				}
			}
            pp.SetTempLeaf(this.element);
            pbOut.SetProperty(pp, val);
            pp.SetTempLeaf("");
        }
    }
    return ret;
}

/**
 * Data binding for a select control
 * @param form the name of the form on which a control is located
 * @param element the name of the form element
 * @param def the default value
 * @param multi true if the control is multivalued, false otherwise
 * @param singleValue true if the control receives and returns a single
 * value even though multiple selection is posible
 * @param separator the spearator between values; defaults to ","
 */
function SelectDataBinding(form, element, def, multi, singleValue, separator) {
    this.form = form;
    this.element = element;
    this.def = def;
    this.multi = (null != multi) ? multi : false;
    this.singleValue = (null != singleValue) ? singleValue : false;
    this.separator = separator ? separator : ",";
}

SelectDataBinding.prototype = new DataBinding("", "", "");

/**
 * Sets selection(s) in a select control according to a property value
 * @param pp the base property path to use
 * @param pbIn the property bag to use
 */
SelectDataBinding.prototype.set = function(pp, pbIn) {
    var f = document.forms[this.form];
    if (!f) return;
    var e = f.elements[this.element];
    if (!e) return;
    if (!e.options) return;

    pp.SetTempLeaf(this.element);
	
	//Settings ArtemisSensitivityLevel if CMBSENSLEVEL is enabled
	if(pp.tempLeaf == "CMBSENSLEVEL")
		pp.tempLeaf="ArtemisSensitivityLevel"

    var val = pbIn.GetPropertyDef(pp, this.def);
    pp.SetTempLeaf("");
    if (this.multi) {
        if (this.singleValue) {
            this.setMultiSV(e, val);
        }
        else {
            this.setMulti(e, pp, pbIn);
        }
    }
    else {
        var options = e.options;
        for (var i = 0; i < options.length; i++) {
            if (options[i].value == val) {
                options[i].selected = true;
                break;
            }
        }
        this.initialValue = val;
    }
}

/**
 * Sets selections in a multi-select control; the input is a delimited string
 * @param e the select element
 * @param val the input value
 */
SelectDataBinding.prototype.setMultiSV = function(e, val) {
    var vals = val.split(this.separator);
    this.setMulti2(e, vals);
}

/**
 * Sets selection in a multi-select control; the input is an array of values
 * @param e the select element
 * @param vals the input values (modified on output)
 */
SelectDataBinding.prototype.setMulti2 = function(e, vals) {
    this.initialValue = vals.join(",");
    var options = e.options;
    for (var i = 0; i < options.length; i++) {
        for (var j = 0; j < vals.length; j++) {
            if (options[i].value == vals[j]) {
                options[i].selected = true;
                vals.splice(j, 1);
                break;
            }
        }
        if (0 == vals.length) {
            break;
        }
    }
}

/**
 * Sets selections in a multi-select control
 * @param e the select element
 * @param pp the base property path to use
 * @param pbIn the property bag to use
 */
SelectDataBinding.prototype.setMulti = function(e, pp, pbIn) {
    pp.SetTempLeaf(this.element);
    var newPP = new PropertyPath(pp.GetPath());
    pp.SetTempLeaf("");
    var vals = new Array();

    newPP.BeginEnum();
    while(true) {
        var val = pbIn.GetProperty(newPP);
        if (undefined === val) {
            break;
        }
        vals.push(val);
        newPP.IncrementEnum();
    }

    this.setMulti2(e, vals);
}

/**
 * Gets selection(s) from a select control and places them in a PropertyBag
 * @param pp the base property path to use
 * @param pbOut the output property bag to use
 * @return true if the value has changed, false otherwise
 */
SelectDataBinding.prototype.get = function(pp, pbOut) {
    var ret = false;
    var f = document.forms[this.form];
    if (!f) return false;
    var e = f.elements[this.element];
    if (!e) return false;
    if (!e.options) return false;
	
	//If Artemis is checked
	if((f.elements["ArtemisSensitivityLevel"]) && (f.elements["ArtemisSensitivityLevel"].checked == true))
	{
		if (this.multi) {
			ret = getMulti(e, pp, pbOut);
		}
		else {
			var options = e.options;
			pp.SetTempLeaf(this.element);
			for (var i = 0; i < options.length; i++) {
				if (options[i].selected) {
					var newVal = options[i].value;
					if (newVal != this.initialValue) {
						ret = true;
					}
					if(pp.tempLeaf == "CMBSENSLEVEL")
						pp.tempLeaf="ArtemisSensitivityLevel"

					pbOut.SetProperty(pp, newVal);
					break;
				}
			}
			pp.SetTempLeaf("");
		}
	}
	else if(this.element != "CMBSENSLEVEL")
	{
		if (this.multi) {
			ret = getMulti(e, pp, pbOut);
		}
		else {
			var options = e.options;
			pp.SetTempLeaf(this.element);
			for (var i = 0; i < options.length; i++) {
				if (options[i].selected) {
					var newVal = options[i].value;
					if (newVal != this.initialValue) {
						ret = true;
					}

					pbOut.SetProperty(pp, newVal);
					break;
				}
			}
			pp.SetTempLeaf("");
		}
	}
    return ret;
}

/**
 * Gets selections from a multi-select control
 * @param e the select element
 * @param pp the base property path to use
 * @param pbOut the output property bag to use
 * @return true if the value has changed, false otherwise
 */
SelectDataBinding.getMulti = function(e, pp, pbOut) {
    var ret = false;
    var val = new Array();
    var options = e.options;
    for (var i = 0; i < options.length; i++) {
        if (options[i].selected) {
            val.push(options[i].value);
        }
    }
    pp.SetTempLeaf(this.element);
    if (this.singleValue) {
        pbOut.SetProperty(pp, val.join(this.separator));
    }
    else {
        var newPP = new PropertyPath(pp.GetPath());
        pbOut.DeleteList(newPP);
        var nVals = val.length;
        newPP.BeginEnum();
        for (var i = 0; i < nVals; i++, newPP.IncrementEnum()) {
            pbOut.SetProperty(newPP, val[i]);
        }
    }
    pp.SetTempLeaf("");
    // Test to see whether the value has changed
    var tVal = val.join(",");
    if (tVal != this.initialValue) {
        ret = false;
    }
    return ret;
}

/**
 * Data binding for InputList widgets
 * @param listObject the list object to bind to
 * @param propName the property name associated with the InputList object
 * @param singleValue true if the input and output is a single value string,
 * false if it is an array
 * @param separator the separator to use between string elements for a single-
 * valued list object
 * @param def the default value of the associated list object
 */
function InputListDataBinding(listObject, propName, singleValue, separator, def) {
    this.listObject = listObject;
    this.propName = propName;
    this.singleValue = singleValue ? true : false;
    this.separator = separator ? separator : ",";
    this.initialValue = null;
    this.def = (null != def) ? def : (singleValue ? "" : []);
}

InputListDataBinding.prototype = new DataBinding("", "", "");

/**
 * Sets the value of an input list object
 * @param pp the base property path to use
 * @param pbIn the property bag to use
 */
InputListDataBinding.prototype.set = function(pp, pbIn) {
    var listVal = [];
    pp.SetTempLeaf(this.propName);
    if (this.singleValue) {
        var v = pbIn.GetPropertyDef(pp, this.def);
        if (v) {
            listVal = v.toString().split(this.separator);
        }
    }
    else {
        var newPP = new PropertyPath(pp.GetPath());
        newPP.BeginEnum();
        while(true) {
            var v = pbIn.GetProperty(newPP);
            if (v === undefined) {
                break;
            }
            listVal.push(v);
            newPP.IncrementEnum();
        }
    }
    if (0 == listVal.length)
        listVal = this.singleValue ? (this.def ? [this.def] : []) : this.def;
    pp.SetTempLeaf("");
    this.listObject.setValue(listVal);
    this.initialValue = listVal.join(",");
}

/**
 * Gets the value from an input list and sets it in a properties object
 * @param pp the base property path to use
 * @param pbOut the output property bag to use
 * @return true if the value has changed, false otherwise
 */
InputListDataBinding.prototype.get = function(pp, pbOut) {
    var ret = false;
    var v = this.listObject.getValue();
    // v is in array format
    pp.SetTempLeaf(this.propName);
    if (this.singleValue) {
        pbOut.SetProperty(pp, v.join(this.separator));
    }
    else {
        var newPP = new PropertyPath(pp.GetPath());
        pbOut.DeleteList(newPP);
        var nVals = v.length;
        newPP.BeginEnum();
        for (var i = 0; i < nVals; i++, newPP.IncrementEnum()) {
            pbOut.SetProperty(newPP, v[i]);
        }
    }
    pp.SetTempLeaf("");
    var tVal = v.join(",");
    if (tVal != this.initialValue) {
        ret = true;
    }
    return ret;
}

/**
 * Alternative data binding for InputList widgets
 * @param listObject the list object to bind to
 * @param basePropName the property name associated with the InputList object
 * (the actual names will be <basePropName>0, <basePropName>1, etc.
 * @param startIdx - the index to start at; defaults to 0
 */
function InputListDataBinding2(listObject, basePropName, startIdx) {
    this.listObject = listObject;
    this.basePropName = basePropName;
    this.startIdx = (null == startIdx) ? 0 : 1;
}

InputListDataBinding2.prototype = new DataBinding("", "", "");

/**
 * Sets the value of an input list object
 * @param pp the base property path to use
 * @param pbIn the property bag to use
 */
InputListDataBinding2.prototype.set = function(pp, pbIn) {
    var listVal = [];
    var idx = this.startIdx;
    while(true) {
        pp.SetTempLeaf(this.basePropName + idx);
        var v = pbIn.GetProperty(pp);
        if (v !== undefined) {
            listVal.push(v);
            idx++;
        }
        else {
            break;
        }
    }
    pp.SetTempLeaf("");
    this.listObject.setValue(listVal);
    this.initialValue = listVal.join(",");
}

/**
 * Gets the value from an input list and sets it in a PropertyBag
 * @param pp the base property path to use
 * @param pbOut the output property bag to use
 * @return true if the value has changed, false otherwise
 */
InputListDataBinding2.prototype.get = function(pp, pbOut) {
    var ret = false;
    var v = this.listObject.getValue();
    // Remove any existing properties from the input
    var idx = this.startIdx;
    while(true) {
        pp.SetTempLeaf(this.basePropName + idx);
        if (pbOut.GetProperty(pp) !== undefined) {
            pbOut.RemoveLeaf(pp);
            idx++;
        }
        else {
            break;
        }
    }
    for (var i = 0, idx = this.startIdx; i < v.length; i++, idx++) {
        pp.SetTempLeaf(this.basePropName + idx);
        pbOut.SetProperty(pp, v[i]);
    }
    pp.SetTempLeaf("");
    var tVal = v.join(",");
    if (tVal != this.initialValue) {
        ret = false;
    }
    return ret;
}


/**
 * Data binding for actions lists
 * @param formNameBase - the base name for action forms
 * @param propName - the property name containing the action list in string
 * form
 * @param def a default action list to use
 */
function ActionListDataBinding(formNameBase, propName, def) {
    this.spForm = formNameBase + "_actions_s_p";
    this.ssForm = formNameBase + "_actions_s_s";
    this.fpForm = formNameBase + "_actions_f_p";
    this.fsForm = formNameBase + "_actions_f_s";
    this.propName = propName;
    this.def = def;
    this.initialValue = null;
}

ActionListDataBinding.prototype = new DataBinding("", "", "");

/**
 * Checks and unchecks action list controls according to the value in a
 * properties object.
 * @param pp the base property path to use
 * @param pbIn the property bag to use
 */
ActionListDataBinding.prototype.set = function(pp, pbIn) {
    pp.SetTempLeaf(this.propName);
    var actionList = pbIn.GetPropertyDef(pp, this.def);
    pp.SetTempLeaf("");
    if (!actionList) {
        this.setPrimary(this.fpForm, "");
        return;
    }
	var tempstr = actionList.split("{");
	
	//tempstr.split("{");

	actionList = tempstr[0];
	
	var score = "";

	if(tempstr.length > 1)
	{
		score = tempstr[1].slice(0,tempstr[1].length-1);
	}
	
	
	
	var currentscore = score.split(' ');

	//alert(currentscore[0].length)


	if(currentscore[0].length > 0)
	{
	   
		var div = document.getElementById("_Tscore");
		if (div)
		{   
			div.value = currentscore[0];
			
		}
		div = document.getElementById("_mcount");
		if (div)
		{   
			div.value = currentscore[1];
			
		}
	}
	else
	{
		
	
		var div = document.getElementById("_Tscore");
		if (div)
		{   
			
			div.value = "";
			
		}	
		div = document.getElementById("_mcount");
		if (div)
		{   
			div.value ="";
			
		}


		 div = document.getElementById("tscr");
		if (div)
		{   
			div.style.display = "none";
			
		}
		div = document.getElementById("mcnt");
		if (div)
		{   
			div.style.display = "none";
			
		}
	}

    this.initialValue = actionList;

    // The action list has the following format:
    // action 1,action 2,(action 1,action 2,action 3)
    // where the actions in parentheses are "fail" actions;
    // the first action in each sublist is a "primary" action, while the
    // others are secondary
    var sublists = actionList.split("(");
    if (sublists.length < 2) {
        sublists[1] = "";
    }

    // Remove any trailing parenthesis from the second sublist
    var failActions = sublists[1];
    if (failActions.charAt(failActions.length - 1) == ')') {
        failActions = failActions.substring(0, failActions.length - 1);
    }

    var fList = failActions.split(",");

    // The first list may end in a trailing comma which we may remove
    var successActions = sublists[0];
    if (successActions.charAt(successActions.length - 1) == ',') {
        successActions = successActions.substring(0, successActions.length - 1);
    }
    var sList = successActions.split(",");
    this.setPrimary(this.spForm, sList[0]);
    this.setSecondary(this.ssForm, sList);
    this.setPrimary(this.fpForm, fList[0]);
    this.setSecondary(this.fsForm, fList);
}

/**
 * Sets a primary action
 * @param formName the name of the form
 * @param actionName the name of the action
 */
/*ActionListDataBinding.prototype.setPrimary = function(formName, actionName) {
    var verbose = false;

    var f = document.forms[formName];
    if (!f)
        return;
    var elems = f.elements;
    var defElem = null;
    var nElems = elems.length;
    if (nElems && elems[0].type == "radio") {
        defElem = elems[0];
        for (var i = 0; i < nElems; i++) {
            var e = elems[i];
            if (e.name == "a2" && e.value == this.def) {
                defElem = e;
            }
        }
    }
    var notFound = true;
    for (var i = 0; i < nElems; i++) {
        var e = elems[i];
        if (e.name == "a2" && e.value == actionName) {
            e.checked = true;
            notFound = false;
            break;
        }
    }

    if (notFound && defElem) {
        defElem.checked = true;
    }

	if(!notFound)
    {  if (f.init)
	   {  f.init.value=actionName;
	   }
    }

}*/
ActionListDataBinding.prototype.setPrimary = function(formName, actionName) {
    var verbose = false;

    var f = document.forms[formName];
    if (!f)
        return;
    var sel = f.elements["pri_actions"];
    if (!sel)
        return;
    if (sel.options !== undefined) {
        var options = sel.options;
        var nOpts = options.length;
        var selIdx = 0;
        var notFound = true;
        for (var i = 0; i < nOpts; i++) {
            if (options[i].value == actionName) {
                selIdx = i;
                notFound = false;
                break;
            }
        }
        sel.selectedIndex = selIdx;
        if (notFound && f.init) {
            f.init.value = actionName;
        }
    }
    else if (undefined !== sel.checked) {
        sel.checked = (actionName == sel.value);
    }
}



/**
 * Sets secondary actions
 * @param formName the name of the form
 * @param actionList the names of selected actions
 */
ActionListDataBinding.prototype.setSecondary = function(formName, actionList) {


	var f = document.forms[formName];
    if (!f)
        return;
    var elems = f.elements;
	var bQtn = false;

	var id = "frd" +formName 
			
	var div = document.getElementById(id);

    if(div)
	div.disabled = true
    for (var i = 0; i < elems.length; i++) {
        var e = elems[i];
        if (e.name == "a2") {
            e.checked = false;
        }
        // The first element in the action list is a primary action, so ignore it
        for (var j = 1; j < actionList.length; j++) {
            if (e.name == "a2" && e.value == actionList[j]) {
				
			
				if(actionList[j] == "QUARANTINE")
				{
					if(div)
					div.disabled = false
					 
				}
				
				  e.checked = true;
                break;
            }
        }
    }
}
/*ActionListDataBinding.prototype.setSecondary = function(formName, actionList) {
    if (actionList.length < 2)
        return;
    var f = document.forms[formName];
    if (!f)
        return;
    var sel = f.elements["sec_actions"];
    if (!sel)
        return;
    var options = sel.options;
    // Clear any current selections
    sel.selectedIndex = -1;
    for (var i = 0; i < options.length; i++) {
        var e = options[i];
        // The first element in the action list is a primary action, so ignore it
        for (var j = 1; j < actionList.length; j++) {
            var sel = false;
            if (e.value == actionList[j]) {
                e.selected = true;
            }
            /*var fnStr = "document.forms['" + formName + "'].elements['sec_actions']" +
                ".options[" + i + "].selected=" + sel + ";";
            alert(e.value + ": " + fnStr);
            setTimeout(fnStr, 10);*/
/*        }
    }
}*/

/**
 * Build up an action list and places it in a PropertyBag
 * @param pp the base property path to use
 * @param pbOut the output property bag to use
 * @return true if the actions have changed, false otherwise
 */
ActionListDataBinding.prototype.get = function(pp, pbOut) {
    var ret = false;
    var aList = "";
    var sp = this.getPrimary(this.spForm);
    var ss = this.getSecondary(this.ssForm);
    var fp = this.getPrimary(this.fpForm);
    var fs = this.getSecondary(this.fsForm);

    var p1 = "";
    if (sp) {
        p1 = sp + ",";
    }
    if (ss && ss.length) {
        p1 += ss.join(",") + ",";
    }
    var p2 = "(";
    if (fp) {
        p2 += fp;
    }
    if (fs && fs.length) {
        p2 += "," + fs.join(",");
    }
    p2 += ")";

	var tbl_div=document.getElementById("_Tscore");
	
	if(tbl_div)
	{
		p2 += "{"
		if(tbl_div.value != "")
			p2 +=  tbl_div.value;
	}

	tbl_div=document.getElementById("_mcount");
	
	if(tbl_div)
	{
		p2 += " "
		if(tbl_div.value != "")
			p2 += tbl_div.value ;
		p2 += "}"
	}


    aList = p1 + p2;
    pp.SetTempLeaf(this.propName);
    pbOut.SetProperty(pp, aList);
    pp.SetTempLeaf("");
    if (aList != this.initialValue) {
        ret = true;
    }
    return ret;
}

/**
 * Gets primary action
 * @param formName the name of the form
 * @return the name of the selected action
 */
/*ActionListDataBinding.prototype.getPrimary = function(formName) {
    var ret = "";
    var f = document.forms[formName];
    if (!f)
        return ret;
    var elems = f.elements;
    for (var i = 0; i < elems.length; i++) {
        var e = elems[i];
        if (e.name == "a2" && e.checked) {
            ret = e.value;
            break;
        }
    }
    return ret;
}*/
ActionListDataBinding.prototype.getPrimary = function(formName) {
    var f = document.forms[formName];
    if (!f)
        return "";
    var sel = f.elements["pri_actions"];
    if (!sel)
        return "";
    if (undefined !== sel.options)
        return sel.value;
    return sel.checked ? sel.value : "";
}

/**
 * Gets secondary actions
 * @param formName the name of the form
 * @return the names of selected actions
 */
ActionListDataBinding.prototype.getSecondary = function(formName) {
 
	var ret = new Array();
    var f = document.forms[formName];
    if (!f)
        return ret;
    var elems = f.elements;
    for (var i = 0; i < elems.length; i++) {
        var e = elems[i];
        if (e.name == "a2" && e.checked) {
            ret.push(e.value);
        }
    }
    return ret;
}
/*ActionListDataBinding.prototype.getSecondary = function(formName) {
    var ret = new Array();
    var f = document.forms[formName];
    if (!f)
        return ret;
    var sel = f.elements["sec_actions"];
    if (!sel)
        return;
    var options = sel.options;
    for (var i = 0; i < options.length; i++) {
        if (options[i].selected) {
            ret.push(options[i].value);
        }
    }
    return ret;
}*/


/**
 * Used when the contents of a text control are a dereferenced
 * StringTable entry; if the text is unchanged, the value must
 * not be stored
 * @param form the name of the form on which a control is located
 * @param element the name of the form element
 * @param st the StringTable to use for value lookup
 */
function STKeyDataBinding(form, element, st) {
    this.form = form;
    this.element = element;
    this.st = st;
    this.originalKey = this.initialValue = null;
}

STKeyDataBinding.prototype = new DataBinding("", "");

/**
 * Sets the value from a PropertyBag object; may involve a string table lookup
 * @param pp the base property path to use
 * @param pbIn the property bag to use
 */
STKeyDataBinding.prototype.set = function(pp, pbIn) {
    var f = document.forms[this.form];
    var e = f.elements[this.element];
    pp.SetTempLeaf(this.element);
    var toDisplay = pbIn.GetPropertyDef(pp, "");
    pp.SetTempLeaf("");
    // The display names for built-in settings are stringtable
    // entries
    this.originalKey = toDisplay;
    if (this.st[toDisplay]) {
        toDisplay = this.st[toDisplay];
    }
    e.value = toDisplay;
    this.initialValue = toDisplay;
}

/**
 * Gets the value from a control and sets it in a PropertyBag; however, if
 * the field is unchanged the value set is the original stringtable key rather
 * than the actual value
 * @param pp the base property path to use
 * @param pbOut the output property bag to use
 * @return true if the value has changed, false otherwise
 */
STKeyDataBinding.prototype.get = function(pp, pbOut) {
    var ret = false;
    var f = document.forms[this.form];
    var t = f.elements[this.element];
    var testVal = t.value;
    pp.SetTempLeaf(this.element);
    if (testVal == this.initialValue) {
        pbOut.SetProperty(pp, this.originalKey);
    }
    else {
        pbOut.SetProperty(pp, testVal);
        ret = true;
    }
    pp.SetTempLeaf("");
    return ret;
}

/**
 * Used when the contents of the text control take a default value from a resource
 * table; if the text is unchanged the value must be set to an empty string
 * @param form the name of the form on which a control is located
 * @param element the name of the form element
 * @param rm a ResourceManager object
 * @param section the resource section for lookup
 * @param key the resource key for lookup
 */
function ResourceDataBinding(form, element, rm, section, key) {
    this.form = form;
    this.element = element;
    this.rm = rm;
    this.section = section;
    this.key = key;
}

ResourceDataBinding.prototype = new DataBinding("", "");

/**
 * Sets the value from a PropertyBag object
 * @param pp the base property path to use
 * @param pbIn the property bag to use
 */
ResourceDataBinding.prototype.set = function(pp, pbIn) {
    // Dereference the resource string
    this.resString = this.rm.getResource(this.section, this.key);
    var f = document.forms[this.form];
    var e = f.elements[this.element];
    pp.SetTempLeaf(this.element);
    var toDisplay = pbIn.GetPropertyDef(pp, "");
    pp.SetTempLeaf("");
    if ("" == toDisplay) {
        // Show the resource string instead of the empty string
        toDisplay = this.resString
    }
    e.value = toDisplay;
    this.initialValue = toDisplay;
}

/**
 * Gets the value from a control and sets it in a PropertyBag; however, if
 * the field is the same as the corresponding resource value, an mepty string is
 * stored
 * @param pp the base property path to use
 * @param pbOut the output property bag to use
 * @return true if the value has changed, false otherwise
 */
ResourceDataBinding.prototype.get = function(pp, pbOut) {
    var f = document.forms[this.form];
    var t = f.elements[this.element];
    var testVal = t.value;
    pp.SetTempLeaf(this.element);
    if (testVal == this.resString) {
        // Store an empty string
        pbOut.SetProperty(pp, "");
    }
    else {
        pbOut.SetProperty(pp, testVal);
    }
    pp.SetTempLeaf("");
    return (testVal != this.initialValue);
}


/**
 * Data binding for properties which take on a default value when empty
 * @param form the name of the form on which a control is located
 * @param element the name of the form element
 * @param def the default value
 */
function DefaultValueDataBinding(form, element, def) {
    this.form = form;
    this.element = element;
    this.initialValue = null;
    this.def = def;
}

DefaultValueDataBinding.prototype = new DataBinding("", "");

/**
 * Sets the field value from a PropertyBag object; shows a default value
 * if the property is an empty string
 * @param pp the base property path to use
 * @param pbIn the property bag to use
 */
DefaultValueDataBinding.prototype.set = function(pp, pbIn) {
    var f = document.forms[this.form];
    if (f) {
        var e = f.elements[this.element];
        if (e) {
            pp.SetTempLeaf(this.element);
            this.initialValue = pbIn.GetProperty(pp);
            if (!this.initialValue)
                this.initialValue = this.def;
            e.value = this.initialValue;
            pp.SetTempLeaf("");
        }
    }
}

/**
 * Gets the field value and places it in a PropertyBag object; does not store
 * the value if it matches a default
 * @param pp the base property path to use
 * @param pbOut the output property bag
 * @return true if the value has changed, false otherwise
 */
DefaultValueDataBinding.prototype.get = function(pp, pbOut) {
    var ret = false;
    var f = document.forms[this.form];
    if (f) {
        var e = f.elements[this.element];
        if (e) {
            var val = e.value;
            if (val != this.initialValue)
                ret = true;
            /*if (val == this.def)
                val = "";*/
            pp.SetTempLeaf(this.element);
            pbOut.SetProperty(pp, val);
            pp.SetTempLeaf("");
        }
    }
    return ret;
}
