/*
 * FileCatSelector.js
 * JavaScript control for setting file categories and sub-categories for
 * file and content filtering rules.
 *
 * Has a number of JS dependencies:
 * Common.js
 * DataBindings.js
 * <LOCALE>/Strings.js
 *
 * Copyright (C) 2016 McAfee, Inc. All rights reserved.
 */

includeJS("../js/Strings.js");
// Metatag for the everything category
var FC_ALL = "ALL";
var subcatsChildMap={};
// Non-breaking space
var NBSP = 160;
/**
 * The constructor takes a properties Object which has the following fields:
 * formName - the name of the enclosing form; defaults to null;
 * width: width, in pixels, of the first select control; defaults to -1 (meaning that no
 * width style will be applied to the select control)
 * width2: width, in pixels, of the second select control; defaults to -1 (meaning that no
 * width style will be applied to the select control)
 * height: the height, in rows, of the select controls; defaults to 6
 * SelectAllCheck: whether or not the control has a checkbox to select "everything"
 * (not the same as selecting all subcategories in all categories); defaults to false
 * ExclusionList: a list of top-level category metatags that are excluded from the category
 * selector; setting "SelectAllCheck" to true will cause the the metatag "ALL" to be added
 * to this exclusion list.
 */

/**
 * Constructor for the FileCatSelector object
 * @param fileCats: a list of file categories and subcategories; an appropriate
 * list for both content scanning and file filtering may be found in FileCategories.js.
 * @param divId - the ID of a div that will be used to contain the control.
 * @param objectName - the name by which the object is declared; defaults to
 * "fcSelect"
 * @param properties - a properties object for the widget; see above for the
 * members.
 * @param type "F" for file-filtering, "C" for content-scanning; defaults to "C"
 */
function FileCatSelector(fileCats, divId, objectName, properties, type) {
    this.divId = divId;
    this.objectName = objectName ? objectName : "fcSelect";
    this.enabled = true;
    this.fileCats = fileCats;
    this.browserInfo = IdentifyBrowser();
    this.currentCat = "";
    this.currentValue = {};
    this.shownOnce = {};
    this.type = (null == type) ? "C" : type;
    fcs_init(properties, this);
}

function fcs_init(propsIn, propsOut) {
    if (null == propsIn) {
        propsIn = {};
    }
    propsOut.formName = propsIn["formName"] ?
        propsIn["formName"] : null;
    propsOut.width = (null != propsIn["width"]) ?
        propsIn["width"] : -1;
    propsOut.width2 = (null != propsIn["width2"]) ?
        propsIn["width2"] : -1;
    propsOut.height = (null != propsIn["height"]) ?
        propsIn["height"] : 6;
    propsOut.SelectAllCheck = (null != propsIn["SelectAllCheck"]) ?
        propsIn["SelectAllCheck"] : false;
    propsOut.ExclusionList = pp_cse_fcExclusionLists[propsOut.type] ?
        pp_cse_fcExclusionLists[propsOut.type] : [];
    if (propsOut.SelectAllCheck) {
        var allExcluded = false;
        for (var i = propsOut.ExclusionList.length - 1; i >= 0; i--) {
            if (FC_ALL == propsOut.ExclusionList[i]) {
                allExcluded = true;
                break;
            }
        }
        if (!allExcluded) {
            propsOut.ExclusionList.push(FC_ALL);
        }
    }
    // Index the exclusion list by name for easier lookup
    if (propsOut.ExclusionList) {
        for (var i = 0; i < propsOut.ExclusionList.length; i++) {
            var x = propsOut.ExclusionList[i];
            propsOut.ExclusionList[x] = 1;
        }
    }
}

/**
 * Sets the value of the control
 * @param value the value; should be a string array
 */
FileCatSelector.prototype.setValue = function(value) {
    this.currentValue = {};
    this.shownOnce = {};
    if ("string" == typeof(value)) {
        this.currentValue[value] = 1;
    }
    else if (value && value.length > 0) {
        var nV = value.length;
        for (var i = 0; i < nV; i++) {
            var v = value[i];
            this.currentValue[v] = 1;
        }
    }
    this.initSelState();
    this.showSelectionHints();
}

/**
 * Shows initial selected hints
 */
FileCatSelector.prototype.showSelectionHints = function() {
    var primSelCtl = document.getElementById(this.divId + "_cats");
    if (!primSelCtl)
        return;
    var opts = primSelCtl.options;
    var nOpts = primSelCtl.length;
    for (var i = 0; i < nOpts; i++) {
        hasSelections = false;
        var option = opts[i];
        var cat = option.value;
        if (this.currentValue[cat]) {
            hasSelections = true;
        }
        else {
            // Go through the options on the subcategory selector to see
            // options that are relevant and then test to see whether
            // they are selected
            var subCSel = document.getElementById(this.divId + "_" + cat);
            if (!subCSel)
                continue;
            var subOpts = subCSel.options;
            var nSubOpts = subCSel.length;
            for (var j = 0; j < nSubOpts; j++) {
                var subCat = subOpts[j].value;
                if (this.currentValue[subCat]) {
                    hasSelections = true;
                    break;
                }
            }
        }
        var leadingAsterisk = ("*" == option.text.charAt(0));
        if (hasSelections) {
            // Show selection hint (an asterisk)
            if (!leadingAsterisk)
                option.text = "*" + option.text;
        }
        else {
            if (leadingAsterisk)
                option.text = option.text.substring(1, option.text.length);
        }
    }
}

/**
 * Gets the value of the control
 * @return an array of values
 */
FileCatSelector.prototype.getValue = function() {
    var ret = [];
    var dCatMap = {};
    for (var prop in this.currentValue) {
        ret.push(prop);
        // Look up any previously unknown dependencies of the selected category
        if (pp_cse_fcDependencies && pp_cse_fcDependencies[prop]) {
            var depList = pp_cse_fcDependencies[prop];
            for (var i = 0; i < depList.length; i++) {
                var dCat = depList[i];
                if (this.currentValue[dCat] || dCatMap[dCat])
                    continue;
                ret.push(dCat);
                dCatMap[dCat] = 1;
            }
        }
    }

    return ret;
}

/**
 * Draws the category selector control
 */
FileCatSelector.prototype.draw = function() {
    // Get a handle to the element that will contain the control
    var div = document.getElementById(this.divId);
    if (!div)
        return;

    // Empty the current contents of the div
    while (div.firstChild) {
        div.removeChild(div.firstChild);
    }

    // Create a table to hold the control
    var table = document.createElement("table");
    table.border = 0;
    table.cellPadding = 0;
    table.cellSpacing = 4;
    table.id = this.divId + "_ctl";
    table.width = "80%";

    var rowIdx = 0, cellIdx = 0;
    var row = null, cell = null;
    var cell0PL = "";

    if (this.SelectAllCheck) {
        row = table.insertRow(rowIdx++);
        cell = row.insertCell(0);
        cell.colSpan = 3;
        cell.paddingBottom = "7px";
        var input = document.createElement("input");
        input.type = input.className = "checkbox";
        input.id = input.name = table.id + "_all";
        var fnStr = this.objectName + ".handleSelectAll(this.checked);top.gPageStack.Active().SetModified(true);";
        input.onclick = new Function(fnStr);
        cell.appendChild(input);
        cell.appendChild(document.createTextNode(String.fromCharCode(NBSP)));
        var label = document.createElement("label");
        label.htmlFor = input.id;
        label.appendChild(document.createTextNode(
            resolveString(fcs_st, "__everything")));
        cell.appendChild(label);
        cell0PL = "10px";
    }

    row = table.insertRow(rowIdx++);
    cell = row.insertCell(cellIdx++);
    if (cell0PL)
        cell.style.paddingLeft = cell0PL;
    cell.appendChild(document.createTextNode(resolveString(fcs_st, "__categories")));
    cell = row.insertCell(cellIdx++);
    cell.appendChild(document.createTextNode(resolveString(fcs_st, "__subcats")));

	cell = row.insertCell(cellIdx++);

    cell = row.insertCell(cellIdx++);
    cell.style.textAlign = "right";
    var a = document.createElement("a");
    a.href = "javascript:" + this.objectName + ".clearSubCats();top.gPageStack.Active().SetModified(true);";
    a.className = "blueLink";
    a.id = table.id + "_clr";
    a.appendChild(document.createTextNode(resolveString(fcs_st, "__clear_all")));
    cell.appendChild(a);

    row = table.insertRow(rowIdx++);
    cellIdx = 0;
    cell = row.insertCell(cellIdx++);
    cell.style.width = "40%";
    if (cell0PL)
        cell.style.paddingLeft = cell0PL;

    var cSelect = document.createElement("select");
    var selName = this.divId + "_cats";
    cSelect.name = cSelect.id = selName;
    if (-1 != this.width) {
        cSelect.style.width = this.width + "px";
    }
    cSelect.size = this.height;
    cSelect.className = "mw";
    var fnStr = this.objectName + ".setCategory(this.value);";
    cSelect.onchange = new Function(fnStr);
    cell.appendChild(cSelect);

    cell = row.insertCell(cellIdx++);
    cell.colSpan = 2;
    cell.style.width = "60%";

    var fileCats = this.fileCats;
    for (var i = 0; i < fileCats.length; i++) {
        var fileCat = fileCats[i];
        // Check whether the category's context matches this object's type
        if (!this.isRelevant(fileCat))
            continue;
        if (this.ExclusionList[fileCat.metatag])
            continue;
        var catTag = this.convertMetatag(fileCat.metatag);
        var catName = fileCat.name;
        this.addSelOption(cSelect, catTag, catName, (0 == i));

        var subcats = fileCat.subcats;
		this.createObsoleteCategoryMap(subcats);
        var scSel = this.makeSubCatSelect(catTag, subcats);
        cell.appendChild(scSel);
		
    }
	
	cSelect = document.createElement("select");
	var selName = "ThirdSelect";
    cSelect.name = cSelect.id = selName;
    if (-1 != this.width) {
        cSelect.style.width = this.width + "px";
    }
    cSelect.size = this.height;
    cSelect.className = "mw";
    cell.appendChild(cSelect);

	cell = row.insertCell(cellIdx++);
    cell.colSpan = 1;
    cell.style.width = "60%";
	cell.disabled = "true";
    var fileCats = this.fileCats;
	
    div.appendChild(table);
	 document.getElementById("ThirdSelect").disabled=true;
	 
    if (fileCats) {
        this.currentCat = "";
        for (var i = 0; i < fileCats.length; i++) {
            if (this.isRelevant(fileCats[i])) {
                this.currentCat = this.convertMetatag(fileCats[i].metatag);
                break;
            }
        }
        document.getElementById(this.divId + "_" + this.currentCat).style.display = "";
    }
    this.clearAll();
}

/**
 * Creates a MAP of sub categories and the obsolete childs with metatagrefs pointing to their parent categories
 * @param list off sub categories of the category being selected
 */
FileCatSelector.prototype.createObsoleteCategoryMap = function(subcats){
	var subcatKey;
	for (var k = 0; k < subcats.length; k++)
	{
		var subcat = subcats[k];
		if(subcat.hasOwnProperty("metatagref"))
		{
			var childWithRef = [];
			childWithRef.push(subcat.name);
			subcatKey= this.convertMetatag(subcat.metatagref);
			if(subcatsChildMap[subcatKey]== undefined)
			{
				subcatsChildMap[subcatKey] = childWithRef
			}
			else{
				var lists = subcatsChildMap[subcatKey];
				lists.push(subcat.name);
				subcatsChildMap[subcatKey] = lists;
			}
		}
	}
}
/**
 * Determines whether a given category (or subcategory) should be included in
 * the select controls
 */
FileCatSelector.prototype.isRelevant = function(cat) {
    return (-1 != cat.context.indexOf(this.type));
}

/**
 * Converts a metatag to the appropriate format for content scanning or file
 * filtering as appropriate
 */
FileCatSelector.prototype.convertMetatag = function(metatag) {
    var ret = metatag;
    if ("F" == this.type) {
        // Replace colons in the metatag with forward slashes
        ret = metatag.replace(/:/g, "/");
        // If the category ends in a trailing slash, remove it
        if (ret.lastIndexOf("/") == (ret.length - 1)) {
            ret = ret.substring(0, ret.length - 1);
        }
    }
    return ret;
}

/**
 * Creates a subcategory selector
 * @param cat the file category
 * @param the subcat list
 * @return a partially populated select control
 */
FileCatSelector.prototype.makeSubCatSelect = function(cat, subcats, display) {
    // Check to see which subcategories are relevant
    var relSubcats = [];
    for (var i = 0; i < subcats.length; i++) {
        if (this.ExclusionList[subcats[i].metatag])
            continue;
        if (this.isRelevant(subcats[i])) {
            relSubcats.push(subcats[i]);
        }
    }
    if (0 == relSubcats.length)
        return;
    var selectedCount = 0;
    var scSelect = document.createElement("select");
    selName = this.divId + "_" + cat;
    scSelect.name = scSelect.id = selName;
    if (-1 != this.width2) {
        scSelect.style.width = this.width2 + "px";
    }
    scSelect.className = "mw";
    scSelect.multiple = true;
    scSelect.size = this.height;
	if(typeof display == 'undefined')
		scSelect.style.display = "none";
    var fnStr = this.objectName + ".handleSubcatSelection(); top.gPageStack.Active().SetModified(true);";
    scSelect.onchange = new Function(fnStr);
    var selName = resolveString(fcs_st, "__all");
    // If there is only one item in the subcategory, show the item
    // itself rather than "All"
    if (1 == relSubcats.length) {
        selName = subcats[0].name;
    }
    this.addSelOption(scSelect, cat, selName);
	
	
	
    // If there is only one subcategory, it will already have been
    // put in the list
    if (relSubcats.length > 1) {
        relSubcats.sort(function(a, b) { return a.name.localeCompare(b.name); });
        for (var i = 0; i < relSubcats.length; i++) {
			if(!relSubcats[i].hasOwnProperty('metatagref'))
			{
			 this.addSelOption(scSelect, this.convertMetatag(relSubcats[i].metatag),
			 relSubcats[i].name);
			}
            	
        }
    }


    return scSelect;
}

/**
 * Adds an option to a select control
 * @param the select object to add to
 * @param value the value of the option being added
 * @param label the label of the option
 * @param selected true to cause the option to be intially selected
 */
FileCatSelector.prototype.addSelOption = function(sel, value, label, selected, disabled) {
    var opt = document.createElement("option");
    opt.value = value;
    opt.text = label;
    opt.selected = selected ? true : false;
    if (disabled)
        opt.disabled = true;
    if (this.browserInfo.isIE)
        sel.add(opt);
    else
        sel.add(opt, null);
}

/**
 * Clears third category selection grid
 */
function clearThirdCategorySelection(){
	for(value in subcatsChildMap){
			var lists = subcatsChildMap[value]
			if(lists != undefined){
				for(list in lists)
				{
					var cSelect = document.getElementById("ThirdSelect");
					var option = document.getElementById(lists[list]);
					if(option != null)
						cSelect.removeChild(option)
				}
			}
		}
}

function addObsoleteCategory(lists){
	if(lists != undefined)
	{	
		var cSelect = document.getElementById("ThirdSelect");
		var index = 0;
		for(list in lists)
		{
			var option = document.createElement("option");
			option.id = option.text = lists[list];
			cSelect.add(option, cSelect[index]);
			index++;
		}
	}
	
}

/**
 * Selects a category
 * @param category the category being selected
 */
FileCatSelector.prototype.setCategory = function(category) {
	clearThirdCategorySelection();
    var selId = this.divId + "_" + category;
    var newSCSel = document.getElementById(selId);
    if (!newSCSel)
        return;
    if (this.currentCat) {
        selId = this.divId + "_" + this.currentCat;
        var oldSel = document.getElementById(selId);
        if (oldSel)
            oldSel.style.display = "none";
    }
    newSCSel.style.display = "";
    if (!this.shownOnce[category]) {
        this.initSelState(category);
    }
	else{
	this.handleSubcatSelection(category)
	}
    this.currentCat = category;
}

/**
 * Initializes the selection state for a given subcat select control
 */
FileCatSelector.prototype.initSelState = function(cat) {
    if (null == cat)
        cat = this.currentCat;
    var selId = this.divId + "_" + cat;
    var sel = document.getElementById(selId);
    if (sel) {
        var opts = sel.options;
        var nOpts = opts.length;
        for (var j = 0; j < nOpts; j++) {
            var v = opts[j].value;
            if (this.currentValue[v]) {
                if (!opts[j].selected) {
                    if (this.browserInfo.isIE) {
                        // Create an artificial delay so that the selection state
                        // is correctly shown
                        var fnStr = "document.getElementById('" + selId + "').options[" +
                            j + "].selected = true";
                        setTimeout(fnStr, 10);
                    }
                    else {
                        opts[j].selected = true;
						var lists = subcatsChildMap[opts[j].value]
						addObsoleteCategory(lists);
                    }
                }
            }
            else if (opts[j].selected) {
			
                if (this.browserInfo.isIE) {
                    var fnStr = "document.getElementById('" + selId + "').options[" +
                        j + "].selected = false";
                    setTimeout(fnStr, 10);
				}
                else {
                    opts[j].selected = false;
                }
				
            }
        }
        //this.handleSubcatSelection(cat);
        this.enableSubCatOptions(sel, 1 == this.currentValue[cat]);
        this.shownOnce[cat] = 1;
    }
}

/**
 * Handles the user selecting / unselecting options in a
 * subcategory
 */
FileCatSelector.prototype.handleSubcatSelection = function(cat) {
    if (null == cat)
        cat = this.currentCat;
    var selId = this.divId + "_" + cat;
    var scSel = document.getElementById(selId);
    if (!scSel)
        return;
    var selectedCount = 0;
    var options = scSel.options;
    var disabled = options[0].selected;
    if (disabled) {
        this.currentValue[options[0].value] = 1;
        ++selectedCount;
    }
    else {
        delete this.currentValue[options[0].value];
    }
    var nOpts = options.length;
	clearThirdCategorySelection();
    for (var i = 1; i < nOpts; i++) {
        var v = options[i].value;
        if (disabled && options[i].selected) {
            options[i].selected = false;
        }
        if (options[i].selected) {
            this.currentValue[v] = 1;
            ++selectedCount;
			var lists = subcatsChildMap[options[i].value]
			addObsoleteCategory(lists);
        }
        else {
            this.deleteValue(v);
			
        }
        options[i].disabled = disabled;
    }
	document.getElementById("ThirdSelect").disabled=true;
    if (selectedCount < 2)
        this.showSelectionIndicator(cat, (0 != selectedCount));
}

/**
 * Shows a visual indicator that one or more subcategories are selected from a
 * given category
 */
FileCatSelector.prototype.showSelectionIndicator = function(cat, show) {
    var primSelCtl = document.getElementById(this.divId + "_cats");
    if (!primSelCtl)
        return;
    var opts = primSelCtl.options;
    var currSelIdx = primSelCtl.selectedIndex
    var option = opts[currSelIdx];
    var currOptText = option.text;
    var leadingAsterisk = ('*' == option.text.charAt(0));
    if (show) {
        if (!leadingAsterisk)
            option.text = "*" + option.text;
    }
    else if (leadingAsterisk) {
        // Remove any leading asterisk
        option.text = option.text.substring(1, option.text.length);
    }
    if (this.browserInfo.isGecko && option.text != currOptText) {
        // Changing the option text in Mozilla browsers resets the
        // scrolling of the select control and can cause the currently
        // selected primary category to become obscured; the following
        // is a hack to work round this
        primSelCtl.selectedIndex = -1;
        var fnStr = "document.getElementById('" + this.divId + "_cats')" +
            ".selectedIndex=" + currSelIdx + ";";
        setTimeout(fnStr, 20);
    }
}

/**
 * Deletes a selection and deletes any dependent categories as well
 */
FileCatSelector.prototype.deleteValue = function(v) {
    delete this.currentValue[v];
    if (pp_cse_fcDependencies && pp_cse_fcDependencies[v]) {
        var depList = pp_cse_fcDependencies[v];
        for (var i = depList.length - 1; i >= 0; i--) {
            delete this.currentValue[depList[i]];
        }
    }
}

/**
 * Handles the "all" subcategory being selected
 */
FileCatSelector.prototype.enableSubCatOptions = function(sel, disabled) {
    var options = sel.options;
    var nOpts = options.length;
    for (var i = 1; i < nOpts; i++) {
        var v = options[i].value;
        if (disabled && options[i].selected) {
            options[i].selected = false;
            this.deleteValue(v);
        }
        options[i].disabled = disabled;
    }
}

/**
 * Clears selected subcategories from a given category
 */
FileCatSelector.prototype.clearSubCats = function(cat) {
    if (null == cat)
        cat = this.currentCat;
    var selId = this.divId + "_" + cat;
    var scSel = document.getElementById(selId);
    if (!scSel)
        return;
    var options = scSel.options;
    for (var i = 0; i < options.length; i++) {
        if (options[i].selected) {
            options[i].selected = false;
        }
        this.deleteValue(options[i].value);
    }
    this.handleSubcatSelection();
}

/**
 * Clears all selections from all subcategories
 */
FileCatSelector.prototype.clearAll = function() {
    for (var i = 0; i < fileCats.length; i++) {
        var selId = this.divId + "_" + fileCats[i].metatag;
        var scSel = document.getElementById(selId);
        if (!scSel)
            return;
        scSel.selectedIndex = -1;
    }
    this.currentValue = {};
}

/**
 * Enables or disables the control.
 * @param b true if the control is to be enabled, false otherwise
 * @param internal true if the enabling or disabling is being done by the
 * control's own "select all" checkbox
 */
FileCatSelector.prototype.enable = function(b, internal) {
    var startAt = 0;
    var enabledState = b;
    if (internal) {
        startAt = 1;
    }
    else {
        var cb = document.getElementById(this.divId + "_ctl_all");
        enabledState = cb && cb.checked ? false : b;
    }

    var table = document.getElementById(this.divId + "_ctl");
    if (!table)
        return;
    var rows = table.rows;
    for (var i = startAt; i < rows.length; i++) {
        var cells = rows[i].cells;
        var eState = (i == 0) ? b : enabledState;
        for (var j = 0; j < cells.length; j++) {
            var clName = cells[j].className;
            if (null == clName)
                clName = "";
            clName = eState ? removeClass(clName, "disabled") : addClass(clName, "disabled");
            cells[j].className = clName;
        }
    }
    var link = document.getElementById(this.divId + "_ctl_clr");
    if (link)
        link.style.display = enabledState ? "" : "none";
    // Enable / disable the select controls
    var sels = [this.divId + "_" + this.currentCat, this.divId + "_cats"];
    for (var i = 0; i < sels.length; i++) {
        var sel = document.getElementById(sels[i]);
        if (sel)
            sel.disabled = !enabledState;
    }
}

/**
 * Handles the "Everything" checkbox
 * @param checked true if the box has been checked
 */
FileCatSelector.prototype.handleSelectAll = function(checked) {
    this.enable(!checked, true);
}


/**
 * FileCatSelector data binding class
 * @param fcs the bound FileCatSelector object
 * @param basePropName the property name associated with the selector object
 */
function FCSDataBinding(fcs, basePropName, leaf) {
    this.fcs = fcs;
    this.basePropName = basePropName;
    this.leaf = leaf;
}

FCSDataBinding.prototype = new DataBinding("", "", "");

/**
 * Sets the selection state of the selector control
 * @param pp the base property path to use
 * @param pbIn the property bag to use
 */
FCSDataBinding.prototype.set = function(pp, pbIn) {
    var selValues = [];
    var fcs = this.fcs;
    var ppT = new PropertyPath(pp.toString() + "." + this.basePropName);
    for (ppT.BeginEnum(); null != pbIn.GetProperty(ppT); ppT.IncrementEnum()) {
        if(this.leaf) ppT.SetTempLeaf(this.leaf);
        var v = pbIn.GetPropertyDef(ppT, "");
        if (v) {
            selValues.push(v);
            selValues[v] = 1;
        }
        ppT.SetTempLeaf("");
    }

    var setSelections = true;
    if (fcs.SelectAllCheck) {
        var e = document.getElementById(fcs.divId + "_ctl_all");
        if (e) {
            e.checked = false;
            // Check to see whether the special "Everything" category is selected
            for (var i = 0; i < selValues.length; i++) {
                if (FC_ALL == selValues[i]) {
                    setSelections = false;
                    e.checked = true;
                    fcs.handleSelectAll(true);
                    break;
                }
            }
        }
    }

    if (setSelections) {
        fcs.setValue(selValues);
    }

    this.initialValues = selValues;
}

/**
 * Gets the value from an file category selector and sets it in a properties object
 * @param pp the base property path to use
 * @param pbOut the output property bag to use
 * @return true if the value has changed, false otherwise
 */
FCSDataBinding.prototype.get = function(pp, pbOut) {
    var newValues = [];
    var ret = false;
    var fcs = this.fcs;
    var getSelections = true;
    if (fcs.SelectAllCheck) {
        var e = document.getElementById(fcs.divId + "_ctl_all");
        if (e && e.checked) {
            // Special "EVERYTHING" category selected
            newValues.push(FC_ALL);
            getSelections = false;
        }
    }
    if (getSelections) {
        newValues = fcs.getValue();
        newValues.sort();
    }

    // Check whether the selection have changed
    var nVals = newValues.length;
    var len = this.initialValues.length;
    if (len != nVals) {
        ret = true;
    }
    else {
        this.initialValues.sort();
        for (var i = 0; i < len; i++) {
            if (this.initialValues[i] != newValues[i]) {
                ret = true;
                break;
            }
        }
    }

    if (ret) {
        // Stick the new values in the property bag
        var newPP = new PropertyPath(pp + "." + this.basePropName);
        pbOut.DeleteList(newPP);
        newPP.BeginEnum();
        for (var i = 0; i < nVals; i++, newPP.IncrementEnum()) {
            if(this.leaf)
            {   pbOut.SetProperty(newPP);
                newPP.SetTempLeaf(this.leaf);
            }
            pbOut.SetProperty(newPP, newValues[i]);
            newPP.SetTempLeaf("");
        }
    }

    return ret;
}



//returns an array of the file selections from property bag
//@pbIn: property bag containing rules
//@parent_pp_str: path container node
//@prop: file selection node name, e.g. "Attribute" or "Applies"
//@leaf: property key, e.g. null or "section"
function FCS_GetPropertyList(pbIn, parent_pp_str, prop, leaf )
{
     var values=[];

    //create base property path to the file selector nodes
    var pp_str = parent_pp_str;
    if(prop!="") pp_str+= "."+prop;
    var ppT = new PropertyPath(pp_str)

    //grab all prop values
    ppT.BeginEnum();
    while(null != pbIn.GetProperty(ppT))
    {
        if(leaf) ppT.SetTempLeaf(leaf);
        var v = pbIn.GetPropertyDef(ppT, "");

        if (v)
        {
            values.push({key:v});
        }
        ppT.SetTempLeaf("");
        ppT.IncrementEnum();
    }
    return values;
}

function FCS_ConvertValues(lst, doStr)
{
    //var str="";
    var lookup=[];

    //create lookup
    for (var i=0; i<fileCats.length; i++)
    {
	    var metatag= fileCats[i].metatag;
		lookup[metatag]=fileCats[i].name;

		if(!fileCats[i].subcats) continue;
        for(var j=0;j<fileCats[i].subcats.length;j++)
        {
		      var metatag= fileCats[i].subcats[j].metatag;
		      lookup[metatag]=fileCats[i].subcats[j].name;
        }
    }

    //associate labels to keys

    for(var i=0;i<lst.length;i++)
    {
	   lst[i].str=lookup[lst[i].key]
    }

    //build output string
    if(!doStr) return "";
    var str="";
    for(var i=0;i<lst.length;i++)
    {
		if(lst[i].str!=McUndefined)
		{
			if(str!="") str+=", ";
			str+=lst[i].str;
		}
    }
    return str;
}

/**
 * Checks for the subCat Selection Validation
 * @param fileCatSelector of interest
 */
function CatSelectionCheckValidator(fileCatSelector) {
    this.name = "CatSelectionCheckValidator";
    this.fileCatSelector = fileCatSelector;
  }

CatSelectionCheckValidator.prototype = new Validator();

/**
 * Checks if the subcategory is selected along with the major category
 */
CatSelectionCheckValidator.prototype.validate = function(value) {

var cb;
if(this.fileCatSelector.SelectAllCheck)   //check to differentiate csnewrule and filefilter rule.
	 {
		cb = document.getElementById(this.fileCatSelector.divId + "_ctl_all");
		enabledState = cb && cb.checked ? true : false;
	 }
else
	 {
		cb =  document.getElementById("chkbox_FormatCheck");
		enabledState = cb && cb.checked ? false : true;
	 }



if( !enabledState  && undefined === this.fileCatSelector.getValue()[0] )
	{
	      return false;
	}
else
	return true;

}

CatSelectionCheckValidator.prototype.errorMessage = function() {
  return resolveString(cse_pp_cs_st,"__create_condition");
}