/*
 * InputList.js
 * JavaScript control for multi-value text input.  Renders itself as a list of
 * text input fields with controls next to them and a blank input control at
 * the end to add a new value.
 *
 * Depends upon variables defined in <locale>/Strings.js and uses the
 * validation framework in validation_framework.js
 *
 * Copyright (C) 2016 McAfee, Inc. All rights reserved.
 */

/**
 * The constructor takes a properties Object which has the following fields:
 * formName - the name of the enclosing form; defaults to null;
 * baseFieldName - the base name for input fields; defaults to "text"
 * maxlength - the maximum length of each input field; defaults to 128
 * defaultRows - the number of input rows drawn before the list overflows
 * into two columns; default is 4
 * singleColumn - true if the list should only occupy a single column; defaults
 * to false
 * scrolling - whether the control should draw a scrollbar after a certain number
 * of rows; defaults to true
 * scrollAfter - the number of rows after which any scrollbar should be drawn;
 * defaults to 4
 * rowHeight - the row height to use when calculating the height of the scrolling
 * area; defaults to 25px
 * filter - a function to pass values through as they are entered (e.g.
 * converting the values to uppercase); defaults to null
 * inputFilter - a function to restrict which characters can be entered at the
 * keyboard; defaults to null
 * directive - an instruction printed at the bottom of the widget; defaults to
 * a global string called "gsInputListDefDirective"
 * vframework - a ValidationFramework object in which to place Validator objects
 * compulsory - true if at least one value must be specified; uses the vframework
 * property; defaults to false
 * displayName - used to report validation failures in the principal input field;
 * defaults to a global string called "gsInputListDefName"
 * nullsPermitted - true if empty strings are valid input values; uses the
 * vframework property; defaults to false
 * duplicatesPermitted - true if two values can be identical; uses the vframework
 * property; defaults to false
 * getUncommitted - true if the control should return any value from the final
 * input field that has not been committed; defaults to true
 * inputNamePattern - used to report validation failures in the other input fields;
 * should take the form "XYZ %1" where %1 is replaced with the number of the input
 * field; defaults to a global string called "gsDefInputPattern"
 * additionalValidators - a list of Validator objects that are applied to each input
 * field; uses the vframework property; defaults to an empty array.
 * fieldWidth - the width of each edit field; defaults to "175px"
 * changeNotify - a callback function for when the widget changes; defaults to null
 *
 * These are all public fields of an InputList object
 */

/**
 * Constructor for the InputList object
 * @param initialValues an array of values with which to initially populate
 * the control.
 * @param divId - the ID of a div that will be used to contain the control.
 * @param objectName - the name by which the object is declared; defaults to
 * "inputList"
 * @param properties - a properties object for the widget; see above for the
 * members.
 */
function InputList(initialValues, divId, objectName, properties) {
    this.currentValue = initialValues;
    this.fieldCount = (null != initialValues) ? this.currentValue.length : 0;
    this.divId = divId;
    this.objectName = objectName ? objectName : "inputList";
    this.enabled = true;

    il_init(properties, this);
}

function il_init(propsIn, propsOut) {
    if (null == propsIn) {
        propsIn = {};
    }
    // Ensure that the stringtable this class uses is not undefined
    if (undefined === this.il_st)
        this.il_st = {};
    propsOut.formName = propsIn["formName"] ?
        propsIn["formName"] : null;
    propsOut.baseFieldName = propsIn["baseFieldName"] ?
        propsIn["baseFieldName"] : "text";
    // To ensure that the IDs of the input fields are unique, use a random
    // value as a prefix
    propsOut.uniquifier = Math.random();
    propsOut.currentInput = propsOut.baseFieldName + "_current";
    propsOut.maxlength = (null != propsIn["maxlength"]) ?
        propsIn["maxlength"] : 128;
    propsOut.defaultRows = (null != propsIn["defaultRows"]) ?
        propsIn["defaultRows"] : 4;
    propsOut.singleColumn = (null != propsIn["singleColumn"]) ?
        propsIn["singleColumn"] : false;
    propsOut.scrolling = (null != propsIn["scrolling"]) ?
        propsIn["scrolling"] : true;
    propsOut.scrollAfter = (null != propsIn["scrollAfter"]) ?
        propsIn["scrollAfter"] : 4;
    propsOut.rowHeight = (null != propsIn["rowHeight"]) ?
        propsIn["rowHeight"] : 25;
    propsOut.filter = (null != propsIn["filter"]) ?
        propsIn["filter"] : null;
    propsOut.inputFilter = (null != propsIn["inputFilter"]) ?
        propsIn["inputFilter"] : null;
    propsOut.inputOnPaste = (null != propsIn["inputOnPaste"]) ?
        propsIn["inputOnPaste"] : null;
    propsOut.directive = (null != propsIn["directive"]) ?
        propsIn["directive"] : resolveString(il_st, "gsInputListDefDirective");
    propsOut.vframework = (null != propsIn["vframework"]) ?
        propsIn["vframework"] : null;
    propsOut.compulsory = (null != propsIn["compulsory"]) ?
        propsIn["compulsory"] : false;
    propsOut.displayName = (propsIn["displayName"]) ?
        propsIn["displayName"] : resolveString(il_st, "gsInputListDefName");
    propsOut.nullsPermitted = (null != propsIn["nullsPermitted"]) ?
        propsIn["nullsPermitted"] : false;
    propsOut.duplicatesPermitted = (null != propsIn["duplicatesPermitted"]) ?
        propsIn["duplicatesPermitted"] : false;
    propsOut.getUncommitted = (null != propsIn["getUncommitted"]) ?
        propsIn["getUncommitted"] : true;
    propsOut.inputNamePattern = (propsIn["inputNamePattern"]) ?
        propsIn["inputNamePattern"] : resolveString(il_st, "gsDefInputPattern");
    propsOut.additionalValidators = (propsIn["additionalValidators"]) ?
        propsIn["additionalValidators"] : [];
    propsOut.stdValidator = new CrossValidator(propsOut.objectName + ".enabled ? " +
        "!!value : true", (new RequiredValidator()).errorMessage());
    propsOut.fieldClass = propsOut.singleColumn ? "il_sc" : "il_dc";

    if (propsOut.vframework) {
        // Add validators for the principal input field
        //propsOut.vframework.addValidator(propsOut.currentInput, new ScriptValidator());
		propsOut.vframework.addValidator(propsOut.currentInput, new InputTextValidator());
		propsOut.vframework.addValidator(propsOut.currentInput, new SquareBracketValidator());
        if (propsOut.compulsory) {
            var validator = new CrossValidator(propsOut.objectName + ".enabled ? " +
                propsOut.objectName + ".getValue() && " + propsOut.objectName + ".getValue()[0] : true",
                resolveString(il_st, "gsInputListErr"));
            propsOut.vframework.addValidator(propsOut.currentInput, validator);
        }
        else {
            propsOut.vframework.addValidator(propsOut.currentInput, new Validator());
        }
        if (!propsOut.duplicatesPermitted) {
            propsOut.vframework.addValidator(propsOut.currentInput,
                new DuplicateEntryValidator(propsOut, -1));
        }
        for (var i = 0; i < propsOut.additionalValidators.length; i++) {
            propsOut.vframework.addValidator(propsOut.currentInput, propsOut.additionalValidators[i]);
        }
        propsOut.vframework.labelMap[propsOut.currentInput] = propsOut.displayName;
    }

    propsOut.changeNotify = null;
    var notifyFn = propsIn.changeNotify;
    if (notifyFn) {
        propsOut.changeNotify = ('function' == typeof(notifyFn)) ? notifyFn : new Function(notifyFn);
    }
}

/**
 * Gets the value from the input list
 * @return an array of string values
 */
InputList.prototype.getValue = function() {
    var ret = new Array();
    // Get the existing values
    for (var i = 0; i < this.fieldCount; i++) {
        ret.push(this.getInputValue(i));
    }
    if (this.getUncommitted && this.enabled) {
        var currEntry = this.getCurrentInputValue();
        if (currEntry) {
            ret.push(currEntry);
        }
    }
    return ret;
}

/**
 * Gets a value from a row of the list
 * @param index the index of the row; note that the calling function will have
 * ensured that the index is valid
 * @return the value of the row (note that subclasses may return an array of
 * values rather than a single value)
 */
InputList.prototype.getInputValue = function(index) {
    var inputId = this.uniquifier + "_" + this.baseFieldName + "_" + index;
    var text = document.getElementById(inputId).value;
    if (this.filter) {
        text = this.filter(text);
    }
    return text;
}

/**
 * Gets any uncommitted value from the current input row
 * @return the value of the row (note that subclasses may return an array of
 * values rather than a single value)
 */
InputList.prototype.getCurrentInputValue = function() {
    var ret = "";
    var currInputField = this.getCurrentInput();
    if (currInputField) {
        var currEntry = currInputField.value;
        if (currEntry) {
            if (this.filter) {
                currEntry = this.filter(currEntry);
            }
            ret = currEntry;
        }
    }
    return ret;
}

/**
 * Gets a handle to the current input field
 */
InputList.prototype.getCurrentInput = function() {
    var inputId = this.uniquifier + "_" + this.currentInput;
    return document.getElementById(inputId);
}

/**
 * Sets the value of the input list
 * @param val an array of values
 */
InputList.prototype.setValue = function(val) {
    if (val instanceof Array) {
        this.currentValue = val;
        this.fieldCount = this.currentValue.length;
    }
    else if (null != val) {
        this.currentValue = [val];
        this.fieldCount = 1;
    }
    else {
        this.currentValue = null;
        this.fieldCount = 0;
    }
}

/**
 * Gets the number of rows and columns to be displayed
 * @return an Object indexed as "rows" and "columns"
 */
InputList.prototype.getDimensions = function(enabled) {
    var colCount = 1;
    var inputCount = this.fieldCount;
    if (enabled || 0 == inputCount) {
        inputCount++;
    }
    if (inputCount > this.defaultRows && this.singleColumn == false) {
        colCount = 2;
    }
    var rowCount = (inputCount < this.defaultRows) ? inputCount : this.defaultRows;
    if (inputCount > this.defaultRows * colCount) {
        rowCount = Math.ceil(inputCount / colCount);
    }
    var ret = { fields:inputCount, rows:rowCount, columns:colCount };
    return ret;
}

/**
 * Draws the InputList control
 * @param enabled true if the control is to be enabled, false otherwise
 */
InputList.prototype.draw = function(enabled) {
    this.enabled = enabled;
    var html = "";
    // The control is drawn in a single column unless the number of values
    // plus any empty text field at the end exceeds four, in which case the
    // control is drawn as two columns
    var dimensions = this.getDimensions(enabled);
    var inputCount = dimensions.fields;
    var colCount = dimensions.columns;
    var rowCount = dimensions.rows;

    html += "<div";
    if (this.scrolling && rowCount > this.scrollAfter) {
        var height = this.rowHeight * this.scrollAfter;
        html += " style=\"height:" + height + "px; overflow:auto;\"";
    }
    html += ">";
    // Specify 95% width to avoid horizontal scrollbar in Mozilla browsers
    html += "<table border=\"0\" cellpadding=\"0\" cellspacing=\"4\" width=\"95%\">";
    html += "<tbody>";
    for (var i = 0; i < rowCount; i++) {
        html += "<tr>";
        for (var j = 0; j < 2; j++) {
            if (j > 0 && this.singleColumn)
                continue;
            var index = 0;
            if (j == 0) {
                index = i;
            }
            else {
                index = i + rowCount;
            }
            if (index < this.currentValue.length) {
                html += this.drawInputRow(enabled, index);
            }
            else if (index == (inputCount - 1)) {
                html += this.drawCurrentInputRow(enabled);
            }
            else {
                html += this.drawEmptyRow();
            }
            html += "<td width=\"10\">&nbsp;</td>";
        }
        html += "<td class=\"mw\">&nbsp;</td>";
        html += "</tr>";
    }
    html += "</tbody>";
    html += "</table>";
    html += "</div>";
    if (enabled && this.directive) {
        html += "<div style=\"padding:0px 0px 0px 4px\">" +
            ReplaceSpecialHTMLChars(this.directive) + "</div>";
    }
    var div = document.getElementById(this.divId);
    if (div) {

        div.innerHTML = html;
    }
    if (enabled && this.inputFilter) {
        var iList = [];
        for (var i = 0; i < this.fieldCount; i++) {
            iList.push(this.uniquifier + "_" + this.baseFieldName + "_" + i);
        }
        iList.push(this.uniquifier + "_" + this.currentInput);
        for (var i = 0; i < iList.length; i++) {
            var e = document.getElementById(iList[i]);
            if (e) {
                e.onkeypress = this.inputFilter;
                if ( this.inputOnPaste )
                	e.onpaste = this.inputOnPaste;
            }
        }
    }
}

/**
 * Draws a single row of the input table
 * @param enabled true if the input controls are enabled, false otherwise
 * @param index the index of this row; used to look up the value to set the value
 * of the control; note the caller will already have done bounds checking on the
 * input.
 */
InputList.prototype.drawInputRow = function(enabled, index) {
    var html = "<td class=\"" + this.fieldClass + "\">";
    var value = this.currentValue[index];
    var fieldName = this.baseFieldName + "_" + index;
    var fieldId = this.uniquifier + "_" + fieldName;
    var fieldFlag = "";
    if (!this.hasValidator(fieldName)) {
        if (!this.nullsPermitted) {
            this.vframework.addValidator(fieldName, this.stdValidator);
			this.vframework.addValidator(fieldName, new InputTextValidator());
			this.vframework.addValidator(fieldName, new SquareBracketValidator());
        }
        if (!this.duplicatesPermitted) {
            this.vframework.addValidator(fieldName,
                new DuplicateEntryValidator(this, index));
        }
        for (var i = 0; i < this.additionalValidators.length; i++) {
            this.vframework.addValidator(fieldName, this.additionalValidators[i]);
        }
        this.vframework.labelMap[fieldName] = formatString(
            this.inputNamePattern, [index + 1]);
    }
    if (!enabled) {
        fieldFlag = "disabled";
    }
    var className = "mw textField";
    html += "<input type=\"text\" name=\"" + fieldName +
        "\" id=\"" + fieldId + "\" " + fieldFlag +
        " maxlength=\"" + this.maxlength + "\" value=\"" +
        value + "\" class=\"" + className + "\"";
    var onblur = this.objectName + ".onblur('" + fieldName+ "');";
    if (this.filter) {
        onblur += "this.value=" + this.objectName + ".filter(this.value);";
    }
    html += " onblur=\"" + onblur + "\"";
    // Add change notification
    if (this.changeNotify) {
        html += " onfocus=\"" + this.objectName + ".changeNotify()\"";
    }
    html += ">";
    html += "</td>";
    html += "<td nowrap class=\"" + this.fieldClass + "_l\">";
    if (enabled) {
        html += "<a href=\"javascript:" + this.objectName + ".deleteItem(" + index +
            ")\" class=\"blueLink\">" + resolveString(il_st, "gsDelete") + "</a>";
    }
    html += "</td>";
    return html;
}

/**
 * Draws an empty row
 */
InputList.prototype.drawEmptyRow = function() {
    return '<td class="' + this.fieldClass + '">&nbsp;</td><td class="' +
        this.fieldClass + '_l">&nbsp;</td>';
}

/**
 * Draws the current input row of the input table
 * @param enabled true if the input controls are enabled, false otherwise
 */
InputList.prototype.drawCurrentInputRow = function(enabled) {
    var html = "<td class=\"" + this.fieldClass + "\">";
    var fieldName = this.currentInput;
    var fieldId = this.uniquifier + "_" + fieldName;
    var fieldFlag = "";
    if (!enabled) {
        fieldFlag = "disabled";
    }
    var className = "mw textField";
    html += "<a name=\"" + this.fieldId + "\"></a>";
    html += "<input type=\"text\" name=\"" + fieldName +
        "\" id=\"" + fieldId + "\" " + fieldFlag +
        " maxlength=\"" + this.maxlength + "\" value=\"\" class=\"" + className + "\"";
    var onblur = this.objectName + ".onblur('" + fieldName+ "');";
    if (this.filter) {
        onblur += "this.value=" + this.objectName + ".filter(this.value);";
    }
    html += " onblur=\"" + onblur + "\"";
    html += ">";
    html += "</td>";
    html += "<td nowrap class=\"" + this.fieldClass + "_l\">";
    if (enabled) {
        html += "<a href=\"javascript:" + this.objectName + ".add()\" class=\"blueLink\">" +
            resolveString(il_st, "gsAdd") + "</a>";
    }
    html += "</td>";
    return html;
}

/**
 * Adds an item to the list; does nothing if there is no value in the bottom edit control
 * and null values are not permitted
 */
InputList.prototype.add = function() {
    var newVal = this.getCurrentInputValue();
    if (!this.duplicatesPermitted) {
        // Check that the value is not already present
        for (var i = 0; i < this.fieldCount; i++) {
            var fieldName = this.baseFieldName + "_" + i;
            var fieldId = this.uniquifier + "_" + fieldName;
            var e = document.getElementById(fieldId);
            if (!e)
                continue;
            if (e.value == newVal) {
                return;
            }
        }
    }
    if (newVal || this.nullsPermitted) {
        if (this.filter) {
            // Set the value in the field to validate it since
            // the filter will have modified it
            this.getCurrentInput().value = newVal;
        }
        var valid = true;
        if (this.vframework) {
            valid = this.vframework.check(this.currentInput);
        }
        if (valid) {
            var unc = this.getUncommitted;
            this.getUncommitted = false;
            var currVal = this.getValue();
            this.getUncommitted = unc;
            currVal.push(newVal);
            this.setValue(currVal);
            this.draw(true);
            if (this.changeNotify) {
                this.changeNotify();
            }
        }
    }
    this.focus();
}

/**
 * Deletes an item from the list
 * @param index the index of the item to remove
 */
InputList.prototype.deleteItem = function(index) {
    var currInputValue = this.getCurrentInputValue();
    var removeFinal = false;
    if (currInputValue) {
        removeFinal = true;
    }
    this.currentValue = this.getValue();
    if (removeFinal) {
        this.currentValue.length--;
    }
    // Remove the item in the middle
    this.currentValue.splice(index, 1);
    this.fieldCount--;
    this.draw(true);
    e = document.getElementById(this.uniquifier + "_" + this.currentInput);
    e.value = currInputValue;
    if (this.changeNotify) {
        this.changeNotify();
    }
    this.focus();
}

/**
 * Sets focus to the current input field
 */
InputList.prototype.focus = function() {
    var fieldId = this.uniquifier + "_" + this.currentInput;
    var focusStr = "document.getElementById('" + fieldId + "').focus()";
    setTimeout(focusStr, 200);
    //location.hash = this.currentInputId;
}

/**
 * onchange action for the fields in the input list
 * @param fieldName the name of the field
 */
InputList.prototype.onblur = function(fieldName) {
    if (this.vframework) {
        this.vframework.onchange(fieldName);
    }
}

/**
 * Checks whether a validator exists for a given field
 * @param fieldName the field in question
 * @return true if yes (or if no validation framework is present),
 * false otherwise
 */
InputList.prototype.hasValidator = function(fieldName) {
    if (!this.vframework) {
        return true;
    }
    if (null != this.vframework.getValidators(fieldName)) {
        return true;
    }
    return false;
}

/**
 * Constructor for the HiddenInputList object
 * @param initialValues an array of values with which to initially populate
 * the control.
 * @param divId - the ID of a div that will be used to contain the control.
 * @param objectName - the name by which the object is declared; defaults to
 * "inputList"
 * @param properties - a properties object for the widget; see the top of this
 * file for the members.  The properties controlling visual appearance will
 * have no effect since the list is made up entirely from hidden fields
 */
function HiddenInputList(initialValues, divId, objectName, properties) {
    this.currentValue = initialValues;
    this.fieldCount = (null != initialValues) ? this.currentValue.length : 0;
    this.divId = divId;
    this.objectName = objectName ? objectName : "inputList";
    this.enabled = true;

    il_init(properties, this);
}

HiddenInputList.prototype = new InputList(null, "", "", {});

/**
 * NO-OP since there is no current value
 */
HiddenInputList.prototype.getCurrentInputValue = function() {
    return null;
}

/**
 * Draws the HiddenInputList control
 * @param enabled ignored
 */
HiddenInputList.prototype.draw = function(enabled) {
    var html = "";
    for (var i = 0; i < this.currentValue.length; i++) {
        var value = this.currentValue[i];
        var fieldName = this.baseFieldName + "_" + i;
        var fieldId = this.uniquifier + "_" + fieldName;
        html += '<input type="hidden" name="' + fieldName + '" id="' +
            fieldId + '" value="' + value + '">';
    }
    var div = document.getElementById(this.divId);
    if (div) {
        div.innerHTML = html;
    }
}

/**
 * NO-OP since there is nothing to focus on
 */
HiddenInputList.prototype.focus = function() {
}

/**
 * Checks against duplicate entries in the input list
 * @param inputList the InputList control of interest
 * @param ctlIdx the index of the input control (-1 means the
 * main input)
 */
function DuplicateEntryValidator(inputList, ctlIdx) {
    this.name = "DuplicateEntryValidator";
    this.inputList = inputList;
    this.ctlIdx = ctlIdx;
}

DuplicateEntryValidator.prototype = new Validator();

/**
 * Validates a field against every other field in the input list,
 * checking that no other field has the same value
 */
DuplicateEntryValidator.prototype.validate = function(value) {
    var ret = true;
    var v = value;
    if (this.inputList.filter) {
        v = this.inputList.filter(value);
    }
    var inputCount = this.inputList.fieldCount;
    var startIdx = (-1 == this.ctlIdx) ? (inputCount - 1) : (this.ctlIdx - 1);
    for (var i = startIdx; i >= 0; i--) {
        var fieldName = this.inputList.baseFieldName + "_" + i;
        var fieldId = this.inputList.uniquifier + "_" + fieldName;
        var e = document.getElementById(fieldId);
        if (!e)
            continue;
        if (e.value == v) {
            ret = false;
            break;
        }
    }
    return ret;
}

DuplicateEntryValidator.prototype.errorMessage = function() {
    return resolveString(il_st, "gsDuplicateEntryErr");
}
