/**
 * JavaScript Online Help Framework
 * Depends on splitPath function in Global.js
 * Copyright (C) 2009 McAfee, Inc. All rights reserved.
 */

// Default height and width of help window - may be customized
// by a point product
// The width of the help window
var oh_hwWidth = 600;
// The height of the help window
var oh_hwHeight = 400;


/**
 * Constructor for the Help object
 * @param hwName the name of the help window
 * @param pathPrefix the path prefix to use to build up URLs
 * to help pages
 * @param subHelp true if this help is to display secondary help
 * (this feature is used by the help window to display popup help)
 */
function Help(hwName, pathPrefix, subHelp) {
    if (!hwName) {
	hwName = "help";
    }
    // Add some random decoration to the window name so as not to
    // clash with any similarly named open window
    this.hwName = Math.floor(Math.random() * 10000000) + "_" + hwName;
    if (!pathPrefix) {
	this.pathPrefix = "";
    }
    else {
	this.pathPrefix = pathPrefix;
    }
    this.helpWindow = null;
    this.currentTopic = "";
	this.currentPage = "";
    this.subHelp = subHelp ? true : false;
	this.windowClosed = false;
}

/**
 * Sets the prefix for help page URLs
 * @param path Prefix the path prefix
 */
Help.prototype.setPathPrefix = function(pathPrefix) {
    this.pathPrefix = pathPrefix;
}

/**
 * Determines whether a help window is currently open
 */
Help.prototype.isHelpOpen = function() {
    var ret = true;
	// Since window.closed property will not work under ePO MMC console
	// check windowClosed member variable first.
    if (!this.helpWindow || this.windowClosed || this.helpWindow.closed) {
	ret = false;
    }
    return ret;
}

/**
 * The main openHelp function
 * @param topic the name of the main topic to fetch
 * @param subTopic an optional parameter to indicate a subtopic to
 * expand
 */
Help.prototype.openHelp = function(topic, subTopic) {
    if (!topic) {
	this.currentTopic = this.currentPage;
	topic = this.currentTopic;
    }
    if (!this.isHelpOpen()) {
	// Open a new Window
	var pageUrl = this.makeHelpUrl(topic);
	if (subTopic) {
	    pageUrl += "#" + subTopic;
	}
	this.openHelpByUrl(pageUrl);
	this.currentTopic = topic;
    }
    else {
	this.setMainHelpTopic(topic);
	if (subTopic) {
	    this.showSubTopic(topic, subTopic);
	}
	else {
	    this.helpWindow.focus();
	}
    }
}

/**
 * Opens a help page by specific URL
 * @param url the URL to load
 * @param centred true if the window should be shown centred on the screen
 */
Help.prototype.openHelpByUrl = function(url, centred) {
    var wFeatures = new Array();
    wFeatures.push("width=" + oh_hwWidth);
    wFeatures.push("height=" + oh_hwHeight);
    wFeatures.push("scrollbars=yes");
    wFeatures.push("resizable=yes");
    wFeatures.push("status=no");

    if (screen) {
	var left;
	if (this.subHelp || centred) {
	    // Display the help window centred
	    var top;
	    if (screen.availHeight) {
		top = (screen.availHeight - oh_hwHeight) / 2;
	    }
	    else if (screen.height) {
		top = (screen.height - oh_hwHeight) / 2;
	    }
	    if (top) {
		wFeatures.push("top=" + top);
	    }
	    left = (screen.availWidth - oh_hwWidth) / 2;
	}
	else {
	    // Display the help flush against the right
	    // of the screen, with a delta to ensure the
	    // chrome doesn't go off the right of the screen
	    left = screen.availWidth - oh_hwWidth - 5;
	}
	wFeatures.push("left=" + left);
    }

    var helpUrl = this.pathPrefix + "help.html";
    helpUrl = helpUriAppend(helpUrl, "page", url);
    var w = window.open(helpUrl, this.hwName, wFeatures.join(","));
    if (!w.opener) {
	w.opener = window;
    }
    this.helpWindow = w;
    this.helpWindow.focus();
	this.windowClosed = false;
}

/**
 * Appends a parameter to a URI
 * @param uri the URI to modify
 * @param param the parameter to append
 * @param val the value of the parameter
 * @return the modified URI
 */
function helpUriAppend(uri, param, val) {
    if (-1 == uri.indexOf("?", 0)) {
        uri += "?";
    } else {
        uri += "&";
    }
    uri += param + "=" + escape(val);
    return uri;
}

Help.prototype.makeHelpUrl = function(topic, subTopic) {
    var ret = this.pathPrefix + topic + ".html";
    if (subTopic) {
	ret += "#" + subTopic;
    }
    return ret;
}

/**
 * Gets the current topic displayed in the help window
 */
Help.prototype.getCurrentTopic = function() {
    var ret = "";
    if (!this.isHelpOpen()) {
	return "";
    }
    var path = this.helpWindow.help_contents.location.pathname;
    if (path) {
	ret = splitPath(path)["basename"];
    }
    return ret;
}

/**
 * Sets the main help topic in the help window
 * @param topic the name of the main topic to display
 */
Help.prototype.setMainHelpTopic = function(topic) {
    if (this.isHelpOpen() && topic) {
	if (topic != this.getCurrentTopic()) {
	    this.helpWindow.help_contents.location.href =
		this.makeHelpUrl(topic, "top");
	    // Close any popup help that may have been opened in the
	    // help window
	    /*if (!this.subHelp && this.helpWindow.help && this.helpWindow.help.isHelpOpen()) {
		this.helpWindow.help.helpWindow.close();
	    }*/
	}
    }
    this.currentTopic = topic;
	if(topic != "Resources")
		this.currentPage = topic;
}

/**
 * Displays a subtopic in a help page
 * @param mainTopic the main help topic (this is used to load the help
 * window if it is not open)
 * @param subTopic the subtopic to display
 */
Help.prototype.showSubTopic = function(mainTopic, subTopic) {
    if (this.isHelpOpen()) {
	if (mainTopic != this.getCurrentTopic()) {
	    this.setMainHelpTopic(mainTopic);
	}
	// Expand the subtopic
	this.helpWindow.expandSubTopic(subTopic);
	// Set the location hash in the help contents frame so that
	// the browser scrolls down to it
	var helpFrame = this.helpWindow.help_contents;
	helpFrame.location.hash = subTopic;
	this.helpWindow.focus();
    }
    else {
	this.openHelp(mainTopic, subTopic);
    }
}

/**
 * Loads a help topic as popup help (without changing the current topic
 * @param mainTopic the main help topic
 * @param subTopic any subtopic to display
 */
Help.prototype.showPopupHelp = function(mainTopic, subTopic) {
    if (!subTopic) {
	subTopic = "top";
    }
    if (this.isHelpOpen()) {
	this.helpWindow.loadPopupHelp(mainTopic, subTopic, this.pathPrefix);
    }
    else {
	var helpUrl = this.makeHelpUrl(mainTopic, subTopic);
	this.openHelpByUrl(helpUrl, true);
    }
}

