/**
 * JavaScript for counting shared object references
 *
 * Copyright (C) 2009 McAfee, Inc. All rights reserved
 */

 /**
  * Reference counting class; keeps a total count as well as counts keyed
  * by policy-set > policy > settings instance
  */
function RefCounter() {
    this.refCount = 0;
    this.pset = [];
}

/**
 * Adds a policy set reference to the counter
 * @param psName the name of the policy set (does not need to be resolved from
 * a stringtable key)
 * @param psPP the property path (string format) to the policy set
 */
RefCounter.prototype.addPSet = function(psName, psPP) {
    this.pset.push({name:resolveString(cse_pp_ps_st, psName), pp:psPP, policies:[]})
}

/**
 * Adds a policy reference to the counter
 * @param psIdx the index of the policy set in the reference counter
 * @param polId the ID of the policy
 * @param polName the name of the policy (does not need to be resolved from
 * a stringtable key)
 * @param polPP the property path (string format) to the policy
 */
RefCounter.prototype.addPolicy = function(psIdx, polId, polName, polPP) {
    if (psIdx >= 0 && psIdx < this.pset.length) {
        var psObj = this.pset[psIdx];
        psObj.policies.push({name:resolveString(cse_pp_st, polName),
            id:polId, pp:polPP, refCount:0});
    }
}

/**
 * Ups the reference count
 * @param psIdx the index of the policy set in the reference counter
 * @param polIdx the index of the policy in the policy-set
 */
RefCounter.prototype.addRef = function(psIdx, polIdx) {
    if (psIdx >= 0 && psIdx < this.pset.length) {
        var polArray = this.pset[psIdx].policies;
        if (polIdx >= 0 && polIdx < polArray.length) {
            // Add both to the policy's ref count and the total
            polArray[polIdx].refCount++;
            this.refCount++;
        }
    }
}

/**
 * Gets the number of dependent policies (as opposed to the total
 * reference count which is available through the refCount field)
 */
RefCounter.prototype.getDependentPolicyCount = function() {
    var depPolicies = 0;
    var psArray = this.pset;
    for (var i = 0; i < psArray.length; i++) {
        var polList = psArray[i].policies;
        for (var j = 0; j < polList.length; j++) {
            if (0 != polList[j].refCount)
                ++depPolicies;
        }
    }
    return depPolicies;
}

/**
 * Generates a data dump of the RefCounter object
 */
RefCounter.prototype.dump = function() {
    var sArray = [];

    var psArray = this.pset;
    for (var i = 0; i < psArray.length; i++) {
        var psObj = psArray[i];
        var polArray = psObj.policies;
        for (var j = 0; j < polArray.length; j++) {
            var polObj = polArray[j];
            var repStr = psObj.name + " > " + polObj.name + ": ";
            if (1 == polObj.refCount) {
                repStr += "1 settings instance";
            }
            else {
                repStr += polObj.refCount + " settings instances";
            }
            sArray.push(repStr);
        }
    }

    sArray.push("");
    sArray.push("Total: " + this.refCount);
    return sArray.join("\n");
}

/**
 * Makes a brief report of the state of the RefCounter object
 */
RefCounter.prototype.briefReport = function(divId) {
    var div = document.getElementById(divId);
    if (!div)
        return;
    while(div.firstChild)
        div.removeChild(div.firstChild);

    var depPolicies = this.getDependentPolicyCount();
    div.appendChild(document.createTextNode(resolveString(
        cse_pp_st, "__dependent_policies", [depPolicies])));
}

/**
 * Convenience function for upping the reference count in a RefCounter object;
 * makes the assumption that you are working on the last policy in the last policy set
 */
RefCounter.addRefD = function(refCounter) {
    refCounter.addRef(refCounter.pset.length - 1,
        refCounter.pset[refCounter.pset.length - 1].policies.length - 1);
}

/**
 * Gets the reference count for a given object type
 * @param typeField a field that identifies the category of the object
 * (e.g. when counting references to a settings instance, this might be
 " "AV" while it might be "File" when counting filtering rules)
 * @param objId the ID of the object being counter
 * @param pb the property bag to use to do the reference counting (needs
 * to contain an up-to-date policy-set tree)
 * @param rFn the function to use to count references within a PolicySettings blob
 * @param tFn the function to use to convert the typeField parameter to an identifer
 * for a policy settings blob such as "AV" or "AS"
 */
RefCounter.getRefCount = function(typeField, objId, pb, rFn, tFn) {
    var refCounter = new RefCounter();
    if (null == pb)
        pb = top.gPB;
    var basePP = new PropertyPath("Policies.PolicySet");
    for (basePP.BeginEnum(); null != pb.GetProperty(basePP); basePP.IncrementEnum()) {
        basePP.SetTempLeaf("name");
        var psName = pb.GetPropertyDef(basePP, "");
        basePP.SetTempLeaf("");
        var psPP = basePP.GetPath();
        refCounter.addPSet(psName, psPP);
        RefCounter.getRefCount2(typeField, objId, pb, psPP, refCounter, rFn, tFn);
    }
    return refCounter;
}

/**
 * Second-stage reference counting
 * @param typeField a field that identifies the category of the object
 * (e.g. when counting references to a settings instance, this might be
 " "AV")
 * @param objId the ID of the object being counter
 * @param pb the property bag to use to do the reference counting
 * @param psPP the property path of the policy set currently beng processed
 * @param refCounter a reference counter object
 */
RefCounter.getRefCount2 = function(typeField, objId, pb, psPP, refCounter, rFn, tFn) {
    var basePP = new PropertyPath(psPP + ".Policy");
    for (basePP.BeginEnum(); null != pb.GetProperty(basePP); basePP.IncrementEnum()) {
        basePP.SetTempLeaf("id");
        var polId = pb.GetPropertyDef(basePP, "");
        basePP.SetTempLeaf("name");
        var polName = pb.GetPropertyDef(basePP, "");
        basePP.SetTempLeaf("");
        var polPP = basePP.GetPath();
        refCounter.addPolicy(refCounter.pset.length - 1, polId, polName, polPP);

        // Convert the typeField parameter into a settings identifier using
        // the function provided
        var sType = typeField;
        if (tFn)
            sType = tFn(typeField);
        // sType should be something like "AV" or "AS"; use it to examine policy settings
        var pp2 = new PropertyPath(polPP + "." + sType + "PolicySettings");
        for (pp2.BeginEnum(); null != pb.GetProperty(pp2); pp2.IncrementEnum()) {
            // The internal details of the XXPolicySettings blob needs to be
            // picked apart by a cusomt function
            rFn(objId, pb, pp2.GetPath(), refCounter, typeField);
        }
    }
}

/**
 * Transform function for filter rules reference couting; turns a rule
 * identifier, such as "File", into a settings identifier, such as "FF"
 */
RefCounter.ruleTypeToSettingsType = function(ruleType) {
    if (!ruleType)
        ruleType = "File";
    var typeMap = {"File":"FF"};
    return typeMap[ruleType] ? typeMap[ruleType] : ruleType;
}

/**
 * Gets the reference count for a given filter rule
 * @param id the ID of the rule being counted
 * @param pb the property bag to use to do the reference counting
 * @param polSettingPP the property path of the policy settings instance
 * currently beng processed
 * @param refCounter a reference counter object
 */
RefCounter.getRuleRefCount = function(id, pb, polSettingPP, refCounter, type) {
    var basePP = new PropertyPath(polSettingPP + ".RuleRef");
    for (basePP.BeginEnum(); null != pb.GetProperty(basePP); basePP.IncrementEnum()) {
        basePP.SetTempLeaf("id");
        if (id == pb.GetProperty(basePP)) {
            RefCounter.addRefD(refCounter);
        }
        basePP.SetTempLeaf("");
    }
}

/**
 * Gets the reference count for a given settings instance
 * @param id the ID of the settings instance being counted
 * @param pb the property bag to use to do the reference counting
 * @param polSettingPP the property path of the policy settings instance
 * currently beng processed
 * @param refCounter a reference counter object
 */
RefCounter.getSettingsRefCount = function(id, pb, polSettingPP, refCounter, type) {
    var testId = pb.GetProperty(polSettingPP + ".RefSettings");
    if (testId == id) {
        RefCounter.addRefD(refCounter);
    }
}

/**
 * Transform function for alerts reference couting; turns an alert
 * identifier, such as "AV_REPLACEMENT", into a settings identifier,
 * such as "AV"
 */
RefCounter.alertTypeToSettingsType = function(alertType) {
    return pp_cse_alertTypeMap[alertType] ? pp_cse_alertTypeMap[alertType] :
        alertType;
}

/**
 * Gets the reference count for a given alert
 * @param id the ID of the alert instance being counted
 * @param pb the property bag to use to do the reference counting
 * @param polSettingPP the property path of the policy settings instance
 * currently beng processed
 * @param refCounter a reference counter object
 */
RefCounter.getAlertRefCount = function(id, pb, polSettingPP, refCounter, type) {
    var alertsPP = polSettingPP + ".Alerts";
    if (null != pb.GetProperty(alertsPP)) {
        // Take a copy of just this part of the property bag
        var pbT = new PropertyBag();
        pb.GetElement(alertsPP, pbT);
        var subPaths = pbT.GetAllPaths();
        for (var i = subPaths.length - 1; i >= 0; i--) {
            if ( id == pbT.GetProperty(subPaths[i]) && ( (subPaths[i].match("."+pp_cse_alertTypeMatch[type])) !=  null ) )
                RefCounter.addRefD(refCounter);
        }
    }
}


