/*
 * Rollups.js
 * Class definition and helper functions for rollups to make collapsible page
 * elements.
 *
 * Copyright (C) 2009 McAfee, Inc. All rights reserved.
 */

// Check for availability of string resources
var useAlts = (this.gsExpand != null);

/**
 * Constructor for the RollupSet class
 * @param objectName the declared name of the RollupSet object; this is
 * required for building up URLs
 * @param eStateImage the URL for the expanded-state image
 * @param cStateImage the URL for the collapsed-state image
 * @param rememberState true if the rollups should retain their state between
 * page visits / refreshes
 * @param useTextLinks true if the controls are to be drawn as text links
 * rather than image links (the default behaviour)
 */
function RollupSet(objectName, eStateImage, cStateImage, rememberState, useTextLinks) {
    this.rollups = new Array();
    this.objectName = objectName;
    this.eStateImage = eStateImage;
    this.cStateImage = cStateImage;
    if (rememberState) {
        this.memory = true;
        // Create the rollups register if it does not already exist
        if (undefined == parent.rollup_register) {
            parent.rollup_register = new Object();
        }
    }
    else {
        this.memory = false;
    }
    this.useTextLinks = useTextLinks ? true : false;
}

/**
 * Adds a Rollup to the RollupSet
 * @param name the name of the Rollup
 * initialState true if the rollup is to be shown expanded initially;
 * if undefined the state will be the last selected expansion state (or true if
 * the RollupSet is not configured to retain state)
 */
RollupSet.prototype.add = function(name, initialState) {
    // Logic for handling initialState:
    // If the initial state parameter is defined it is honoured unless there is
    // a saved selection state
    // If the initial state parameter is undefined, the expansion state is set
    // to true unless there is a saved selection state
    var expanded = undefined;
    if (undefined == initialState) {
        if (this.memory) {
            expanded = parent.rollup_register[name];
        }
        else {
            expanded = true;
        }
    }
    else {
        if (this.memory && parent.rollup_register.hasOwnProperty(name)) {
            expanded = parent.rollup_register[name];
        }
        else {
            expanded = initialState;
        }
    }
    if (undefined == expanded) {
        expanded = true;
    }
    // Build values for the image ID for the control and for the divs swapped
    // by it.
    var imageId = name + "_ctl";
    var expandDiv = name + "_expanded";
    var contractDiv = name + "_collapsed";
    var rollup = new Rollup(name, imageId, expandDiv, contractDiv, expanded, this.memory);
    rollup.setImageURLs(this.eStateImage, this.cStateImage, this.useTextLinks);
    this.rollups.push(rollup);
}

/**
 * Adds a TableRollup to the RollupSet
 * @param name the name of the TableRollup
 * @param table_id the ID of the table controlled by the rollup
 * @param initialState true if the rollup is to be shown expanded initially;
 * if undefined the state will be the last selected expansion state (or true if
 * the RollupSet is not configured to retain state)
 * @param startRow the first row hidden when the rollup is collapsed (defaults to 1)
 * @param endRow the last row hidden (defaults to the last row in the table)
 */
RollupSet.prototype.addTableRollup = function(name, table_id, initialState, startRow, endRow) {
    // Logic for handling initialState:
    // If the initial state parameter is defined it is honoured unless there is
    // a saved selection state
    // If the initial state parameter is undefined, the expansion state is set
    // to true unless there is a saved selection state
    var expanded = undefined;
    if (undefined == initialState) {
        if (this.memory) {
            expanded = parent.rollup_register[name];
        }
        else {
            expanded = true;
        }
    }
    else {
        if (this.memory && parent.rollup_register.hasOwnProperty(name)) {
            expanded = parent.rollup_register[name];
        }
        else {
            expanded = initialState;
        }
    }
    if (undefined == expanded) {
        expanded = true;
    }
    // Build values for the image ID for the control and for the divs swapped
    // by it.
    var imageId = name + "_ctl";
    var rollup = new TableRollup(name, imageId, table_id, expanded, this.memory, startRow, endRow);
    rollup.setImageURLs(this.eStateImage, this.cStateImage, this.useTextLinks);
    this.rollups.push(rollup);
}

/**
 * Gets a reference to a Rollup
 * @param idx the index into the rollup set
 */
RollupSet.prototype.get = function(idx) {
    if (idx < 0 || idx >= this.rollups.length) {
        throw new RangeError(idx);
    }
    return this.rollups[idx];
}

/**
 * Displays all the Rollup objects in the set
 */
RollupSet.prototype.showAll = function() {
    for (var i = 0; i < this.rollups.length; i++) {
        this.rollups[i].show();
    }
}

/**
 * Creates the HTML necessary for making a rollup control
 * @param idx the index of the control being drawn in the rollup set
 */
RollupSet.prototype.makeControlHTML = function(idx) {
    var rollup = this.get(idx);
    if (null == rollup)
        return "";
    var html = '<a href="javascript:' + this.objectName +
        '.get(' + idx + ').toggle();" id="' + rollup.link_id + '" class="rollup';
    if (this.useTextLinks)
        html += ' blueLink';
    html += '"></a>';
    return html;
}

/**
 * Uses DOM methods to create a rollup control
 * @param idx the index of the control being drawn in the rollup set
 */
RollupSet.prototype.domMakeControl = function(idx) {
    var rollup = this.get(idx);
    if (null == rollup)
        return null;
    var ru = document.createElement("a");
    ru.href = "javascript:" + this.objectName +
        ".get(" + idx + ").toggle();"
    ru.className = "rollup";
    if (this.useTextLinks) {
        ru.className += " blueLink";
    }
    ru.id = rollup.link_id;
    return ru;
}

/**
 * Creates the HTML necessary for making a rollup control
 */
RollupSet.prototype.makeControl = function(idx) {
    document.write(this.makeControlHTML(idx));
}

/**
 * Gets the number of controls in the RollupSet
 */
RollupSet.prototype.getCount = function() {
    var ret = 0;
    if (this.rollups)
        ret = this.rollups.length;
    return ret;
}

/**
 * Constructor for the Rollup class
 * @param name an identifier for this rollup
 * @param link_id the id of the rollup link
 * @param expandDiv the <div> block that is displayed when the rollup is expanded
 * @param contractDiv the <div> block that is displayed when the rollup is contracted
 * @param expanded true if the rollup is expanded, false otherwise
 * @param rememberState true if the rollup should retain its toggle state between
 * page visits / refreshes
 */
function Rollup(name, link_id, expandDiv, contractDiv, expanded, rememberState) {
    this.name = name;
    this.link_id = link_id;
    this.expandDiv = expandDiv;
    this.contractDiv = contractDiv;
    if (undefined == expanded) {
        expanded = true;
    }
    this.expanded = expanded;
    if (undefined == rememberState) {
        rememberState = false;
    }
    this.memory = rememberState;
}

/**
 * Specifies the image URLs to use in the Rollup control
 * @param eStateImage the URL for the expanded-state image
 * @param cStateImage the URL for the collapsed-state image
 * @param useTextLinks true if the control should draw itself as a text link
 * rather than an image link (the default); if true, the image URLs are treated
 * as the link text to use.
 */
Rollup.prototype.setImageURLs = function(eStateImage, cStateImage, useTextLinks) {
    this.eStateImage = eStateImage;
    this.cStateImage = cStateImage;
    this.useTextLinks = useTextLinks;
}

/*
 * Displays or hides the area controlled by the rollup
 */
Rollup.prototype.show = function() {
    var img;
    if (this.expanded) {
        // Rolled-down state
        img = this.cStateImage;
        var div = document.getElementById(this.expandDiv);
        if (div) {
            div.style.display = "";
        }
        div = document.getElementById(this.contractDiv);
        if (div) {
            div.style.display = "none";
        }
    }
    else {
        // Rolled-up state
        img = this.eStateImage;
        var div = document.getElementById(this.expandDiv);
        if (div) {
            div.style.display = "none";
        }
        div = document.getElementById(this.contractDiv);
        if (div) {
            div.style.display = "";
        }
    }

    var link = document.getElementById(this.link_id);
    if (link) {
        if (!this.useTextLinks) {
            link.innerHTML = '<img width="11" ' +
            'height="9" border="0" src="' + img + '">';
        }
        else {
            link.innerHTML = img;
        }
    }
}

/**
 * Rollup handler
 */
Rollup.prototype.toggle = function() {
    // Toggle state
    this.expanded = this.expanded ? false : true;
    if (this.memory) {
        // Remember state
        parent.rollup_register[this.name] = this.expanded;
    }
    // Display
    this.show();
}

/**
 * Creates the HTML required to start a rollup area
 */
Rollup.prototype.start = function() {
    var html = "<div id=\"" + this.expandDiv + "\">";
    document.write(html);
}

/**
 * Creates the boundary between the expanded and collapsed sections of a rollup
 */
Rollup.prototype.boundary = function() {
    var html = "</div><div id=\"" + this.contractDiv + "\" style=\"display:none\">";
    document.write(html);
}

/**
 * Creates the end of a rollup area
 */
Rollup.prototype.end = function() {
    document.write("</div>");
}


/**
 * Constructor for the TableRollup class
 * @param name an identifier for this rollup
 * @param link_id the id of the rollup image
 * @param table_id the ID of the table controlled by this object
 * @param expanded true if the rollup is expanded, false otherwise
 * @param rememberState true if the rollup should retain its toggle state between
 * page visits / refreshes
 * @param startRow the first row hidden when the rollup is collapsed (defaults to 1)
 * @param endRow the last row hidden (defaults to the last row in the table)
 */
function TableRollup(name, link_id, table_id, expanded, rememberState, startRow, endRow) {
    this.name = name;
    this.link_id = link_id;
    this.table_id = table_id;
    if (undefined == expanded) {
        expanded = true;
    }
    this.expanded = expanded;
    if (undefined == rememberState) {
        rememberState = false;
    }
    this.memory = rememberState;
    this.startRow = (null != startRow) ? startRow : 1;
    this.endRow = endRow;
}

TableRollup.prototype = new Rollup("", "", "", "", false, false);
document.onmousemove=mouseTrack;
/*
 * Displays or hides the area controlled by the rollup
 */
TableRollup.prototype.show = function() {
    var img, alt;
    var display = "";
    if (this.expanded) {
        // Rolled-down state
        img = this.cStateImage;
        alt = useAlts ? gsCollapse : null;
    }
    else {
        // Rolled-up state
        img = this.eStateImage;
        alt = useAlts ? gsExpand : null;
        display = "none";
    }
    var table = document.getElementById(this.table_id);
    if (table) {
        var rows = table.rows;
        var start = this.startRow;
        var end = (null != this.endRow) ? this.endRow : rows.length;
        for (var i = start; i < end; i++) {
            rows[i].style.display = display;
        }
    }
    var link = document.getElementById(this.link_id);
    if (link) {
        var html = "";
        /*if (!this.expanded && useAlts) {
            html += gsExpand;
        }*/
        if (this.useTextLinks) {
            html += img;
        }
        else {
            html += '<img class="rollup" width="11" ' +
                'height="9" border="0" src="' + img + '"';
			var val = getCookieValue("McDynamic")
			if ( val == "NoIISMoz")
			{
				var fnStrShow = "TooltipControl(true,'" + EscapeForJS(alt) + "');";
				var fnStrVanish = "TooltipControl(false,'" + EscapeForJS(alt) + "');";

				if (alt) {
					html += ' onmouseover ="' + fnStrShow + '"';
					html += ' onmouseout ="' + fnStrVanish + '"';
				}
				html += '/>';
			}
			else
			{
				if (alt) {
					html += ' alt="' + alt + '"';
				}
				html += '/>';
				if (alt) {
					link.title = alt;
				}
			}
        }
        link.innerHTML = html;
    }
}

