/**
 * Settings viewer classes and associated factory class
 *
 * Copyright (C) 2009 McAfee, Inc. All rights reserved
 */

var NBSP = 160;

/**
 * Dummy settings viewer
 */
function BaseSettingsViewer() {
    this.settingsType = "";
}

BaseSettingsViewer.prototype.toString = function() {
    return "Base settings viewer";
}

BaseSettingsViewer.prototype.hasOptions = function() {
    return false;
}

BaseSettingsViewer.prototype.hasActions = function() {
    return false;
}

BaseSettingsViewer.prototype.hasAlertSettings = function() {
    return false;
}

/**
 * Gets scanner configuration details
 * @return an array of arrays, suitable for placing in a table
 */
BaseSettingsViewer.prototype.getOptions = function() {
    return [];
}

/**
 * Gets the preferred height of the container element for the
 * options view; return an empty string for default height
 */
BaseSettingsViewer.prototype.getPreferredHeight = function() {
    return "";
}

/**
 * Base method for returning action data; should work for most
 * settings types
 * @param globalProperties a reference to the global property bag should
 * it be needed
 * @return an array of arrays, suitable for placing in a table
 */
BaseSettingsViewer.prototype.getActions = function(globalProperties) {
    var ret = [];
    var settingsType = this.settingsType;
    var actionList = this.getActionList();
    if (!actionList) {
        return [];
    }
    var ppT = PolSetting_GetPPNode("");
    var pb = globalProperties ? globalProperties : PolSetting_GetPb();

    var actObj = new PolicyAction(pb, ppT, false);
    var nActions = actionList.length;
    // For each action in the action list generate a row of data
    for (var i = 0; i < nActions; i++) {
        var a = [];
        var actName = actionList[i];
        var stKey = "ACT_" + settingsType + "_" + actName;
        a.push(resolveString(cse_pp_st, stKey));
        actObj.BuildTable(actName, i, this.settingsType);
        a.push(actObj.GetVal(i));
        ret.push(a);
    }
    return ret;
}

/**
 * Gets a list of action names associated with a settings type
 */
BaseSettingsViewer.prototype.getActionList = function() {
    var ret = pp_cse_actionNames["STD"];
    if (null != pp_cse_actionNames[this.settingsType])
        ret = pp_cse_actionNames[this.settingsType];

    return ret;
}

BaseSettingsViewer.prototype.getAlertList = function() {
    return [];
}

/**
 * Settings viewer for anti-phish
 */
function PhishSettingsViewer() {
    this.settingsType = "AF";
}

PhishSettingsViewer.prototype = new BaseSettingsViewer();

PhishSettingsViewer.prototype.toString = function() {
    return "Phish settings viewer";
}

/**
 * Determines whether a given settings type has any options
 */
PhishSettingsViewer.prototype.hasOptions = function() {
    return true;
}

/**
 * Determines whether a given settings type has any actions
 */
PhishSettingsViewer.prototype.hasActions = function() {
    return true;
}

/**
 * Gets phish options (references external options_pb object)
 * @return an array of arrays for placing in a table
 */
PhishSettingsViewer.prototype.getOptions = function() {
    var ret = [];
    // Report on subject prefix text and other options
    var pp = PolSetting_GetOptionsPath(this.settingsType);
    var pb = options_pb;
    var pfxSubj = ("1" == pb.GetProperty(pp + ".PrefixSubject"));
    var pfxText = pfxSubj ? this.getAFPrefix(pp, pb) : "";
    var a = [];
    a.push(resolveString(cse_pp_st, "S_PREFIX"));
    var t = pfxSubj ? resolveString(cse_pp_st, "AF_SP_YES", [pfxText]) :
        resolveString(cse_pp_st, "AF_SP_NO");
    a.push(t);
    ret.push(a);
    a = [];
    a.push(resolveString(cse_pp_st, "OTHER_OPTS"));
    var opts = [];
    var b = ("1" == pb.GetProperty(pp + ".PhishIndicator"));
    if (b)
        opts.push(resolveString(cse_pp_st, "PINDICATOR"));
    b = ("1" == pb.GetProperty(pp + ".AttachReport"));
    if (b) {
        opts.push(resolveString(cse_pp_st, "PREPORT"));
        if ("1" == pb.GetProperty(pp + ".VerboseLogging")) {
            opts.push(resolveString(cse_pp_st, "OPT_VERBOSE"));
        }
    }
    if (0 == opts.length)
        opts.push(resolveString(cse_pp_st, "OO_NONE"));
    a.push(opts.join(", "));
    ret.push(a);
    return ret;
}

/**
 * Gets the anti-phishing subject prefix
 */
PhishSettingsViewer.prototype.getAFPrefix = function(pp, pb) {
    var pfxText = pb.GetPropertyDef(pp + ".PrefixSubjectText", "");

    if (!pfxText) {
        try {
            // Get the default prefix text from resources
            var rm = new ResourceManager("DefaultAlerts");
            pfxText = rm.getResource("PhishTokens", "POSSIBLEPHISH");
        }
        catch(e) {
        }
    }

    return pfxText;
}

/**
 * Settings viewer for file-filtering
 */
function SSSettingsViewer() {
    this.settingsType = "SS";
}

SSSettingsViewer.prototype = new BaseSettingsViewer();

SSSettingsViewer.prototype.hasOptions = function() {
    return true;
}

/**
 * Gets scanner control options (references external options_pb object)
 * @return an array of arrays for placing in a table
 */
SSSettingsViewer.prototype.getOptions = function() {
    var ret = [];
    // Report on subject prefix text and other options
    var pp = PolSetting_GetOptionsPath(this.settingsType);
    var pb = options_pb;

    var optNames = ["maxdepth", "maxwidth", "ScannerTimeout"];
    for (var i = 0; i < optNames.length; i++) {
        var a = [];
        var optName = optNames[i];
        var stKey = "OPT_" + optName;
        a.push(resolveString(cse_pp_st, stKey));
        a.push(this.processOptionValue(optName,
            pb.GetPropertyDef(pp + "." + optName, " ")));
        ret.push(a);
    }
    return ret;
}

SSSettingsViewer.prototype.processOptionValue = function(opt, val) {
    var ONE_MINUTE = 60000;
    if ("ScannerTimeout" == opt)
        val = Math.floor((val - 0) / ONE_MINUTE);
    return val.toString();
}

/**
 * Determines whether a given settings type has any actions
 */
SSSettingsViewer.prototype.hasActions = function() {
    return true;
}

SSSettingsViewer.prototype.hasAlertSettings = function() {
    return true;
}

SSSettingsViewer.prototype.getAlertList = function() {
    return [{prop:"Alert", cat:"DOS_PROTECTION_MSG"}];
}

/**
 * Settings viewer for mail-size filtering
 */
function MFSettingsViewer() {
    this.settingsType = "MF";
}

MFSettingsViewer.prototype = new BaseSettingsViewer();

MFSettingsViewer.prototype.hasOptions = function() {
    return true;
}

/**
 * Gets scanner control options (references external options_pb object)
 * @return an array of arrays for placing in a table
 */
MFSettingsViewer.prototype.getOptions = function() {
    var ret = [];
    // Report on subject prefix text and other options
    var pp = PolSetting_GetOptionsPath(this.settingsType);
    var pb = options_pb;

    var optNames = ["MaxSize", "BiggerThanVal", "MoreThanVal"];
    for (var i = 0; i < optNames.length; i++) {
        var a = [];
        var optName = optNames[i];
        var stKey = "OPT_" + optName;
        a.push(resolveString(cse_pp_st, stKey));
        a.push(pb.GetPropertyDef(pp + "." + optName, top.defValues["MF"][optName]));
        ret.push(a);
    }
    return ret;
}

/**
 * Determines whether a given settings type has any actions
 */
MFSettingsViewer.prototype.hasActions = function() {
    return true;
}

/**
 * Settings viewer for alert settings
 */
function ALERTSettingsViewer() {
    this.settingsType = "ALERT";
}

ALERTSettingsViewer.prototype = new BaseSettingsViewer();

ALERTSettingsViewer.prototype.hasOptions = function() {
    return true;
}

/**
 * Gets alert options (references external options_pb object)
 * @return an array of arrays for placing in a table
 */
ALERTSettingsViewer.prototype.getOptions = function() {
    var ret = [];
    // Report on alert format and other options
    var pp = PolSetting_GetOptionsPath(this.settingsType);
    var pb = options_pb;

    var optName = "PlainText";
    var fmt = pb.GetPropertyDef(pp + "." + optName, "0");
    var stKey = ("0" == fmt) ? "__html" : "__plain_text";
    ret.push([resolveString(cse_pp_st, "OPT_" + optName),
        resolveString(cse_pp_st, stKey)]);

    optName = "Charset";
    var defCS = ("0" == fmt) ? "NCR" : "unicode";
    var stKey = pb.GetPropertyDef(pp + "." + optName, defCS);
    ret.push([resolveString(cse_pp_st, "OPT_" + optName),
        resolveString(cse_pp_st, stKey)]);

    optName = "ASCIIFILENAME";
    var defFN = ("0" == fmt) ? pp_cse_defAlertFN : pp_cse_defAlertFNP;
    var aFN = pb.GetPropertyDef(pp + "." + optName, defFN);
    if ("" == aFN)
        aFN = defFN;
    ret.push([resolveString(cse_pp_st, "OPT_" + optName), aFN]);

    var optNames = ["ALERTHEADER", "ALERTFOOTER"];
    for (var i = 0; i < optNames.length; i++) {
        optName = optNames[i];
        var txt = pb.GetPropertyDef(pp + "." + optName, "");
        var stKey = ("" == txt) ? "__default" : "__custom";
        stKey += "_" + optName;
        ret.push([resolveString(cse_pp_st, "OPT_" + optName),
            resolveString(cse_pp_st, stKey)]);
    }

    return ret;
}

/**
 * Settings viewer for disclaimer text
 */
function DISCSettingsViewer() {
    this.settingsType = "DISC";
}

DISCSettingsViewer.prototype = new BaseSettingsViewer();

DISCSettingsViewer.prototype.hasOptions = function() {
    return true;
}

/**
 * Gets disclaimer options (references external options_pb object)
 * @return an array of arrays for placing in a table
 */
DISCSettingsViewer.prototype.getOptions = function() {
    var ret = [];
    // Report on disclaimer text and other options
    var pp = PolSetting_GetOptionsPath(this.settingsType);
    var pb = options_pb;

    /*var optName = "DisclaimerText";
    var txt = pb.GetPropertyDef(pp + "." + optName, "");
    var stKey = ("" == txt) ? "__none" : "__custom";
    ret.push([resolveString(cse_pp_st, "OPT_" + optName),
        resolveString(cse_pp_st, stKey)]);*/
		
	var optName = "UseHTML";
    var fmt = pb.GetPropertyDef(pp + "." + optName, "0");
    var stKey = ("0" == fmt) ?  "__plain_text" : "__html";
    ret.push([resolveString(cse_pp_st, "OPT_" + optName),
        resolveString(cse_pp_st, stKey)]);

	optName = "DisclaimerType";
	var defDType = top.defValues["DISC"][optName];
	var dType = pb.GetPropertyDef(pp + "." + optName, defDType);
	stKey = optName + dType;
	ret.push([resolveString(cse_pp_st, "OPT_" + optName),
		resolveString(cse_pp_st, stKey)]);

    return ret;
}

/**
 * Settings viewer class for signed, encrypted, protocted and corrupt content
 * @param settingsType on of SM, EM, PM or CM
 */
function SEPCSettingsViewer(settingsType) {
    this.settingsType = settingsType;
}

SEPCSettingsViewer.prototype = new BaseSettingsViewer();

/**
 * Determines whether a given settings type has any actions
 */
SEPCSettingsViewer.prototype.hasActions = function() {
    return true;
}

/**
 * Settings viewer for mail settings
 */
function MIMESettingsViewer() {
    this.settingsType = "MIME";
}

MIMESettingsViewer.prototype = new BaseSettingsViewer();

MIMESettingsViewer.prototype.hasOptions = function() {
    return true;
}

/**
 * Gets mailoptions (references external options_pb object)
 * @return an array of arrays for placing in a table
 */
MIMESettingsViewer.prototype.getOptions = function() {
    var ret = [];
    // Report on disclaimer text and other options
    var pp = PolSetting_GetOptionsPath(this.settingsType);
    var pb = options_pb;

    var optName = "ModSubjPrefix";
    var ppT = pp.toString().replace("MIME", "GM");
    var txt = pb.GetPropertyDef(ppT + "." + optName, "");
    ret.push([resolveString(cse_pp_st, "OPT_" + optName),
        txt ? txt : resolveString(cse_pp_st, "__none")]);

    // CCW: 3.3.2006 inlining now a hidden option
    /*optName = "InlineReplacementText";
    var optVal = pb.GetPropertyDef(pp + "." + optName,
        top.defValues["MIME"][optName]);
    var stKey = ("1" == optVal) ? "__inline_yes" : "__inline_no";
    ret.push([resolveString(cse_pp_st, "OPT_" + optName),
        resolveString(cse_pp_st, stKey)]);*/

    var span = document.createElement("span");
    span.style.fontWeight = "bold";
    span.appendChild(document.createTextNode(resolveString(cse_pp_st, "__mail_encoding")));
    ret.push([span]);

    var mapOpts = ["CharSetMap", "FallbackCharset"];
    var nMappings = 0;
    for (var i = 0; i < mapOpts.length; i++) {
        ppT = new PropertyPath(pp + "." + mapOpts[i]);
        for (ppT.BeginEnum(); null != pb.GetProperty(ppT); ppT.IncrementEnum()) {
            ++nMappings;
        }
    }
    var stKey = (1 == nMappings) ? "__onemapping" : "__nmappings";
    var mapReport = resolveString(cse_pp_st, stKey, [nMappings]);

    optName = "ReEncodingName";
    var charSet = pb.GetPropertyDef(pp + "." + optName, top.defValues["MIME"][optName]);
    optName = "ReEncodingScheme";
    optVal = pb.GetPropertyDef(pp + "." + optName, top.defValues["MIME"][optName]);
    stKey = "";
    switch(parseInt(optVal)) {
        case 1:
            stKey = "__reencode_charset";
            break;
        case 2:
            stKey = "__reencode_html";
            break;
        case 3:
            stKey = "__reencode_original";
            break;
        default:
            stKey = "__invalid_encoding_value";
            break;
    }
    span = document.createElement("span");
    span.style.marginLeft = "15px";
    span.appendChild(document.createTextNode(resolveString(cse_pp_st, "OPT_" + optName)));
    ret.push([span, resolveString(cse_pp_st, stKey, [mapReport, resolveString(cse_pp_st, charSet)])]);

    optName = "HeaderEncodingCharset";
    charSet = pb.GetPropertyDef(pp + "." + optName, top.defValues["MIME"][optName]);
    optName = "HeaderEncodingFallback";
    var fb = pb.GetPropertyDef(pp + "." + optName, top.defValues["MIME"][optName]);
    var stKey = ("1" == fb) ? "__utf8_on_err" : "__fail_on_err";
    var heErr = resolveString(cse_pp_st, stKey);
    optName = "HeaderEncodingScheme";
    optVal = pb.GetPropertyDef(pp + "." + optName, top.defValues["MIME"][optName]);
    var stKey = "";
    switch(parseInt(optVal)) {
        case 0:
            stKey = "__h_reencode_utf8";
            break;
        case 1:
            stKey = "__h_reencode_original";
            break;
        case 2:
            stKey = "__h_reencode_charset";
            break;
        default:
            stKey = "__invalid_encoding_value";
            break;
    }
    span = document.createElement("span");

    span.style.marginLeft = "15px";
    span.appendChild(document.createTextNode(resolveString(cse_pp_st, "OPT_" + optName)));
    ret.push([span, resolveString(cse_pp_st, stKey, [heErr, resolveString(cse_pp_st, charSet)])]);

    var span = document.createElement("span");
    span.style.fontWeight = "bold";
    span.appendChild(document.createTextNode(resolveString(cse_pp_st, "__msg_corruption")));
    ret.push([span]);

    var cOpts = ["HdrCorruptionCheck", "HdrNULLCheck", "QPCorruptionDetection"];
    for (var i = 0; i < cOpts.length; i++) {
        span = document.createElement("span");
        span.style.marginLeft = "15px";
        optName = cOpts[i];
        span.appendChild(document.createTextNode(resolveString(cse_pp_st, "OPT_" + optName)));
        var optVal = pb.GetPropertyDef(pp + "." + optName, top.defValues["MIME"][optName]);
        var stKey = ("1" == optVal) ? "__treat_as_corrupt" : "__do_not_treat_as_corrupt";
        ret.push([span, resolveString(cse_pp_st, stKey)]);
    }

    // CCW 3.3.2006: max boundaries exceeded treated as DoS is a hidden options
    /*optName = "MaxBoundaries";
    optVal = pb.GetPropertyDef(pp + "." + optName, top.defValues["MIME"][optName]);
    span = document.createElement("span");
    span.style.marginLeft = "15px";
    optName = "MaxBoundariesExceededAsDoS";
    span.appendChild(document.createTextNode(resolveString(cse_pp_st, "OPT_" + optName, [optVal])));
    optVal = pb.GetPropertyDef(pp + "." + optName, top.defValues["MIME"][optName]);
    stKey = ("1" == optVal) ? "__treat_as_corrupt" : "__treat_as_DoS";
    ret.push([span, resolveString(cse_pp_st, stKey)]);*/

    return ret;
}

/**
 * Gets the preferred height of the container element for the options view
 */
MIMESettingsViewer.prototype.getPreferredHeight = function() {
    return "105px";
}

MIMESettingsViewer.prototype.hasActions = function() {
    return true;
}

MIMESettingsViewer.prototype.hasAlertSettings = function() {
    return true;
}

MIMESettingsViewer.prototype.getAlertList = function() {
    return [{prop:"HighRiskAlert", cat:"BLOCKEDMSGTYPE_DESC"}];
}

/**
 * Settings viewer for mail settings
 */
function HTMLSettingsViewer() {
    this.settingsType = "HTML";
}

HTMLSettingsViewer.prototype = new BaseSettingsViewer();

HTMLSettingsViewer.prototype.hasOptions = function() {
    return true;
}

HTMLSettingsViewer.prototype.getOptions = function() {
    var ret = [];

    var pp = PolSetting_GetOptionsPath(this.settingsType).toString();
    var pb = options_pb;

    var scanList = [];
    var opts = ["ScanComments", "ScanMetaData", "ScanLinkURLs",
        "ScanSourceURLs", "ScanScript"];
    for (var i = 0; i < opts.length; i++) {
        var optName = opts[i];
        var optVal = options_pb.GetPropertyDef(pp + "." + opts[i],
            top.defValues["HTML"][optName]);
        if ("1" == optVal)
            scanList.push(resolveString(cse_pp_st, "OPT_" + optName));
    }
    if (0 == scanList.length)
        scanList.push(resolveString(cse_pp_st, "__none"));
    ret.push([resolveString(cse_pp_st, "__scan_elements"), scanList.join(", ")]);

    var stripList = [];
    opts = ["StripScript", "StripApplets", "StripActiveX", "StripFlash"];
    for (var i = 0; i < opts.length; i++) {
        var optName = opts[i];
        var optVal = options_pb.GetPropertyDef(pp + "." + opts[i],
            top.defValues["HTML"][optName]);
        if ("1" == optVal)
            stripList.push(resolveString(cse_pp_st, "OPT_" + optName));
    }
    if (0 == stripList.length)
        stripList.push(resolveString(cse_pp_st, "__none"));
    ret.push([resolveString(cse_pp_st, "__strip_elements"), stripList.join(", ")]);

    return ret;
}
/*Settings viewer for URL Filtering*/
function MailURLSettingsViewer() {
    this.settingsType = "MailURL";
}

MailURLSettingsViewer.prototype = new BaseSettingsViewer();

MailURLSettingsViewer.prototype.hasOptions = function() {
    return true;
}

MailURLSettingsViewer.prototype.hasActions = function() {
    return true;
}

MailURLSettingsViewer.prototype.getOptions = function() {
    var ret = [];

    var pp = PolSetting_GetOptionsPath(this.settingsType).toString();
    var pb = options_pb;
	
	var optNames = ["HighThresholdValue", "LowThresholdValue", "MaxOverallURLLookup"];
    for (var i = 0; i < optNames.length; i++) {
        var a = [];
        var optName = optNames[i];
        var stKey = "OPT_" + optName;
        a.push(resolveString(cse_pp_st, stKey));
        a.push(pb.GetPropertyDef(pp + "." + optName, top.defValues["MailURL"][optName]));
        ret.push(a);
    }
    return ret;    
}
/**
 * Gets a list of action names associated with Mail URL settings type
 */
MailURLSettingsViewer.prototype.getActionList = function() {
    var ret = pp_cse_actionNames["MAILURL"];
    if (null != pp_cse_actionNames[this.settingsType])
        ret = pp_cse_actionNames[this.settingsType];

    return ret;
}
MailURLSettingsViewer.prototype.hasAlertSettings = function() {
    return false;
}

MailURLSettingsViewer.prototype.getAlertList = function() {
    return [{prop:"Alert", cat:"MAILURL_MSG"}];
}

/**
 * Settings viewer for email compliancy
 */
function PostXSettingsViewer() {
    this.settingsType = "PX";
}

PostXSettingsViewer.prototype = new BaseSettingsViewer();

PostXSettingsViewer.prototype.hasOptions = function() {
    return true;
}

// Scanner options for email compliancy
PostXSettingsViewer.prototype.getOptions = function() {
    var ret = [];

    var pp = PolSetting_GetOptionsPath(this.settingsType).toString();

    var HIPAA = 1, PRIVACY = 2;
    var optName = "ScanType";
    var scanType = options_pb.GetPropertyDef(pp + "." + optName,
        top.defValues["PX"][optName]);
    var typeList = [];
    if (scanType & HIPAA)
        typeList.push(resolveString(cse_pp_st, "OPT_ScanType_HIPAA"));
    if (scanType & PRIVACY)
        typeList.push(resolveString(cse_pp_st, "OPT_ScanType_Privacy"));
    var stKey = (1 == typeList.length) ? "OPT_ScanType_1" : "OPT_ScanType_MT1";
    ret.push([resolveString(cse_pp_st, stKey), typeList.join(", ")]);

    optName = "ThresholdHIPAA";
    var thresholdH = options_pb.GetPropertyDef(pp + "." + optName,
        top.defValues["PX"][optName]);
    optName = "ThresholdPrivacy";
    var thresholdP = options_pb.GetPropertyDef(pp + "." + optName,
        top.defValues["PX"][optName]);
    if (scanType & HIPAA)
        ret.push([resolveString(cse_pp_st, "OPT_ThresholdHIPAA"), thresholdH]);
    if (scanType & PRIVACY)
        ret.push([resolveString(cse_pp_st, "OPT_ThresholdPrivacy"), thresholdP]);

    return ret;
}

PostXSettingsViewer.prototype.hasAlertSettings = function() {
    return true;
}

PostXSettingsViewer.prototype.getAlertList = function() {
    return [{prop:"Alert", cat:"POSTX_MSG"}];
}

PostXSettingsViewer.prototype.hasActions = function() {
    return true;
}

// Factory code for returning settings viewer instances
var g_bsv = new BaseSettingsViewer();
var svMap = {"AF":new PhishSettingsViewer(),
    "SS":new SSSettingsViewer(),
    "SM":new SEPCSettingsViewer("SM"),
    "EM":new SEPCSettingsViewer("EM"),
    "PM":new SEPCSettingsViewer("PM"),
    "CM":new SEPCSettingsViewer("CM"),
    "PPF":new SEPCSettingsViewer("PPF"),
    "ALERT":new ALERTSettingsViewer(),
    "DISC":new DISCSettingsViewer(),
	"MailURL":new MailURLSettingsViewer(),
    "MF":new MFSettingsViewer(),
    "MIME":new MIMESettingsViewer(),
    "HTML":new HTMLSettingsViewer(),
    "PX":new PostXSettingsViewer()};

function SettingsViewerFactory() {
}

SettingsViewerFactory.getViewer = function(settingsType) {
    if (null != svMap[settingsType])
        return svMap[settingsType];

    return g_bsv;
}

