/**
 * Shared JavaScript for the settings pages; this file needs to be included
 * before any of the XXSettings.js files.
 *
 * Copyright (C) 2009 McAfee, Inc. All rights reserved
 */

/**
 * Fixes up the input property bag and ensures that it is in a correct state
 * @param pbIn the input property bag
 * @param pp the property path for the page
 * @param settingsType the settings type for the current page
 * @return true if a new settings instance is created
 */
function shs_initPB(pbIn, pp, settingsType) {
    var newInstance = false;
    // Check that the settings properties and policy tree (for reference counting) are loaded
    if (null == pbIn.GetProperty("Settings")) {
        // Copy the Settings node into the property bag that we've been passed
        top.gPB.GetElement("Settings", pbIn);
    }
    if (null == pbIn.GetProperty("Policies")) {
        top.gPB.GetElement("Policies", pbIn);
    }
    if (null != pp) {
        // Check for presence of property path in property bag
        if (undefined === pbIn.GetProperty(pp)) {
            // New instance
            // Generate a random ID
            newId = MakeGuid(settingsType + "S");
            var ppT = PropertyPath.MakePP(pp);
            pbIn.SetProperty(ppT);
            ppT.SetTempLeaf("id");
            pbIn.SetProperty(ppT, newId);
            ppT.SetTempLeaf("");
            // See whether there are dependent settings and create new instances of these
            try {
                var dependencyList = pp_cse_settingsDependencies[settingsType];
                for (var i = dependencyList.length - 1; i >= 0; i--) {
                    var dType = dependencyList[i];
                    var dsId = newId.replace(settingsType, dType);
                    var dsPP = pp.toString().replace(settingsType, dType);
                    pbIn.SetProperty(dsPP);
                    pbIn.SetProperty(dsPP + ".id", dsId);
                    pbIn.SetProperty(dsPP + ".root", "0");
                }
            }
            catch(e) {}
            newInstance = true;
        }
    }
    return newInstance;
}

/**
 * Support function for dependent settings.  Given a field name, checks
 * whether the field uses the normal property path and, if not, returns
 * the path to use
 * @param baseSettingsType the default settings type for the current page
 * @param db a data binding obejct for a field
 * @param fieldOverrides a map of field name: settings type mappings
 * @param pp the candidate property path (which may be overridden
 * @return the property path to use
 */
function getPPToUse(baseSettingsType, db, fieldOverrides, pp) {
    var ret = pp;
    var fieldName = db.element ? db.element : (db.propName ? db.propName : null);
    if (fieldName) {
        var stOverride = fieldOverrides[fieldName];
        if (null != stOverride) {
            ret = new PropertyPath(pp.toString().replace(baseSettingsType, stOverride));
        }
    }
    return ret;
}

/**
 * Asynchronous data validation function
 * @param tabset the tabset for the current page; may be null
 */
function shs_Validate(tabset) {
    var ok = true;

    var useTabs = (undefined !== tabset);
    if (useTabs) {
        if (vframeworks["InstanceSettings"]) {
            ok = vframeworks["InstanceSettings"].onsubmit();
        }
        if (ok) {
            var vfList = tabset.getValidationList(tabset.getCurrent());
            for (var i = 0; i < vfList.length; i++) {
                ok = vfList[i].onsubmit();
                if (!ok) break;
            }
        }
    }
    else {
        for (var vf in vframeworks) {
            ok = vframeworks[vf].onsubmit();
            if (!ok) {
                break;
            }
        }
    }

    return ok;
}

/**
 * Asynchronous data save function
 * @param pb the property bag to fill
 * @param basePP the base property path to use
 * @param newInstance true if this is a new settings instance
 * @param dataBindings an array of DataBinding objects for retrieving
 * the settings
 * @param _changed (optional) true if the settings are already known to have
 * changed, false otherwise
 * @param baseSettingsType (optional) the default settings type for the current page
 * @param fieldOverrides (optional) a map of field name: settings type mappings
 * @return true if the settings have changed, false otherwise
 */
function shs_Apply(pb, basePP, dataBindings, newInstance, _changed,
    baseSettingsType, fieldOverrides) {
    var changed = (_changed != null) ? _changed : false;
    // Update the settings
    var ppT = new PropertyPath(basePP);
    for (var i = 0; i < dataBindings.length; i++) {
        var ppToUse = ppT;
        var db = dataBindings[i];
        if (baseSettingsType && fieldOverrides) {
            ppToUse = getPPToUse(baseSettingsType, db, fieldOverrides, ppT);
        }
        if (db.get(ppToUse, pb)) {
            changed = true;
        }
    }

    if (changed && newInstance) {
        // Notify any interested caller about the new instance
        if (top.polctx) {
            top.polctx.newInstance = newId;
        }
    }
    top.gPageStack.Active().SetModified(changed);
    return changed;
}

/**
 * Hides / shows button bar
 */
function shs_showSave(show) {
    var f = top.frames["frame_buttons"];
    var div;
    if (f)
        div = f.document.getElementById("configButtons");
    if (div)
        div.style.display = show ? "" : "none";
}

/**
 * Reports on affected policies
 */
function shs_affectedPolicyReport(newInstance, settingsType, id, pb) {
    if (document.getElementById("ap_report_area")) {
        if (!newInstance) {
            var refCounter = RefCounter.getRefCount(settingsType, id, pb,
                RefCounter.getSettingsRefCount, null);
            if (0 != refCounter.getDependentPolicyCount())
                refCounter.briefReport("ap_report_area");
        }
    }
}

/**
 * Default cancel operation
 */
function shs_cancel(newInstance, pb, basePP, baseSettingsType) {
    if (newInstance) {
        // Remove any newly created settings blobs
        pb.DeleteElement(basePP);
        if (baseSettingsType) {
            // Remove any newly created dependent settings blobs
            var dsList = pp_cse_settingsDependencies[baseSettingsType];
            for (var i = dsList.length - 1; i >= 0; i--) {
                var pp = new PropertyPath(basePP.toString().replace(baseSettingsType, dsList[i]));
                pb.DeleteElement(pp);
            }
        }
    }
}
