/*
 * Tabs.js
 * Class definition and helper functions for creating a tabbed interface.
 * Depends on the ValidationFramework JavaScript files
 *
 * Copyright (C) 2009 McAfee, Inc. All rights reserved.
 */

/**
 * Constructor for the TabSet class
 * @param objectName the declared name of the TabSet object; this is
 * required for building up URLs on the tabs themseleves
 * @param rememberSelection true if you want to the TabSet to remember its
 * selection when the page is revisited
 * @param classPfx a prefix used apply different styles to the tabs by modifying
 * the class names
 * elements to hold the tabs
 */
function TabSet(objectName, rememberSelection, classPfx) {
    initTabSet(objectName, rememberSelection, this, classPfx);
}

function initTabSet(objectName, rememberSelection, tabSetObj, classPfx) {
    tabSetObj.objectName = objectName;
    tabSetObj.containerName = objectName + "_container";
    tabSetObj.tabs = new Array();
    tabSetObj.vframeworks = new Array();
    tabSetObj.current = -1;
    if (rememberSelection) {
        tabSetObj.memory = true;
        // Create a tab register in the parent if it does not already exist
        if (undefined == parent.tab_register) {
            parent.tab_register = new Object();
        }
        // Initialize the register for tabSetObj tabset
        if (undefined == parent.tab_register[objectName]) {
            parent.tab_register[objectName] = 0;
        }
    }
    else {
        tabSetObj.memory = false;
    }
    tabSetObj.classPfx = (null != classPfx) ? classPfx : "";
}

/**
 * Gets the number of tabs in the tabset
 */
TabSet.prototype.length = function() {
    return this.tabs.length;
}

/**
 * Adds a tab to the tabset
 * @param label the label of the tab
 * @param div the <div> block that is displayed or hidden when the tab is
 * clicked
 * @param customActions a list of additional actions to take when the tab
 * is selected
 */
TabSet.prototype.add = function(label, div, customActions) {
    this.tabs.push(new Tab(label, div, customActions, this.tabs.length));
    this.vframeworks[this.vframeworks.length] = new Array();
}

/**
 * Gets a reference to a tab
 * @param idx the index into the tabset
 */
TabSet.prototype.get = function(idx) {
    if (idx < 0 || idx >= this.tabs.length) {
        throw new RangeError(idx);
    }
    return this.tabs[idx];
}

/**
 * Creates the HTML to make a tab container
 */
TabSet.prototype.makeContainer = function() {
    this.makeContainer2("tabsContainer");
}

/**
 * Creates the HTML to make a tab container
 * @param className the class of the tabs container
 */
TabSet.prototype.makeContainer2 = function(className) {
    var html = "<div";
    if (className) {
        html += " class=\"" + this.classPfx+className + "\"";
    }
    html += " id=\"" + this.containerName + "\"></div>";
    document.write(html);
}

/**
 * Draws a set of tabs and places them in a container
 * @selection the number of the selected tab
 */
TabSet.prototype.drawTabs = function(selection) {
    var tCount = this.tabs.length;
    var html = '<table border="0" cellpadding="0" cellspacing="0" class="' +
        this.classPfx + 'tabs"><tr>';
    html += '<td class="indent"></td>';
    for (var i = 0; i < tCount; i++) {
        var selected = false;
        if (i == selection) {
            selected = true;
        }
        var tab = this.tabs[i];
        html += tab.renderSelf(this.objectName, i, selected, selection, tCount);
    }
    html += '</tr></table>';

    var tab_container = document.getElementById(this.containerName);
    if (tab_container) {
        tab_container.innerHTML = html;
    }

    this.current = selection;
}

/**
 * Handles a user clicking a tab
 * @selection the number of the selected tab
 */
TabSet.prototype.handler = function(selection) {
    if (-1 != this.current) {
        // Perform any necessary form validation
        var vfList = this.vframeworks[this.current];
        var ok = true;
        if (vfList) {
            for (var i = 0; i < vfList.length; i++) {
              ok = vfList[i].onsubmit();
              if (!ok) break;
            }
        }
        if (this.ValidateCallback)
            ok = this.ValidateCallback(this.current);

        if (!ok) return;
    }
    if (undefined == selection) {
        selection = -1;
    }
    if (-1 == selection) {
        if (this.memory) {
            // Use the previous selection
            selection = parent.tab_register[this.objectName];
        }
        else {
            selection = 0;
        }
    }
    this.drawTabs(selection);
    // Hide or display elements on the page according to which tab
    // has been clicked
    var tCount = this.tabs.length;
    for (var i = 0; i < tCount; i++) {
        var tab = this.tabs[i];
        tab.action(i == selection);
    }

    if (this.memory) {
        // Remember the selection
        parent.tab_register[this.objectName] = selection;
    }
}

/**
 * Gets the index of the first selectable tab in a tabset; if no tab is selectable,
 * the method will throw (so don't disable all the tabs!)
 */
TabSet.prototype.getDefault = function() {
    var ret = -1;
    var tCount = this.tabs.length;
    for (var i = 0; i < tCount; i++) {
        var tab = this.tabs[i];
        if (tab.enabled) {
            ret = i;
            break;
        }
    }
    if (-1 == ret) {
        throw new Error("No tab is selectable");
    }
    return ret;
}

/**
 * Gets the last selected tab; if the TabSet has not been initialized to
 * remember its selections, this will always return 0
 */
TabSet.prototype.getLastSelected = function() {
    var ret = 0;
    if (this.memory) {
        ret = parent.tab_register[this.objectName];
    }
    return ret;
}

/**
 * Adds form validation to a tab
 * @param index - the index into the tabset
 * @param vframework - a validation framework object
 */
TabSet.prototype.addValidation = function(index, vframework) {
    if (index < this.vframeworks.length) {
        this.vframeworks[index].push(vframework);
    }
}

/**
 * Gets the current selection
 * @return the current selection
 */
TabSet.prototype.getCurrent = function() {
    return this.current;
}

/**
 * Gets a validation list for the specified tab
 * @param index the index into the tabset
 * @return a list of ValidationFramework objects or empty list if index is OOB
 */
TabSet.prototype.getValidationList = function(index) {
    var ret = [];
    if (index >= 0 && index < this.vframeworks.length) {
        ret = this.vframeworks[index];
    }
    return ret;
}

/**
 * Clears the selection register for this tabset
 */
TabSet.prototype.clearSelection = function() {
    if (undefined != parent.tab_register) {
        parent.tab_register[this.objectName] = undefined;
    }
}

/**
 * Constructor for the LinkBar object - works like a tabset but draws links instead
 * of tabs
 * @param objectName the declared name of the TabSet object; this is
 * required for building up URLs on the tabs themseleves
 * @param rememberSelection true if you want to the TabSet to remember its
 * selection when the page is revisited
 * @param bold true if the links should be rendered in bold, false otherwise.
 */
function LinkBar(objectName, rememberSelection, bold) {
    initTabSet(objectName, rememberSelection, this);
    this.bold = (null != bold) ? bold : false;
}

LinkBar.prototype = new TabSet("", false);

/**
 * Creates the HTML to make a linkbar container
 */
LinkBar.prototype.makeContainer = function() {
    var html = '<div id="' + this.containerName + '" style="display:inline;"';
    if (this.bold) {
        html += ' class="bold"';
    }
    html += '></div>';
    document.write(html);
}

/**
 * Adds a link to the linkbar
 * @param label the label of the link
 * @param div the <div> block that is displayed or hidden when the tab is
 * clicked
 * @param customActions a list of additional actions to take when the tab
 * is selected
 */
LinkBar.prototype.add = function(label, div, customActions) {
    this.tabs.push(new Link(label, div, customActions, this.tabs.length));
    this.vframeworks[this.vframeworks.length] = new Array();
}

/**
 * Adds a TableLink object to the linkbar
 * @param label the label of the link
 * @param table the table object controlled by the links
 * @param tbody the tbody to display when the link is selected
 * @param customActions a list of additional actions to take when the tab
 * is selected
 */
LinkBar.prototype.t_add = function(label, table, tbody, customActions) {
    this.tabs.push(new TableLink(label, table, tbody, customActions,
        this.tabs.length));
    this.vframeworks[this.vframeworks.length] = new Array();
}


/**
 * Draws a set of links and places them in a container
 * @selection the number of the selected tab
 */
LinkBar.prototype.drawTabs = function(selection) {
    var tCount = this.tabs.length;
    var links = [];
    for (var i = 0; i < tCount; i++) {
        var selected = false;
        if (i == selection) {
            selected = true;
        }
        var link = this.tabs[i];
        links.push(link.renderSelf(this.objectName, i, selected, selection, tCount));
    }

    var html = links.join(" | ");

    var link_container = document.getElementById(this.containerName);
    if (link_container) {
        link_container.innerHTML = html;
    }

    this.current = selection;
}

/**
 * Constructor for the tab class
 * @param label the label of the tab
 * @param div the <div> block that is displayed or hidden when the tab is clicked
 * @param customActions a list of additional actions to take when the tab is selected
 * @param index the index of the tab in the array
 * @param enabled true if the tab is to be clickable (defaults to true)
 */
function Tab(label, div, customActions, index, enabled) {
    initTab(label, div, customActions, index, enabled, this);
}

function initTab(label, div, customActions, index, enabled, tabObj) {
    tabObj.label = label;
    tabObj.div = div;
    tabObj.customActions = customActions;
    tabObj.index = index;
    tabObj.enabled = (null == enabled) ? true : enabled;
    tabObj.started = false;
}

/**
 * Enables / disables a tab
 */
Tab.prototype.setEnabled = function(enabled) {
    this.enabled = (null == enabled) ? true : enabled;
}

/**
 * Renders a tab as a string of HTML; if the tab is selected while disabled, the
 * method will throw, so don't do it
 * @param tabset the name of the TabSet object holding the tab
 * @param order the order of the tab in a tabset
 * @param selected true if the tab is selcted, false otherwise
 * @param selectedIdx the order of the selected tab
 * @param tCount the number of tabs in a tabset
 * @return a string of HTML to render the tab
 */
Tab.prototype.renderSelf = function(tabset, order, selected, selectedIdx, tCount) {
    var ret = "";
    if (0 == order) {
        ret += "<td class=\"leftEdgeOn\"></td>";
    }
    if (selected) {
        if (!this.enabled) {
            throw new Error("Unable to select a disabled tab");
        }
        ret += "<td nowrap class=\"labelOn\">" + this.label + "</td>";
    }
    else {
        var className = "labelOff";
        if (!this.enabled) {
            className += " disabled";
        }
        ret += "<td nowrap class=\"" + className + "\">";
        if (this.enabled) {
            ret += "<a href=\"javascript:" + tabset +
            ".handler(" + this.index + ");";
            if (this.customActions) {
                for (var i = 0; i < this.customActions.length; i++) {
                    ret += " " + this.customActions[i] + ";";
                }
            }
            ret += "\">";
        }
        ret += this.label;
        if (this.enabled)
            ret += "</a>";
        ret += "</td>";
    }
    var tCl = "";
    if (order == tCount - 1) {
        if (selected) {
            tCl = "rightEdgeOn";
        }
        else {
            tCl = "rightEdgeOff";

        }
    }
    else {
        if (selected) {
            tCl = "separatorOnOff";
        }
        else if (order == selectedIdx - 1) {
            tCl = "separatorOffOn";
        }
        else {
            tCl = "separatorOffOff";
        }
    }
    ret += "<td class=\"" + tCl + "\"></td>\n";
    return ret;
}

/**
 * Creates an area on the page controlled by a tab
 */
Tab.prototype.start = function() {
    // Check the started flag to protect against the start method being
    // called twice
    if (!this.started) {
        document.write("<div id=\"" + this.div + "\" style=\"display:inline\">");
        this.started = true;
    }
}

/**
 * Ends an area on the page controlled by a tab
 */
Tab.prototype.end = function() {
    if (this.started) {
        document.write("</div>");
        this.started = false;
    }
}

/**
 * Hides or shows the div controlled by the tab
 * @param true if the tab has been selected, false otherwise
 */
Tab.prototype.action = function(selected) {
    var elem = document.getElementById(this.div);
    if (elem) {
        elem.style.display = selected ? "inline" : "none";
    }
}

/**
 * Constructor for the Link class
 * @param label the text of the link
 * @param div the <div> block that is displayed or hidden when the link is clicked
 * @param customActions a list of additional actions to take when the link is selected
 * @param index the index of the link in the array
 * @param enabled true if the link is to be clickable (default is true)
 */
function Link(label, div, customActions, index, enabled) {
    initTab(label, div, customActions, index, enabled, this);
}

Link.prototype = new Tab("", "", [], 0);

/**
 * Renders a link as a string of HTML
 * @param linkbar the name of the LinkBar object holding the tab
 * @param order the order of the link in a linkbar
 * @param selected true if the link is selcted, false otherwise
 * @param selectedIdx the order of the selected link
 * @param lCount the number of links in a linkbar
 * @return a string of HTML to render the tab
 */
Link.prototype.renderSelf = function(linkbar, order, selected, selectedIdx, lCount) {
    var ret = "";

    if (selected) {
        if (!this.enabled) {
            throw new Error("Unable to select a disabled tab");
        }
        ret += "<img width=\"4\" height=\"7\" border=\"0\" src=\"../../images/bullet_next.gif\">";
        ret += "&nbsp;" + this.label;
    }
    else if (!this.enabled) {
        ret += "<span class=\"disabled\">" + this.label + "</span>";
    }
    else {
        ret += "<a class=\"blueLink\" href=\"javascript:" + linkbar + ".handler(" + this.index + ");";
        if (this.customActions) {
            for (var i = 0; i < this.customActions.length; i++) {
            ret += " " + this.customActions[i] + ";";
            }
        }
        ret += "\">";
        ret += this.label + "</a>";
    }

    return ret;
}

/**
 * Alternative link class that shows and hides row on a table rather than
 * swapping divs.
 * @param label the text of the link
 * @param table the ID of the table object controlled by the linkbar
 * @param tbody the index of the tbody to display when the tab is selected
 * @param customActions a list of additional actions to take when the link is selected
 * @param index the index of the link in the array
 * @param enabled true if the link is to be clickable
 */
function TableLink(label, table, tbody, customActions, index, enabled) {
    initTab(label, "", customActions, index, enabled, this);
    this.table = table;
    this.tbody = tbody;
}

TableLink.prototype = new Link("", "", [], 0);

/**
 * Hides or shows the table rows controlled by the link
 * @param true if the link has been selected, false otherwise
 */
TableLink.prototype.action = function(selected) {
    var table = document.getElementById(this.table);
    if (table) {
        var tbs = table.tBodies;
        if (this.tbody >= 0 && this.tbody < tbs.length) {
            tbs[this.tbody].style.display = selected ? "" : "none";
        }
    }
}

/**
 * Null functions to match the superclass's API
 */
TableLink.prototype.start = function() {
}
TableLink.prototype.end = function() {
}

