/**
 * JavaScript Form Validation Framework
 * Copyright (C) 2016 McAfee, Inc. All rights reserved
 */

// Note to developers:
// include <lang>/vframework_strings.js before you include this file

/***** Validator Class *****/

/**
 * Dummy validator object; root of the object hierarchy
 */
function Validator() {
    this.name = "Validator";
}

/**
 * Dummy validate method - always returns true
 */
Validator.prototype.validate = function(value) {
    return true;
}

/**
 * Dummy errorMessage method - should never be called
 */
Validator.prototype.errorMessage = function() {
    return "?";
}

Validator.prototype.toString = function() {
    return this.name;
}

/***** RegExpValidator Class *****/

/**
 * Validates fields against a regular expression
 * @param re a regular expression literal that serves as the test pattern
 * @param errMsg a value to return as the object's error message
 * @throws SyntaxError - re is a string literal and not a legal regular expression
 */
function RegExpValidator(re, errMsg) {
    if (re instanceof RegExp) {
        this.re = re;
    }
    else {
        this.re = new RegExp(re);
    }
    this.errMsg = errMsg;
    this.name = "RegExpValidator";
}

RegExpValidator.prototype = new Validator();

/**
 * Validate method - returns true if the value matches a specific pattern,
 * false otherwise
 */
RegExpValidator.prototype.validate = function(value) {
    if (this.re.test(value)) {
        return true;
    }
    return false;
}

/**
 * Returns the error message specified in the constructor
 */
RegExpValidator.prototype.errorMessage = function() {
    return this.errMsg;
}

/*** WildCardValidator Class ***/

/**
 * Validates fields for the presence of wildcards
 * @param re a regular expression literal that serves as the test pattern
 * @param errMsg a value to return as the object's error message
 * @throws SyntaxError - re is a string literal and not a legal regular expression
 */
function WildCardValidator(form_name) {
    this.name = "WildCardValidator";
    this.form_name = form_name;
}

WildCardValidator.prototype = new Validator();

/**
 * Validate method - returns true if the value matches a specific pattern,
 * false otherwise
 */
WildCardValidator.prototype.validate = function(value) {
    var form = document.forms[this.form_name]
	//wildcard checked
	if(form.wildcard.checked)
	 if(value.indexOf("*")==-1 && value.indexOf("?")==-1)
		return false;

	return true;
}

/**
 * Returns the error message specified in the constructor
 */
WildCardValidator.prototype.errorMessage = function() {
    return gsWildCardRequired;
}
/***** RequiredValidator Class *****/

/**
 * Validator object for compulsory fields
 */
function RequiredValidator() {
    this.name = "RequiredValidator";
}

RequiredValidator.prototype = new RegExpValidator(/\S+/, vfr_resolveString("value_required"));

/***** NumericValidator Class *****/

/**
 * Checks that a field is numeric
 */
function NumericValidator() {
    this.name = "NumericValidator";
}

// Number is an optional sign followed by at least one digit and nothing else
NumericValidator.prototype = new RegExpValidator(/^[+-]{0,1}\d+$/,
    vfr_resolveString("integer_required"));

/***** FloatValidator Class *****/

/**
 * Checks that a field is numeric
 */
function FloatValidator() {
    this.name = "FloatValidator";
}

// Number is an optional sign followed by at least one digit and nothing else
FloatValidator.prototype = new RegExpValidator(/^[+-]{0,1}\d*.{0,1}\d+$/,
    vfr_resolveString("integer_required"));


/***** UserName Class *****/

/**
 * Checks that a field is a valid username
 */
function UserNameValidator() {
    this.name = "UserNameValidator";
}

UserNameValidator.prototype = new RegExpValidator(/^[a-zA-Z0-9]+([._-][a-zA-Z0-9]+)*$/,
    vfr_resolveString("invalid_username"));

/***** RangeValidator Class *****/

/**
 * Checks that a field is a number within a certain range
 * @param min the minimum value
 * @param max the maxmimum value
 */
function RangeValidator(min, max) {
    this.min = min;
    this.max = max;
    this.name = "RangeValidator";
}

RangeValidator.prototype = new NumericValidator();

/**
 * Validate method - returns true if the value lies between two values,
 * false otherwise
 */
RangeValidator.prototype.validate = function(value) {
    if (!this.re.test(value)) {
        return false;
    }
    var v = parseInt(value, 10);
    if (v < this.min || v > this.max) {
        return false;
    }
    return true;
}

/**
 * Returns an "invalid range" error message
 */
RangeValidator.prototype.errorMessage = function() {
    return vfr_resolveString("integer_invalid_range", [this.min, this.max]);
}

/***** RangeFloatValidator Class *****/

/**
 * Checks that a field is a number within a certain range
 * @param min the minimum value
 * @param max the maxmimum value
 */
function RangeFloatValidator(min, max) {
    this.min = min;
    this.max = max;
    this.name = "RangeFloatValidator";
}

RangeFloatValidator.prototype = new FloatValidator();

/**
 * Validate method - returns true if the value lies between two values,
 * false otherwise
 */
RangeFloatValidator.prototype.validate = function(value) {
    if (!this.re.test(value)) {
        return false;
    }
    var v = parseFloat(value);
    if (v < this.min || v > this.max) {
        return false;
    }
    return true;
}

/**
 * Returns an "invalid range" error message
 */
RangeFloatValidator.prototype.errorMessage = function() {
    return vfr_resolveString("integer_invalid_range", [this.min, this.max]);
}

/***** GTEValidator Class *****/

/**
 * Checks that a field is >= a given number
 */
function GTEValidator(bound) {
    this.bound = (null == bound) ? 0 : bound
    this.name = "GTEValidator";
}

GTEValidator.prototype = new NumericValidator();

/**
 * Validate method - returns true if the value is
 * greater than a given number, false otherwise
 */
GTEValidator.prototype.validate = function(value) {
    if (!this.re.test(value)) {
        return false;
    }
    var v = parseInt(value, 10);
    if (v >= this.bound) {
        return true;
    }
    return false;
}

/**
 * Returns an "invalid range" error message
 */
GTEValidator.prototype.errorMessage = function() {
    return vfr_resolveString("lterr_int", [this.bound]);
}


/***** FloatValidator *****/

/**
 * Checks that a field is a floating point number; note that this validator
 * does not handle exponential format, just decimals
 */
function FloatValidator() {
    this.name = "FloatValidator";
}

// Number is an optional sign followed by zero or more digits, followed by
// a possible decimal point and decimal digits.
function makeFloatPattern() {
    var pat = "^[+-]{0,1}";	// Optional sign
    pat += "\\d*";		// Zero or more digits
    pat += "("			// Start of decimal part
    var ds = (this.locale_ds ? this.locale_ds : '.');
    if ('.' == ds) {
        // Escape the period
        ds = "\\.";
    }
    pat += ds;			// Decimal separator
    pat += "\\d*)";		// Digits following the decimal separator
    pat += "?$";		// Decimal part optional; end of string
    return pat;
}

FloatValidator.prototype = new RegExpValidator(makeFloatPattern(),
    vfr_resolveString("number_required"));

FloatValidator.prototype.validate_a = FloatValidator.prototype.validate;

FloatValidator.prototype.validate = function(value) {
    var ret = this.validate_a(value);
    if (ret) {
        // -. matches the regex so we need to check whether the input is a legit number
        var f = parseFloat(value);
        ret = isFinite(f);
    }
    return ret;
}

/***** FloatRangeValidator Class *****/

/**
 * Checks that a field is a number within a certain range
 * @param min the minimum value
 * @param max the maxmimum value
 * @param dp the number of decimal points of precision
 */
function FloatRangeValidator(min, max, dp) {
    this.min = min;
    this.max = max;
    this.dp = (null == dp) ? 2 : dp;
    this.name = "FloatRangeValidator";
}

FloatRangeValidator.prototype = new FloatValidator();

/**
 * Validate method - returns true if the value lies between two values,
 * false otherwise
 */
FloatRangeValidator.prototype.validate = function(value) {
    if (!this.re.test(value)) {
        return false;
    }
    var v = parseFloat(value).toFixed(this.dp);
    if (v < this.min || v > this.max) {
        return false;
    }
    return true;
}

/**
 * Returns an "invalid range" error message
 */
FloatRangeValidator.prototype.errorMessage = function() {
    return vfr_resolveString("invalid_range", [this.min, this.max]);
}


/***** CrossValidator Class *****/

/**
 * "Private" class that links a rule with an error message
 */
function _Rule2Message(rule, errorMessage) {
    this.rule = rule;
    this.errorMessage = errorMessage;
}

/**
 * CrossValidator constructor
 * @param rule JavaScipt code that captures a dependency against another field
 * (or this field).
 * @param errMsg an error message to return if the rule is violated
 */
function CrossValidator(rule, errMsg) {
    this.rules = new Array();
    this.rules.push(new _Rule2Message(rule, errMsg));
    this.badRule = -1;
    this.name = "CrossValidator";
}

CrossValidator.prototype = new Validator();

/**
 * Adds a rule to a CrossValidator object
 * @param rule JavaScipt code that captures a dependency against another field.
 * @param errMsg an error message to return if the rule is violated
 */
CrossValidator.prototype.addRule = function(rule, errMsg) {
    this.rules.push(new _Rule2Message(rule, errMsg));
}

/**
 * Validate method
 * Evals all the dependency rules.  If a rule has bad syntax, an alert will be
 * shown; rules should be tested thoroughly before production to ensure that
 * this does not occur at runtime.
 * @return true if all dependency rues are satisifed, false otherwise.
 */
CrossValidator.prototype.validate = function(value) {
    var lastIdx = 0;
    var ret = true;
    try {
        for (var i = 0; i < this.rules.length; i++, lastIdx++) {
            var result = eval(this.rules[i].rule);
            if (!result) {
                ret = false;
                this.badRule = i;
                break;
            }
        }
    }
    catch(e) {
        var errs = [];
        errs.push(this.rules[lastIdx].rule);
        if (e instanceof Error) {
            errs.push(e.name);
            errs.push(e.message);
        }
        else {
            errs.push(e);
        }
        alert(errs.join("\n"));
    }
    return ret;
}

/**
 * errorMessage method
 * Gets the message corresponding to any rule that did not validate
 */
CrossValidator.prototype.errorMessage = function() {
    var ret = "";
    if (this.badRule >= 0 && this.badRule < this.rules.length) {
        ret = this.rules[this.badRule].errorMessage;
    }
    return ret;
}


/***** ScriptValidator class *****/

/**
 * ScriptValidator constructor
 */
function ScriptValidator() {
    this.name = "ScriptValidator";
}

ScriptValidator.prototype = new Validator();

/**
 * Performs script Validation
*/
ScriptValidator.prototype.validate = function(value) {
    return ScriptInputValidation (value, false) ;
}

/**
 * Returns an error message
 */
ScriptValidator.prototype.errorMessage = function() {
		return gsScriptNotAllowed ;
}


/***** EmailValidator class *****/

/**
 * EmailValidator constructor
 */
function EmailValidator(flag) {
	this.flag = flag;
    this.name = "EmailValidator";
}

EmailValidator.prototype = new Validator();

/**
 * Performs rudimentary address validation: no forbidden characters,
 * one and only one '@' character, something on either side of the '@'
 * @return true if the address passes these tests, false otherwise
 */
EmailValidator.prototype.validate = function(value) {
	if(this.flag)
	{
		if (!validateEmail(value))
			return;
		else
			return true;
	}
	else
		return ValidEmail(value, false);
}

/**
 * Returns an "invalid email address" message
 */
EmailValidator.prototype.errorMessage = function() {
    return vfr_resolveString("invalid_email");
}


/***** InputTextValidator class *****/
/**
 * InputTextValidator constructor
 */
function InputTextValidator(flag) {
	this.flag = flag;
    this.name = "InputTextValidator";
}

InputTextValidator.prototype = new Validator();

/**
 * Performs rudimentary address validation: no forbidden characters,
 * one and only one '@' character, something on either side of the '@'
 * @return true if the address passes these tests, false otherwise
 */
InputTextValidator.prototype.validate = function(value) {
	if (!ValidateTextInput(value))
		return;
	else if (!ValidateHtmlInput(value))		
		return;
	else if(!ValidateXMLInput(value))
		return;
	else
		return true;
}

/**
 * Returns an "invalid text input" message
 */
InputTextValidator.prototype.errorMessage = function() {
    return gsTagsNotAllowed ;
}

/***** ASCIIValidator class *****/

/**
 * ASCIIValidator constructor
 */
function ASCIIValidator()
{
	this.name = "ASCIIValidator";
}
ASCIIValidator.prototype = new Validator();

ASCIIValidator.prototype.validate = function(alertname) {
	var length = alertname.length;
	var valid = true;
	var validator = ["AZ", "az", "09", "_", "."]
	var vlength = validator.length;
	for(i=0; i<length; i++)
	{
		var character = alertname.charAt(i);
		var charfound = false;
		for(j=0; j<vlength; j++)
		{
			if( character >= validator[j].charAt(0) && character <= validator[j].charAt(validator[j].length-1))
			{
				charfound = true;
				break;
			}
		}
		if(!charfound)
		{
			valid = false;
			break;
		}
	}
	return valid;
}

/**
 * Returns an "invalid ASCII" message
 */
ASCIIValidator.prototype.errorMessage = function() {
    return gsASCIIAlert;
}

/**
 * Checks for duplicate entries for a particular Field.
 * @param pbIn the PropertyBag of the Section(type of SharedResource)/Policies
 * @param inputArray, It is a string Array containing the required strings.
 */
function DuplicatesValidator(pbIn,inputArray) {
    this.name = "DuplicatesValidator";
    this.pbIn= pbIn;
    this.inputArray=inputArray;
}

DuplicatesValidator.prototype = new Validator();


function ValidateInstanceNames(duplicatesObject,value)
{
	var origName=duplicatesObject.inputArray["origName"]; //origName , has a value incase we are editing the "name" field.
	origName = resolveString(cse_pp_st,origName);
	origName = trim_string(origName);
	value = trim_string(value);

	if(value.toUpperCase() == origName.toUpperCase()) //while editing if the changed name is same as the original name //we come out gracefully
		return true;

	var pbIn = duplicatesObject.pbIn;    //propertyBag
	var section=duplicatesObject.inputArray["SECTION"]; // Section:Type of SharedofResource

	var currType;



	if (section == "Alerts")  // then we are dealing with alerts
		{
			currType=duplicatesObject.inputArray["origType"];    //Dealing with alerts
			var instancename=resolveString(cse_pp_alerts_st, currType+"_S"); //check in the PolicyRes.js
			instancename = resolveString(cse_pp_st,"__default")+ " " + instancename + " " +resolveString(cse_pp_st,"__alert");// since the default alert name is not present in the XML we have to create it.
			instancename =  trim_string(instancename);
			if(value.toUpperCase() == instancename.toUpperCase())       //to check for the default name
				return false; //duplicate
		}


	var pp = new PropertyPath(duplicatesObject.inputArray["basePP"]);    // we make use of the basePP
	pp.BeginEnum();
	var tempLeaf=".name";
	if(section == "Scanners")  //dealing with scanners
		tempLeaf=".displayName";

	while(pp.GetPath() != McUndefined)
	{
		var displayName = pbIn.GetProperty(pp.GetPath()+tempLeaf);
		if ( section == "Alerts")
			alertType   = pbIn.GetProperty(pp.GetPath()+".type");

		if(displayName == McUndefined)
			break;      // necessary to break out of the loop.
		var instname=resolveString(cse_pp_st, displayName);
		if ( instname )
		{
			displayName = instname;
		}

		displayName =  trim_string(displayName);
		if(value.toUpperCase() == displayName.toUpperCase())     //present value is already present in the propertyBag.
			{
			if(section == "Alerts")  // Dealing with alerts
				{
					if(currType==alertType)
			 			return false; //flag an error
				}
			else
				return false;     //flag an error
			}
		pp.IncrementEnum();
	}

	return true;	// No duplicates found.
}

function ValidatePolicies(duplicatesObject,value)
{
	var origName = duplicatesObject.inputArray["origName"];
	value = trim_string(value);
	origName = trim_string(origName);
	if(origName != "")
		if(value.toUpperCase() == origName.toUpperCase())      //Useful when user edits the name and puts it back to the original name.
			return true;

	var pset = duplicatesObject.pbIn; //policies set
		var len=pset.treeNdx.length

		for(var ndx=0;ndx<len;ndx++)
		{
			pset.policies[ndx].name =  trim_string(pset.policies[ndx].name);
			if(value.toUpperCase() == pset.policies[ndx].name.toUpperCase())     //duplicate found.
					return false; //flag error
		}
		return true;
}

function ValidateRepositoryDesc(duplicatesObject, value)
{
	var RepositoryNames = duplicatesObject.inputArray["RepositoryNames"];
	var repnames = RepositoryNames.split(",");
	for(i = 0; i < repnames.length; i++)
	{
		if(value.toUpperCase() == repnames[i].toUpperCase())
		{
			return false;
		}
	}
	return true;
}

/**
 * Validates the Name field.
 * checking that the Name is unique.
 */

DuplicatesValidator.prototype.validate = function(value) {

 if(value == "")     // do not worry about duplicates, reqdvalidator takes care.
  return true;

 switch(this.inputArray["SECTION"])
	{
	 case "Policies":
	    return ValidatePolicies(this,value);
	 case "SiteList":
	    return ValidateRepositoryDesc(this,value);
	 default :
	    return ValidateInstanceNames(this,value);
	}

 }

DuplicatesValidator.prototype.errorMessage = function() {
        return gsDuplicateNotAllowed;
}
/***** LengthValidator class *****/

/**
 * LengthValidator constructor
 */
function LengthValidator() {
    this.name = "LengthValidator";
}

LengthValidator.prototype = new Validator();

/**
 * Performs length validation for instance names.
 * Limit agreed upon is 100 characters.
 * @return true if the instance name is lesser than 100 characters.
 */
LengthValidator.prototype.validate = function(value) {
    //code to check for length
	value = trim_string(value);
    if(value.length <= 100)
     return true;
    else
     return false;

}

/**
 * Returns an "invalid instance name:length limit exceeded" message
 */
LengthValidator.prototype.errorMessage = function() {
    //return vfr_resolveString("invalid_email");
    return gsInvalidNameTooLong;
}


/***** PasswordValidator class *****/

/**
 * PasswordValidator constructor
 */
function PasswordValidator(password) {
    this.password = password;
    this.name = "PasswordValidator";
}

PasswordValidator.prototype = new Validator();

/**
 * checks password and confirm password are same
 * @return true if same
 */
PasswordValidator.prototype.validate = function(confirmpassword) {

	var bOK = false;
	if(this.password == confirmpassword)
	{
		bOK = true;
	}
    	return bOK;
}

/**
 * Returns an "The passwords that you typed donot match" message
 */
PasswordValidator.prototype.errorMessage = function() {
    //return vfr_resolveString("invalid_email");
    return gsPasswordmismatch;
}

/*********************************/



/***** ValidationFramework class *****/

/**
 * Framework constructor
 * @param formName the name of a form to be validated.
 */
function ValidationFramework(formName) {
    this.formName = formName;
    this.validators = new Object();
    this.labelMap = new Object();
    this.formAction = _vfr_default_form_action;
    this.fieldAction = null;
}

/**
 * Adds a field validator; note that multiple validators per field are possible
 * (and expected)
 * @param fieldName the name of the input field to be validated
 * @param validator a functional subclass of Validator to perform the validation
 */
ValidationFramework.prototype.addValidator = function(fieldName, validator) {
    if (fieldName && validator) {
        if (!this.validators.hasOwnProperty(fieldName)) {
            this.validators[fieldName] = [];
        }
        this.validators[fieldName].push(validator);
    }
}

ValidationFramework.prototype.addLabeledValidator = function(fieldName, fieldLabel, validator) {
     this.addValidator(fieldName, validator);
	 this.labelMap[fieldName]= fieldLabel;
}


/**
 * Gets the validator list for a field
 * @param fieldName the field of interest
 * @return an array of validators (may be an empty list or null)
 */
ValidationFramework.prototype.getValidators = function(fieldName) {
    var ret = null;
    if (this.validators.hasOwnProperty(fieldName)) {
        ret = this.validators[fieldName];
    }
    return ret;
}

/**
 * Removes the validator list for a field
 * @param fieldName the field of interest
 */
ValidationFramework.prototype.clearValidators = function(fieldName) {
    if (this.validators.hasOwnProperty(fieldName)) {
        delete this.validators[fieldName];
    }
}

/**
 * Validates the entire form in one go
 * @return true if every field is valid, false otherwise
 */
ValidationFramework.prototype.onsubmit = function() {
    var ret = true;
    var f = document.forms[this.formName];
    var elems = f.elements;
	var nInputs = (elems == undefined) ? 0 : elems.length;
    var goodFields = [];
    var badFields = [];
    var errMsgs = [];

    for (var i = 0; i < nInputs; i++) {
        var elem = elems[i];
        var fieldName = elem.name;
        // At least one validator must have been specified for the field
        if (!this.validators.hasOwnProperty(fieldName)) {
            continue;
        }
        var eleVal = elem.value;
        var vList = this.validators[fieldName];
        // Get a field name to use for error reporting; default is the
        // element name specified in the HTML (which may not be very meaningful)
        var eleLabel = fieldName;
        if (this.labelMap.hasOwnProperty(fieldName)) {
            eleLabel = this.labelMap[fieldName];
        }
        var goodField = true;
        for (var j = vList.length - 1; j >= 0; j--) {
            var validator = vList[j];
            if (!validator.validate(eleVal)) {
                badFields.push(fieldName);
                var errReport = vfr_resolveString("err_report", [eleLabel, vList[j].errorMessage()]);
                errMsgs.push(errReport);
                goodField = false;
                ret = false;
                // Stop processing this field
                break;
            }
        }
        if (goodField) {
            goodFields.push(fieldName);
        }
    }

    if (this.formAction) {
        // Callback
        this.formAction(this.formName, ret, goodFields, badFields, errMsgs);
    }

    return ret;
}

/**
 * Default form action
 * @param formName the name of the form being validated
 * @param validationResult the validation result for the form
 * @param goodFields a list of element IDs of valid fields
 * @param badFields a list of element IDs of fields that failed to validate
 * @param errMsgs a corresponding list of error messages relating to the
 * failed fields
 */
function _vfr_default_form_action(formName, validationResult, goodFields, badFields, errMsgs) {
    // Default action is simply to display an alert and set focus to the
    // first bad field
    if (!validationResult) {
        var errString = vfr_make_bad_form_err_msg(errMsgs).join("\n");
        alert(errString);
        setTimeout("_set_focus(\"" + formName + "\", \"" + badFields[0] + "\")", 200);
    }
}

function _set_focus(form, field) {
    var e = document.forms[form][field];
    if (e && !e.disabled) {
        e.focus();
    }
}

/**
 * Creates an error message for display in list form
 */
function vfr_make_bad_form_err_msg(errMsgs) {
    var errList = [];
    if (!errMsgs) {
        return errList;
    }
    var one_err = (1 == errMsgs.length);
    if (one_err) {
        errList.push(vfr_resolveString("bad_form_one_error"));
    }
    else {
        errList.push(vfr_resolveString("bad_form_mt1_error"));
    }
    for (var i = 0; i < errMsgs.length; i++) {
        errList.push(vfr_resolveString("list_item", [errMsgs[i]]));
    }
    if (one_err) {
        errList.push(vfr_resolveString("one_error_resubmit"));
    }
    else {
        errList.push(vfr_resolveString("mt1_error_resubmit"));
    }
    return errList;
}

/**
 * Validates a single field
 * @param fieldName the name of the field to validate
 */
ValidationFramework.prototype.onchange = function(fieldName) {
    if (!this.validators.hasOwnProperty(fieldName)) {
        return;
    }
    var e = document.forms[this.formName].elements[fieldName];
    if (!e) {
        return;
    }
    var vList = this.validators[fieldName];
    var valid = true;
    var fieldVal = e.value;
    var errMsg = "";
    for (var i = 0; i < vList.length; i++) {
        if (!vList[i].validate(fieldVal)) {
            valid = false;
            errMsg = vList[i].errorMessage();
            break;
        }
    }
    if (this.fieldAction) {
        this.fieldAction(this.formName, fieldName, valid, errMsg);
    }
}

/**
 * Checks whether a field is OK
 * @param fieldName the field of interest
 * @return true if OK, false otherwise
 */
ValidationFramework.prototype.check = function(fieldName) {
    var ok = true;
    var vList = this.validators[fieldName];
    if (vList) {
        var e = document.forms[this.formName].elements[fieldName];
        if (e) {
            var fieldVal = e.value;
            for (var i = 0; i < vList.length; i++) {
                if (!vList[i].validate(fieldVal)) {
                    ok = false;
                    break;
                }
            }
        }
    }
    return ok;
}

ValidationFramework.prototype.toString = function() {
    return "ValidationFramework[" + this.formName + "]";
}


/**
 * Formats a string, interpolating variable values into placeholders; the
 * unformatted string takes the form "text {0} more text {1}: {2}"
 * @param unf the unformatted string
 * @param inserts a list of values to insert into the string
 * @return the formatted string
 */
function vfr_formatString(unf, inserts) {
    var ret = unf;
    if (inserts) {
	var iCount = inserts.length;
	for (var i = 0; i < iCount; i++) {
	    var pattern = "\\{" + i + "\\}";
	    var re = new RegExp(pattern, "g");
	    ret = ret.replace(re, inserts[i]);
	}
    }
    return ret;
}

/**
 * Resolves a string and formats it
 * @param key the lookup key in the table
 * @param inserts a list of values to insert into the string
 * @return the formatted string
 */
function vfr_resolveString(key, inserts) {
    var ret = key;
    if (vfr_st.hasOwnProperty(key)) {
        ret = vfr_formatString(vfr_st[key], inserts);
    }
    return ret;
}

function validateEmail(value) {
	//var ctrl = document.getElementById("McIdAdminEmail")
	var emailIDs = trim_string(value);

	if(emailIDs.substring(emailIDs.length-1) == ";")
	{
		emailIDs = emailIDs.substring(0,emailIDs.length-1);
	}
	var emailID = SplitOutsideQuotes(emailIDs,";") 
	for (var i=0; i<emailID.length;i++ )
	{				
			if (!ValidEmail(emailID[i],false))
			{
				//ctrl.focus();
				return	false;
			}	
			
			 for(var j=i+1;j<emailID.length;j++)
			{
				 if (emailID[i].toLowerCase() == emailID[j].toLowerCase() )
				{
					alert( gsInvalidBWEmailAlert + [emailID[i]] );
				   // ctrl.focus();
				    return false;
				}	
			}
	}
	return true;
}

function ValidateTextInput(text)
{
	var isValid = true;
	var pattern = "";
	var patternString = "[<|&lt;]*[\w\s\d=/\"'():]*script[a-zA-Z0-9\s=/\"'():]*[>|&gt;]"
	pattern = new RegExp(patternString, 'i');
	//pattern = new RegExp("[<|&lt;]*[\s]*script[\s]*[>|&gt;]", 'i');
	if(pattern.test(text)) {
		isValid = false;	
	}	
	
	return isValid;
}


function ValidateAlertTextInput(text)
{
	var isValid = true;
	var pattern = "";
	//var patternString = "[\w\s\d=/\"'():]*alert[a-zA-Z0-9\s=/\"'():]+"
	  var patternString = "[\\w\\s\\d=\/\"'():]*alert[\\s]*[=/\"'():]+"
	pattern = new RegExp(patternString, 'i');
	if(pattern.test(text)) {
		isValid = false;	
	}	
	
	return isValid;
}

function ValidateHtmlInput(text)
{
	var isValid = true;
	var pattern = "";
	
	//block all html tags in the input fields. Add more tags to the below array, if any other tags needs to be blocked.
	var htmlTags = ["BODY", "IMG", "SRC", "HTML", "INPUT", "STYLE", "iframe", "TITLE", "XSS", "META", "FRAME", "TABLE", 
					"DIV", "BASE", "OBJECT", "EMBED", "XML", "SPAN", "A", "BGSOUND", "BR", "LINK", "SCRIPT"];

	//1. get the tags one by one from the array
	//2. prepare a pattern that matches "<TAG", "</TAG", "</ TAG", "<  /TAG", "<  /  TAG" or "<  TAG"
	//3. if the input string matches the pattern, mark invalid
	for(var i=0; i < htmlTags.length; i++)
	{			
		var tag = htmlTags[i];
		var patternStr = "[<][/\' \']*"+tag;
		pattern = new RegExp(patternStr, "i")
		if(pattern.test(text))
		{
			return false;
		}
	}
	return true;
}


function ValidateLimitedHtmlInput(text)
{
	var isValid = true;
	var pattern = "";
	
	//block all html tags in the input fields. Add more tags to the below array, if any other tags needs to be blocked.
	
	//var htmlTags = ["INPUT", "iframe", "XSS", "FRAME", "BASE", "OBJECT", "EMBED", "BGSOUND",  "SCRIPT", "BODY"];
	var htmlTags = ["INPUT", "iframe", "XSS", "FRAME", "BASE", "OBJECT", "EMBED", "BGSOUND",  "SCRIPT"];
	//Check for javascript events attached to img and anchor tags
	var patternForImageAndHyperlink = "(<img[^>]*\\s+on[a-z]+=[^>]+>)|(<a[^>]*\\s+on[a-z]+=[^>]+>)";
	patternEventCheck = new RegExp(patternForImageAndHyperlink, 'i');
	if(patternEventCheck.test(text))
	return false
	
	//1. get the tags one by one from the array
	//2. prepare a pattern that matches "<TAG", "</TAG", "</ TAG", "<  /TAG", "<  /  TAG" or "<  TAG"
	//3. if the input string matches the pattern, mark invalid
	else
	{
		for(var i=0; i < htmlTags.length; i++)
		{			
			var tag = htmlTags[i];
			var patternStr = "[<][/\' \']*"+tag;
			pattern = new RegExp(patternStr, "i")
			if(pattern.test(text))
			{
				return false;
			}
		}
	}
	return true;
}

function ValidateXMLInput(text)
{
var isValid = true;
	var pattern = "";
	var patternString = "(<!--\\s*)|(-->\\s*)";
	pattern = new RegExp(patternString, 'i');
	
	if(pattern.test(text)) {
		isValid = false;	
	}	
	
	return isValid;
}


/***** SquareBracketValidator class *****/
/**
 * SquareBracketValidator constructor
 */
function SquareBracketValidator(flag) {
	this.flag = flag;
    this.name = "SquareBracketValidator";
}

SquareBracketValidator.prototype = new Validator();

/**
 * Checks for the presence of square brackets in the input
 * '[' and ']'characters
 * @return true if the address passes these tests, false otherwise
 */
SquareBracketValidator.prototype.validate = function(value) {
	if (!ValidateSquareInput(value))
		return;
	
		return true;
}

/**
 * Returns an "invalid text input" message
 */
SquareBracketValidator.prototype.errorMessage = function() {
    return gsSquareBracketsNotAllowed;
}

ValidateSquareInput = function(text)
{
	var isValid = true;
    var pattern = "";
    var patternString = "(\\[)|(\\])";
	pattern = new RegExp(patternString, 'i');
                
	if(pattern.test(text)) 
	{
		isValid = false;   
	}               
	return isValid;
}
