/**
 * RichText.js
 * Rich-text editor for editing alert texts
 *
 * Copyright (C) 2009 McAfee, Inc. All rights reserved.
 * This code is partly derived from Kevin Roth's rich text editor;
 * Kevin Roth's original notice is reproduced below.
 *
 * The areas that use Kevin Roth's code are the initRTE() function,
 * the popDlg() function, the enableDesignMode() method and the
 * functions for raising and lowering toolbar buttons
 */

// KEVIN ROTH'S ORIGINAL NOTICE:
//
// Cross-Browser Rich Text Editor
// http://www.kevinroth.com/rte/demo.htm
// Written by Kevin Roth (kevin@NOSPAMkevinroth.com - remove NOSPAM)
// Visit the support forums at http://www.kevinroth.com/forums/index.php?c=2
// This code is public domain. Redistribution and use of this code, with or without modification, is permitted.

/**
 * Class for handling insertion tokens
 * @param name the name of the token (the visible element in the tokens list)
 * @param value the token that actually gets inserted
 * @param description a description of the token (specified as a title attribute
 * On the select option
 */
function IToken(name, value, description) {
    this.name = name;
    this.value = value;
    this.description = description;
}

// Converts a token object into an option that can go into a select
IToken.prototype.toOption = function() {
    var opt = document.createElement("option");
    opt.value = this.value;
    opt.text = this.name;
    opt.title = this.description;
    return opt;
}

// Edit mode constants
SOURCE_MODE = -1;
WYSIWYG_MODE = 0;
PLAINTEXT_MODE = 1;
NBSP = 160;

// Default values for the popup dialogs
var dlgDefaults = {"tableForm":[{name:"Rows", value:2},
        {name:"Columns", value:2},
        {name:"Width", value:100},
        {name:"WTypepc", checked:true},
        {name:"Border", value:1},
        {name:"Padding", value:4},
        {name:"Spacing", value:0}],
    "linkForm":[{name:"Url", value:""},
        {name:"LinkText", value:""},
        {name:"_Blank", checked:false}],
    "imageForm":[{name:"Url", value:""},
        {name:"AltText", value:""},
        {name:"SetTitle", checked:true}]};

// Properties for a rich-text control
// width: the width of the control
// height: the height of the control
// readOnly: true if the edit control is read-only
// richText: true if the edit control is to be a rich-text control; plain
// text controls are simple text-areas
// tokens: whether the control should support the token list select
// plainTextOption: whether the control should support the simultaneous editing
// of an HTML document and plain text version
// htmlControl: the name of the control for setting / getting HTML; defaults to
// the name of the control passed to the constructor
// textControl: the name of the control for setting / getting plain text; defaults
// to the name of the control passed to the constructor + "plain"

// Global variables that are initialized through a call to initRTE
var richTextCapable = false;
var browserInfo = {};
try {
    browserInfo = IdentifyBrowser();
}
catch(e) {
    // Browser.js not included
    browserInfo = {isIE:false, isGecko:false, isOpera:false, isSafari:false, isKonqueror:false, isChrome:false};
    var ua = navigator.userAgent.toLowerCase();
    browserInfo.isIE = ((ua.indexOf("msie") != -1) && (ua.indexOf("opera") == -1) && (ua.indexOf("webtv") == -1));
    browserInfo.isGecko = (ua.indexOf("gecko") != -1);
    browserInfo.isOpera = (ua.indexOf("opera") != -1);
    browserInfo.isSafari = (ua.indexOf("safari") != -1);
    browserInfo.isKonqueror = (ua.indexOf("konqueror") != -1);
	browserInfo.isChrome = (ua.indexOf("chrome") != -1);
	if((browserInfo.isChrome)&&(browserInfo.isSafari)) {browserInfo.isSafari=false;}
}
var imagesPath = "";
var cssFile = "";
var cssText = "";
var lang = "en_US";
var encoding = "iso-8859-1";

// Form validators
var rte_vframeworks = new Object();
var reqV = null, posV = null; mt1V = null, pcV = null;
try {
    reqV = new RequiredValidator();
    posV = new GTEValidator();
    mt1V = new GTEValidator(1);
    pcV = new RangeValidator(1, 100);
}
catch(e) {
    // Validation framework not included - no problem if
    // the control is read-only
}


var labelMap = new Object();
labelMap["url"] = resolveString(rte_st, "url1");
labelMap["linkText"] = resolveString(rte_st, "linkText");
labelMap["imageUrl"] = resolveString(rte_st, "url1");
labelMap["rows"] = resolveString(rte_st, "rows1");
labelMap["columns"] = resolveString(rte_st, "columns1");
labelMap["width"] = resolveString(rte_st, "width1");
labelMap["border"] = resolveString(rte_st, "thickness1");
labelMap["padding"] = resolveString(rte_st, "padding1");
labelMap["spacing"] = resolveString(rte_st, "spacing1");

// List of declared rich-text objects
var rteMap = {};

/**
 * Initializes the rich-text library
 * @param imgPath the path for images
 * @param cssF the name of an external stylesheet file to use within rich-text documents
 * @param cssT inline styles to apply in the absence of a CSS file, in the absence of
 * both an external file and inline stylesheet a default stylesheet will be used
 */
function initRTE(imgPath, cssF, cssT) {
    // Check to see whether the browser is capable of hosting the rich-text control
    if (document.getElementById && document.designMode && !browserInfo.isSafari && !browserInfo.isKonqueror) {
        richTextCapable = true;
    }

    if (browserInfo.isIE) {
        document.onmouseover = raiseButton;
        document.onmouseout  = normalButton;
        document.onmousedown = lowerButton;
        document.onmouseup   = raiseButton;
    }

    imagesPath = imgPath;
    if (cssF)
        cssFile = cssF;
    if (cssT)
        cssText = cssT;
}

/**
 * Constructor for the rich-text control
 * @param headerText the prompt for the control
 * @param rte the name of the control; used for creating element IDs
 * @param ctlName the name by which the control has been declared in the HTML
 * @param properties properties of the control, such as geight and width and
 * whether or not it is read-only; may be null
 */
function RTControl(headerText, rte, ctlName, properties) {
    if (null == properties) {
        properties = {};
    }
    initControl(properties, this);
    this.headerText = headerText;
    this.name = rte;
    this.ctlName = (null != ctlName) ? ctlName : "rte";
    this.created = false;
    this.mode = WYSIWYG_MODE;
    this.initialContent = "";
    this.initialText = "";
    this.currentContent = "";
    this.currentText = "";

    // Add this object to the global map
    rteMap[rte] = this;
}

/**
 * Initializes the properties of a rich-text control
 * @param propsIn the property source
 * @param propsOut the property destination
 */
function initControl(propsIn, propsOut) {
    propsOut.width = (null != propsIn.width) ? propsIn.width : "540";
    propsOut.height = (null != propsIn.height) ? propsIn.height : "400";
    propsOut.readOnly = (null != propsIn.readOnly) ? propsIn.readOnly : false;
    propsOut.richText = richTextCapable ? ((null != propsIn.richText) ? propsIn.richText : true) : false;
    propsOut.tokens = (null != propsIn.tokens) ? propsIn.tokens : false;
    if (null != propsIn.buttons && propsIn.buttons == false) {
        propsOut.buttons = false;
    }
    else {
        propsOut.buttons = ((propsOut.richText || propsOut.tokens) && !propsOut.readOnly) ? true : false;
    }
    propsOut.ptOpt = (null != propsIn.plainTextOption) ? propsIn.plainTextOption : false;
    propsOut.ptBtn = (null != propsIn.plainTextButton) ? propsIn.plainTextButton : (propsOut.ptOpt ? true : false);
    propsOut.ctlR = (null != propsIn.htmlControl) ? propsIn.htmlControl : this.name;
    propsOut.ctlT = (null != propsIn.textControl) ? propsIn.textControl : this.name + "plain";
    propsOut.resetBtn = (null == propsIn.resetBtn) ? false : (this.readOnly ?  false : propsIn.resetBtn);
    propsOut.switchable = (null != propsIn.switchable) ? propsIn.switchable : false;
    propsOut.selector = (null != propsIn.selector) ? propsIn.selector : true;
    propsOut.srcCtl = (null != propsIn.srcCtl) ? propsIn.srcCtl : true;
    propsOut.changeNotifyFn = propsIn.changeNotifyFn ?
        ('function' == typeof(propsIn.changeNotifyFn) ? propsIn.changeNotifyFn : new Function(propsIn.changeNotifyFn)) : null;
}

/**
 * Gets the current editing mode of the control
 */
RTControl.prototype.getMode = function() {
    return (this.mode == 0) ? WYSIWYG_MODE :
        ((this.mode < 0) ? SOURCE_MODE : PLAINTEXT_MODE);
}

/**
 * Writes the rich-text control in the page
 * @param container the ID of the element that will hold the control
 * @param html the text to use to initialize the control; may be null
 * @param text the text to use to initialize the plaintext version of the
 * document; may be null
 * @param clearParent whether the contents of the container should be cleared
 * when creating the control
 */
RTControl.prototype.makeControl = function(container, html, text, clearParent) {
    var e = document.getElementById(container);
    if (!e)
        return;
    if (null == html)
        html = "";
    if (null == text)
        text = "";
    var rte = this.name;
    var rteCtl = this.ctlName;
    // Fix the width of the control to allow for the button bar if necessary
    var widthVal = parseInt(this.width);
    var re = /\d+(\D*)/;
    var units = "px";
    var match = re.exec(this.width);
    if (null != match && match.length > 1 && match[1]) {
        units = match[1];
    }
    if (this.buttons && widthVal < 540 && "px" == units) {
        widthVal = 540;
    }
    var editWidth = widthVal;
    var taWidth = widthVal;
    if ("%" == units) {
        var baseWidth = browserInfo.isIE ? e.offsetWidth : e.clientWidth;
        editWidth = Math.floor(widthVal * baseWidth / 100);
    }
    //if ("px" == units) {
        // Allow for 1-pixel borders
        editWidth -= 2;
        taWidth = editWidth;
        if (browserInfo.isIE) {
            if (this.richText && browserInfo.browser && browserInfo.browser.versionNum < 7) {
                // Additional adjustment for IE's handling of IFRAMEs
                editWidth -= 2;
            }
            // Textarea controls are drawn slightly too wide
            taWidth -= 2;
        }
    //}
    var heightVal = parseInt(this.height);
    // Create a div that will be used for positioning the popup dialogs
    var ctl = document.createElement("div");
    ctl.id = rte + "tlObject";
    ctl.className = "rteDiv";
    // Create the header
    ctl.appendChild(this.makeHeader(widthVal, units));
    // Create the first toolbar
    var toolbar = this.makeToolbar1(widthVal, units);
    if (toolbar)
        ctl.appendChild(toolbar);
    // Second toolbar
    toolbar = this.makeToolbar2(widthVal, units);
    if (toolbar)
        ctl.appendChild(toolbar);

    if (this.richText) {
        // Textarea for the source view of the document
        var ta = this.makeTextArea(this.ctlR, rte + "Src", (taWidth) + "px",
            (heightVal) + "px", html);
        ta.style.display = "none";
        ctl.appendChild(ta);

        // IFrame for editable area
        var iframe = document.createElement("iframe");
        iframe.id = rte;
        iframe.name = rte;
        iframe.width = editWidth;
        iframe.height = heightVal;
        iframe.style.border = "1px solid #B8B8B8";
        iframe.src = "javascript:parent.emptyDoc();";
        if (browserInfo.isIE) {
            iframe.frameBorder = 0;
        }
        ctl.appendChild(iframe);

        if (!this.readOnly && this.buttons) {
            // Create popup dialogs
            ctl.appendChild(this.createPalette());
            ctl.appendChild(this.createImageDlg());
            ctl.appendChild(this.createLinkDlg());
            ctl.appendChild(this.createTableDlg());
            this.currentPopup = "";

            // Create validators for the various forms
            var formName = rte + "linkForm";
            var vf = new ValidationFramework(formName);
            vf.labelMap = labelMap;
            vf.addValidator("url", reqV);
            vf.addValidator("linkText", reqV);
            rte_vframeworks[formName] = vf;

            formName = rte + "imageForm";
            vf = new ValidationFramework(formName);
            vf.labelMap = labelMap;
            vf.addValidator("url", reqV);
            rte_vframeworks[formName] = vf;

            formName = rte + "tableForm";
            vf = new ValidationFramework(formName);
            vf.labelMap = labelMap;
            vf.addValidator("rows", reqV);
            vf.addValidator("rows", mt1V);
            vf.addValidator("columns", reqV);
            vf.addValidator("columns", mt1V);
            vf.addValidator("width", reqV);
            vf.addValidator("width", pcV);
            vf.addValidator("border", reqV);
            vf.addValidator("border", posV);
            vf.addValidator("padding", reqV);
            vf.addValidator("padding", posV);
            vf.addValidator("spacing", reqV);
            vf.addValidator("spacing", posV);
            rte_vframeworks[formName] = vf;

            for (var form in rte_vframeworks) {
                rte_vframeworks[form].formAction = std_val_form_action;
                rte_vframeworks[form].fieldAction = std_val_field_action;
            }
        }
    }
    else {
        var ta = this.makeTextArea(this.ctlR, rte, (taWidth) + "px",
            (heightVal) + "px", html);
        ctl.appendChild(ta);
    }
    if (this.ptOpt) {
        // Textarea for the plain-text version of the document
        var ta = this.makeTextArea(this.ctlP, rte + "Plain", (taWidth) + "px",
            (heightVal) + "px", text);
        ta.style.display = "none";
        ctl.appendChild(ta);
    }

    if (this.selector) {
        toolbar = this.makeToolbar3(widthVal, units);
        if (toolbar)
            ctl.appendChild(toolbar);
    }

    // Insert the control in the page
    if (e) {
        if (clearParent) {
            while(e.firstChild) {
                e.removeChild(e.firstChild);
            }
        }
        e.appendChild(ctl);
    }

    this.currentContent = html;
    this.currentText = text;
    if (!this.initialContent) {
        this.initialContent = html;
    }
    if (!this.initialText) {
        this.initialText = html;
    }
    this.container = container;
    // Only turn on design mode if the control is visible
    //if (isVisible(e))
    this.enableDesignMode(false);
    e = document.getElementById("chkRO" + rte);
    if (e) {
        e.checked = this.readOnly;
    }
    this.created = true;
}

RTControl.prototype.makeTextArea = function(name, id, width, height, content) {
    var ta = document.createElement("textarea");
    ta.className = "ptEditor";
    ta.name = name;
    ta.id = id;
    ta.readOnly = this.readOnly;
    ta.value = content;

    ta.style.width = width;
    ta.style.height = height;
    ta.style.border = "1px solid #B8B8B8";
    //ta.style.paddingLeft = "10px";
    ta.style.marginTop = "-1px";
    if (browserInfo.isIE)
        ta.style.marginBottom = "-1px";
    if (this.changeNotifyFn)
        ta.onfocus = this.changeNotifyFn;
    return ta;
}

/**
 * Creates the header of the control
 */
RTControl.prototype.makeHeader = function(widthVal, units) {
    var tableHeader = document.createElement("table");
    tableHeader.border = 0;
    tableHeader.cellPadding = 0;
    tableHeader.cellSpacing = 0;
    tableHeader.id = "header" + this.name;
    tableHeader.className = "shadedTable";
    tableHeader.style.width = widthVal + units;
    tableHeader.style.borderBottomWidth = 0;

    var row0 = tableHeader.insertRow(0);
    var cell = row0.insertCell(0);
    cell.className = "st";

    var span = document.createElement("span");
    span.style.fontWeight = "bold";
    span.id = this.name + "_header_text";
    span.appendChild(document.createTextNode(this.headerText));
    cell.appendChild(span);
    cell = row0.insertCell(1);
    cell.className = "st noLeft";
    cell.style.textAlign = "right";
    cell.id = this.name + "_header_opts";
    return tableHeader;
}

/**
 * Sets the header text
 */
RTControl.prototype.setHeaderText = function(text) {
    var span = document.getElementById(this.name + "_header_text");
    if (span) {
        span.removeChild(span.firstChild);
        span.appendChild(document.createTextNode(text));
        this.headerText = text;
    }
}

/**
 * Gets a reference to the options cell in the header where you
 * can add your own controls
 */
RTControl.prototype.getOptionsContainer = function() {
    return document.getElementById(this.name + "_header_opts");
}

RTControl.prototype.makeOption = function(value, label) {
    var opt = document.createElement("option");
    opt.value = value;
    opt.text = label;
    return opt;
}

RTControl.prototype.addSelOption = function(sel, opt) {
    if (browserInfo.isIE)
        sel.add(opt);
    else
        sel.add(opt, null);
}

/**
 * Creates the first toolbar with or without text-formatting controls
 */
RTControl.prototype.makeToolbar1 = function(widthVal, units) {
    if (!this.buttons || this.readOnly)
        return null; // No toolbars
    if (!this.richText && !this.tokens)
        return null; // No controls to place in the first toolbar
    var rte = this.name;
    var rteCtl = this.ctlName;

    var toolbar = document.createElement("table");
    toolbar.border = 0;
    toolbar.cellPadding = 0;
    toolbar.cellSpacing = 0;
    toolbar.id = "Buttons1_" + rte;
    toolbar.className = "shadedTable";
    toolbar.style.width = widthVal + units;
    toolbar.style.borderBottomWidth = 0;

    var row0 = toolbar.insertRow(0);
    var cell0 = row0.insertCell(0);
    cell0.className = "st";
    cell0.noWrap = true;
    if (this.richText) {
        // Create a style picker
        var span = document.createElement("span");
        span.id = "formatblocksp_" + rte;
        span.style.marginRight = "20px";
        var sel = document.createElement("select");
        sel.id = "formatblock_" + rte;
        sel.style.width = "120px";
        var fnStr = rteCtl + ".styleText('" + sel.id + "');"
        sel.onchange = new Function(fnStr);
        var options = [];
        options.push(this.makeOption("", resolveString(rte_st, "style")));
        var opts = ["<P>", "<H1>", "<H2>", "<H3>", "<H4>", "<H5>", "<H6>", "<ADDRESS>", "<PRE>"];
        for (var i = 0; i < opts.length; i++) {
            options.push(this.makeOption(opts[i], resolveString(rte_st, opts[i])));
        }
        for (var i = 0; i < options.length; i++) {
            this.addSelOption(sel, options[i]);
        }
        span.appendChild(sel);
        cell0.appendChild(span);

        // Font picker
        span = document.createElement("span");
        span.id = "fontnamesp_" + rte;
        span.style.marginRight = "20px";
        sel = document.createElement("select");
        sel.id = "fontname_" + rte;
        sel.style.width = "90px";
        fnStr = rteCtl + ".styleText('" + sel.id + "');"
        sel.onchange = new Function(fnStr);
        options = [];
        options.push(this.makeOption("", resolveString(rte_st, 'font')));
        options.push(this.makeOption("Verdana, Arial, Helvetica, sans-serif",
            resolveString(rte_st, 'sans')));
        options.push(this.makeOption("Times New Roman, Times, serif",
            resolveString(rte_st, 'serif')));
        options.push(this.makeOption("Courier New, Courier, monospace",
            resolveString(rte_st, 'mono')));
        for (var i = 0; i < options.length; i++) {
            this.addSelOption(sel, options[i]);
        }
        span.appendChild(sel);
        cell0.appendChild(span);

        // Size picker
        var span = document.createElement("span");
        span.id = "fontsizesp_" + rte;
        span.style.marginRight = "20px";
        var sel = document.createElement("select");
        sel.id = "fontsize_" + rte;
        sel.style.width = "90px";
        fnStr = rteCtl + ".styleText('" + sel.id + "');"
        sel.onchange = new Function(fnStr);
        var options = [];
        options.push(this.makeOption("", resolveString(rte_st, "size")));
        opts = ["1", "2", "3", "4", "5", "6", "7"];
        for (var i = 0; i < opts.length; i++) {
            options.push(this.makeOption(opts[i], resolveString(rte_st, opts[i])));
        }
        for (var i = 0; i < options.length; i++) {
            this.addSelOption(sel, options[i]);
        }
        span.appendChild(sel);
        cell0.appendChild(span);
    }

    if (this.tokens) {
        // Token picker
        var sel = document.createElement("select");
        sel.id = "tokens_" + rte;
        sel.style.width = "220px";
        var fnStr = this.ctlName + ".handleInsert();"
        sel.onchange = new Function(fnStr);
        this.addSelOption(sel, this.makeOption("", resolveString(rte_st, "tokens")));
        cell0.appendChild(sel);
    }
    return toolbar;
}

/**
 * Creates the second toolbar
 */
RTControl.prototype.makeToolbar2 = function(widthVal, units) {
    if (!this.richText || !this.buttons || this.readOnly)
        return null; // No toolbars
    var rte = this.name;
    var rteCtl = this.ctlName;

    var toolbar = document.createElement("table");
    toolbar.border = 0;
    toolbar.cellPadding = 0;
    toolbar.cellSpacing = 0;
    toolbar.id = "Buttons2_" + rte;
    toolbar.className = "shadedTable";
    toolbar.style.width = widthVal + units;
    toolbar.style.borderBottomWidth = 0;

    var row0 = toolbar.insertRow(0);
    var cellIdx = 0;
    var ctls = [{id:"bold", option:""},
        {id:"italic", option:""},
        {id:"underline", option:"", key:"uline"},
        {id:"rteSep"},
        {id:"justifyleft", option:""},
        {id:"justifycenter", option:""},
        {id:"justifyright", option:""},
        {id:"justifyfull", option:""},
        {id:"rteSep"},
        {id:"insertorderedlist", option:"", img:"numbered_list"},
        {id:"insertunorderedlist", option:"", img:"list"},
        {id:"rteSep"},
        {id:"outdent", option:""},
        {id:"indent", option:""},
        {id:"forecolor", fn:"dlgColorPalette", img:"textcolor"},
        {id:"hilitecolor", fn:"dlgColorPalette", img:"bgcolor"},
        {id:"rteSep"},
        {id:"inserthorizontalrule", option:"", img:"hr"},
        {id:"link", key:"insertlink", fn:"dlgInsertLink", img:"hyperlink"},
        {id:"image", key:"insertimage", fn:"dlgInsertImage"},
        {id:"table", key:"inserttable", fn:"dlgInsertTable", img:"insert_table"}];
    for (var i = 0; i < ctls.length; i++) {
        var cell = row0.insertCell(cellIdx++);
        cell.className = "toolbar";
        var ctlId = ctls[i].id;
        if (ctlId == "rteSep") {
            cell.appendChild(makeSep());
            continue;
        }
        var btnInfo = {id:ctls[i].id};
        if (ctls[i].key)
            btnInfo.name = resolveString(rte_st, ctls[i].key);
        else
            btnInfo.name = resolveString(rte_st, ctls[i].id);
        btnInfo.option = ctls[i].option;
        if (ctls[i].img)
            btnInfo.img = ctls[i].img;
        var func = makeTBButton;
        if (ctls[i].fn) {
            btnInfo.fn = ctls[i].fn;
            func = makeTBButton2;
        }
        cell.appendChild(func(rte, rteCtl, imagesPath, btnInfo));
    }

    cell = row0.insertCell(cellIdx++);
    cell.appendChild(document.createTextNode(" "));
    cell.style.width = "100%";

    return toolbar;
}

/**
 * Creates the third toolbar
 */
RTControl.prototype.makeToolbar3 = function(widthVal, units) {
    var rte = this.name;
    var rteCtl = this.ctlName;

    var toolbar = document.createElement("table");
    toolbar.border = 0;
    toolbar.cellPadding = 0;
    toolbar.cellSpacing = 0;
    toolbar.id = "Buttons3_" + rte;
    toolbar.className = "shadedTable";
    toolbar.style.width = widthVal + units;
    if (!browserInfo.isIE) {
        // Remove a double border effect that appears in non-IE browsers
        toolbar.style.borderTopWidth = 0;
    }
    toolbar.style.marginTop = "-1px";

    var row0 = toolbar.insertRow(0);
    var cellIdx = 0;
    var cell = row0.insertCell(cellIdx++);
    cell.noWrap = true;
    cell.className = "st";
    cell.id = rte + "_b3_left";

    var hasContent = false;
    var sel = document.createElement("select");
    sel.id = rte + "ModeSel";
    var fnStr = rteCtl + ".setEditMode(this.value);";
    sel.onchange = new Function(fnStr);
    if (this.richText) {
        this.addSelOption(sel, this.makeOption(WYSIWYG_MODE, resolveString(rte_st, "wysiwygMode")));
        if (this.srcCtl) {
            this.addSelOption(sel, this.makeOption(SOURCE_MODE, resolveString(rte_st, "sourceMode")));
        }
    }
    if (this.ptOpt && this.ptBtn) {
        this.addSelOption(sel, this.makeOption(PLAINTEXT_MODE, resolveString(rte_st, "ptMode")));
    }
    if (sel.length > 0) {
        cell.appendChild(document.createTextNode(resolveString(rte_st, "modeSel")));
        hasContent = true;
    }
    else {
        sel.style.display = "none";
    }
    cell.appendChild(sel);

    if (this.switchable) {
        cell.appendChild(document.createTextNode(" "  + String.fromCharCode(NBSP) + " "));
        var input = document.createElement("input");
        input.type = "checkbox";
        input.className = "checkbox";
        var id = "chkRO" + rte;
        input.id = id;
        if (this.readOnly) {
            input.checked = true;
        }
        fnStr = rteCtl + ".toggleReadOnly(this.checked);";
        input.onclick = new Function(fnStr);
        cell.appendChild(input);
        cell.appendChild(document.createTextNode(String.fromCharCode(NBSP)));
        var label = document.createElement("label");
        label.htmlFor = id;
        var span = document.createElement("span");
        span.style.fontWeight = "bold";
        span.appendChild(document.createTextNode(resolveString(rte_st, "readonly")));
        label.appendChild(span);
        cell.appendChild(label);
        hasContent = true;
    }

    if (!hasContent)
        cell.appendChild(document.createTextNode(String.fromCharCode(NBSP)));

    if (this.resetBtn && !this.readOnly) {
        cell = row0.insertCell(cellIdx++);
        cell.style.textAlign = "right";
        cell.className = "st";
        cell.id = rte + "_b3_right";
        input = document.createElement("input");
        input.type = "button";
        input.className = "btnSmall";
        input.value = resolveString(rte_st, "reset");
        fnStr = rteCtl + ".reset();";
        input.onclick = new Function(fnStr);
        cell.appendChild(input);
    }

    return toolbar;
}

/**
 * Creates a separator bar
 */
function makeSep() {
    var btn = document.createElement("img");
    btn.className = "rteVertSep";
    btn.src = imagesPath + "blackdot.gif";
    btn.width = 1;
    btn.height = 20;
    btn.alt = "";
    btn.border = 0;
    return btn;
}

/**
 * Creates a toolbar button tied to a design-mode function
 * @param editorId the ID of the edit control
 * @param editor the name of the edit control
 * @param imagesPath the path for button images
 * @param btnInfo image properties
 */
function makeTBButton(editorId, editor, imagesPath, btnInfo) {
    var id = btnInfo.id;
    var img = btnInfo.img ? btnInfo.img : id;
    var name = btnInfo.name;
    var alt = btnInfo.alt ? btnInfo.alt : btnInfo.name;
    var option = btnInfo.option;
    var fnStr = editor + ".doCommand('" + id + "'";
    if (null != option)
        fnStr += ", '" + option + "'";
    fnStr += ");";
    var btn = document.createElement("img");
    btn.id = id;
    btn.className = "rteImage";
    btn.src = imagesPath + img + ".gif";
    btn.width = 25;
    btn.height = 24;
    btn.alt = alt;
    btn.title = name;
    btn.onclick = new Function(fnStr);
    return btn;
}

/**
 * Creates a toolbar button tied to an RTControl method
 * @param editorId the ID of the edit control
 * @param editor the name of the edit control
 * @param imagesPath the path for button images
 * @param btnInfo image properties
 */
function makeTBButton2(editorId, editor, imagesPath, btnInfo) {
    var id = btnInfo.id;
    var img = btnInfo.img ? btnInfo.img : id;
    var name = btnInfo.name;
    var alt = btnInfo.alt ? btnInfo.alt : btnInfo.name;
    var jsFn = btnInfo.fn;
    var fnStr = editor + "." + jsFn + "('" + id + "', '');";
    var btn = document.createElement("img");
    btn.id = id;
    btn.className = "rteImage";
    btn.src = imagesPath + img + ".gif";
    btn.width = 25;
    btn.height = 24;
    btn.alt = alt;
    btn.title = name;
    btn.onclick = new Function(fnStr);
    // Create a surrounding div for use with popup dialogs
    var div = document.createElement("div");
    div.id = id + "_" + editorId;
    div.appendChild(btn);
    return div;
}

/**
 * Shows / hides the toolbars; toolbar1 will remain visible if there is a
 * tokens picklist; otherwise it will disappear
 */
RTControl.prototype.showHideToolbars = function(show) {
    var display = show ? "" : "none";
    var rte = this.name;
    var toolbar = document.getElementById("Buttons1_" + rte);
    if (!toolbar)
        return;
    if (!this.tokens) {
        toolbar.style.display = display;
    }
    else {
        var elems = ["formatblocksp_" + rte, "fontnamesp_" + rte, "fontsizesp_" + rte];
        for (var i = 0; i < elems.length; i++) {
            var e = document.getElementById(elems[i]);
            if (e)
                e.style.display = display;
        }
    }
    toolbar = document.getElementById("Buttons2_" + rte);
    if (toolbar)
        toolbar.style.display = display;
}

/**
 * Styles a text selection
 * @param ctlName the name of the select control handling the style command
 */
RTControl.prototype.styleText = function(ctlName) {
    // Get the index of the selection
    var ctl = document.getElementById(ctlName);
    var idx = ctl.selectedIndex;
    // The first item in all the text styling select control is a label
    if (idx > 0) {
        var ctlValue = ctl.options[idx].value;
        // The name of the control contains the text styling command
        var cmd = ctlName.replace('_' + this.name, '');
        this.doCommand(cmd, ctlValue);
        ctl.selectedIndex = 0;
    }
}

/**
 * Handles token insertion
 */
RTControl.prototype.handleInsert = function() {
    var ctl = document.getElementById("tokens_" + this.name);
    if (ctl) {
        var idx = ctl.selectedIndex;
        if (idx > 0) {
            var insStr = ctl.options[idx].value;
            this.insertToken(insStr);
            ctl.selectedIndex = 0;
        }
    }
}

/**
 * Performs a design-mode command
 * @param command the command to run
 * @param option an option value for the command
 */
RTControl.prototype.doCommand = function(command, option) {
    var oRTE = this.getRTE();
    try {
        oRTE.focus();
        oRTE.document.execCommand(command, false, option);
        oRTE.focus();
    }
    catch (e) {
        alert(e);
    }
}

/**
 * Gets a reference to the edit frame belonging to this object
 */
RTControl.prototype.getRTE = function() {
    var ret = null;
    var mode = this.getMode();
    if (PLAINTEXT_MODE == mode) {
        ret = document.getElementById(this.name + "Plain");
    }
    else if (SOURCE_MODE == mode) {
        ret = document.getElementById(this.name + "Src");
    }
    else {
        if (this.richText) {
            if (browserInfo.isIE) {
                ret = frames[this.name];
            }
            else {
                var f = document.getElementById(this.name);
                if (f)
                    ret = f.contentWindow;
            }
        }
        else {
            ret = document.getElementById(this.name);
        }
    }
    return ret;
}

/**
 * Toggles between read-only and editable mode
 */
RTControl.prototype.toggleReadOnly = function(checked) {
    this.toWYSIWYG();
    var html = this.readOnly ? this.currentContent : this.getValue();
    var text = this.readOnly ? this.currentText : this.getText();
    this.readOnly = checked;
    this.buttons = ((this.richText || this.tokens) && !checked) ? true : false;
    if (this.readOnly) {
        this.makeControl(this.container, html, text, true);
    }
    else {
        this.makeControl(this.container, html, text, true);
    }
}

/**
 * Sets editing mode
 */
RTControl.prototype.setEditMode = function(mode, focus) {
    if (mode == this.getMode())
        return;
    if (null == focus)
        focus = true;
    if (mode <= SOURCE_MODE) {
        this.setSource();
    }
    else if (mode >= PLAINTEXT_MODE) {
        this.setPT();
    }
    else {
        this.setWYSIWYG();
    }

    if (focus)
        setTimeout(this.ctlName + ".getRTE().focus();", 50);
}

RTControl.prototype.syncModeCtl = function() {
    var mode = this.getMode();
    var modeCtl = document.getElementById(this.name + "ModeSel");
    if (modeCtl) {
        var opts = modeCtl.options;
        for (var i = 0; i < opts.length; i++) {
            if (opts[i].value == mode) {
                modeCtl.selectedIndex = i;
                break;
            }
        }
        var disabled = (PLAINTEXT_MODE == mode && !this.ptBtn) ? true : false;
        modeCtl.disabled = disabled;
    }
}

/**
 * Sets WYSIWYG mode
 */
RTControl.prototype.setWYSIWYG = function() {
    var mode = this.getMode();
    var rte = this.name;
	
	var rad0 = document.getElementById("radio_0");
	if(rad0)
		document.getElementById("radio_0").checked = true;
		
    var rtE = document.getElementById(rte);
    if (!rte)
        return;

    // Sync and hide the current views
    if (PLAINTEXT_MODE == mode) {
        var ptE = document.getElementById(rte + "Plain");
        if (ptE) {
            ptE.style.display = "none";
        }
    }
    else if (SOURCE_MODE == mode) {
        this.sync_r();
        var srcE = document.getElementById(rte + "Src");
        if (srcE) {
            srcE.style.display = "none";
        }
    }
    // Show the WYSIWYG display and the toolbars
    rtE.style.display = "";
    if (this.buttons) {
        this.showHideToolbars(true);
    }
    if (browserInfo.isGecko && !this.readOnly) {
        try {
            // We need to turn design mode back on
            document.getElementById(rte).contentDocument.designMode = "on";
        }
        catch(e) {alert(e);}
    }
    this.mode = WYSIWYG_MODE;
    this.syncModeCtl();
}

/**
 * Sets source edit mode
 */
RTControl.prototype.setSource = function() {
    var mode = this.getMode();
    var rte = this.name;
    
	var rad0 = document.getElementById("radio_0");
	if(rad0)
		document.getElementById("radio_0").checked = true;
		
	var srcE = document.getElementById(rte + "Src");
    if (!srcE)
        return;

    // Ensure that any popup dialog is dismissed
    showHideElement(this.currentPopup, 'hidden');
    // Hide toolbars
    if (this.buttons) {
        this.showHideToolbars(false);
    }
    // Hide existing edit views
    if (PLAINTEXT_MODE == mode) {
        var ptE = document.getElementById(rte + "Plain");
        if (ptE) {
            ptE.style.display = "none";
        }
    }
    else if (WYSIWYG_MODE == mode) {
        // Sync
        this.sync();
        var rtE = document.getElementById(rte);
        if (rtE)
            rtE.style.display = "none";
    }
    // Show source view
    srcE.style.display = "";
    this.mode = SOURCE_MODE;
    this.syncModeCtl();
}

/**
 * Sets plain-text edit mode
 */
RTControl.prototype.setPT = function() {
    var mode = this.getMode();
    var rte = this.name;
	
	var rad1 = document.getElementById("radio_1");
	if(rad1)
		document.getElementById("radio_1").checked = true;
	
    var ptE = document.getElementById(rte + "Plain");
    if (!ptE)
        return;

    // Ensure that any popup dialog is dismissed
    showHideElement(this.currentPopup, 'hidden');
    // Hide toolbars
    if (this.buttons) {
        this.showHideToolbars(false);
    }
    // Hide existing edit views
    if (SOURCE_MODE == mode) {
        var srcE = document.getElementById(rte + "Src");
        if (srcE) {
            srcE.style.display = "none";
        }
    }
    else if (WYSIWYG_MODE == mode) {
        // Sync
        this.sync();
        var rtE = document.getElementById(rte);
        if (rtE)
            rtE.style.display = "none";
    }
    // Show source view
    ptE.style.display = "";
    this.mode = PLAINTEXT_MODE;
    this.syncModeCtl();
}


/**
 * Gets the text from the control source mode
 */
RTControl.prototype.getSource = function() {
    var html = "";
    var ptE = document.getElementById(this.name + "Src");
    if (ptE) {
        html = ptE.value;
    }
    return html;
}

/**
 * Creates the HTML content for the edit frame
 * @param html the HTML to be edited (goes inside the edit frame's document body)
 */
function makeDocumentHtml(rte, html) {
    var frameHtml = "<html id=\"" + rte + "\">\n";
    frameHtml += "<head>\n";
    // Set the global cssFile variable to specify a stylesheet to use
    if (cssFile.length > 0) {
        frameHtml += "<link media=\"all\" type=\"text/css\" href=\"" + cssFile + "\" rel=\"stylesheet\">\n";
    }
    else if (cssText.length > 0) {
        frameHtml += cssText;
    }
    else {
        // Apply default styles
        frameHtml += "<style>\n";
        frameHtml += "body {\n";
        frameHtml += "  background: #FFFFFF;\n";
        frameHtml += "  margin: 0px;\n";
        frameHtml += "  padding: 5px;\n";
        frameHtml += "}\n";
        frameHtml += "</style>\n";
    }
    frameHtml += "</head>\n";
    frameHtml += "<body>\n";
    frameHtml += html + "\n";
    frameHtml += "</body>\n";
    frameHtml += "</html>";
    return frameHtml;
}

/**
 * Turns on design mode in the edit document (and sets the content since
 * turning on design mode destroys any content already present)
 */
RTControl.prototype.enableDesignMode = function(triedOnce) {
    var rte = this.name;
    if (this.richText) {
        var oRTE = this.getRTE();
        if (null == oRTE) {
            return;
        }
        try {
            var frameHtml = makeDocumentHtml(rte, this.currentContent);
            var doc = oRTE.document;
            if (browserInfo.isIE) {
                doc.open();
                doc.write(frameHtml);
                doc.close();
                if (!this.readOnly) {
                    doc.designMode = "On";
                    oRTE.attachEvent("onfocus", this.changeNotifyFn);
                }
            }
            else {
                if (!this.readOnly)
                    document.getElementById(rte).contentDocument.designMode = "on";
                doc.open();
                doc.write(frameHtml);
                doc.close();
                if (!this.readOnly) {
                    if (browserInfo.isGecko) {
                        doc.addEventListener("keypress", geckoKeyPress, true);
                    }
                    doc.addEventListener("focus", this.changeNotifyFn, true);
                }
            }
        }
        catch(e) {
            if (!triedOnce)
                alert(e.message);
            setTimeout(this.ctlName + ".enableDesignMode(true);", 50);
        }
    }
}

/**
 * Turns off design mode in the edit document
 */
RTControl.prototype.disableDesignMode = function() {
    if (this.richText) {
        var rte = this.name;
        var oRTE = this.getRTE();
        if (null == oRTE) {
            return;
        }
        try {
            this.toWYSIWYG();
            if (browserInfo.isIE) {
                var doc = this.getRTE().document;
                doc.designMode = "Off";
            }
            else {
                var doc = document.getElementById(rte).contentDocument;
                doc.designMode = "off";
            }
        }
        catch(e) {
            alert(e);
        }
    }
}

/**
 * Sets the source HTML and enables design mode (if the control is not read-only)
 * @param html the HTML source
 */
RTControl.prototype.setHTML = function(html) {
    this.sync_r(html);
    this.currentContent = this.initialContent = html;
}

/**
 * Sets the content of the plain-text document
 * @param text the text content
 * @param reinit true if the operation should cause the initialText to be set
 * together with the currentText; defaults to "true"
 */
RTControl.prototype.setText = function(text, reinit) {
    if (null == reinit)
        reinit = true;
    if (this.ptOpt) {
        var ptE = document.getElementById(this.name + "Plain");
        if (ptE)
            ptE.value = text;
        this.currentText = text;
        if (reinit) {
            this.initialText = text;
        }
    }
}

/**
 * Inserts a block of HTML at the current insertion point in the document
 */
RTControl.prototype.insertHTML = function(html) {
    var oRTE = this.getRTE();
    oRTE.focus();
    if (browserInfo.isIE) {
        var oRng = this.currentRange ? this.currentRange :
            oRTE.document.selection.createRange();
        oRng.pasteHTML(html);
        oRng.collapse(false);
        oRng.select();
        this.currentRange = null;
    }
    else {
        oRTE.document.execCommand('insertHTML', false, html);
    }
}

/**
 * Initializes popup dialogs
 */
RTControl.prototype.initForm = function(form) {
    var rte = this.name;
    var defaults = dlgDefaults[form];
    if (defaults) {
        for (var i = 0; i < defaults.length; i++) {
            var id = rte + form + defaults[i].name;
            var e = document.getElementById(id);
            if (e) {
                if (null != defaults[i].value) {
                    e.value = defaults[i].value;
                }
                else if (null != defaults[i].checked) {
                    e.checked = defaults[i].checked;
                }
            }
            else {
            }
        }
    }
}

/**
 * Creates the HTML for a colour-picker dialog
 */
RTControl.prototype.createPalette = function() {
    var rowCount = 7;

    var colors = [["#FFFFFF", "#FFCCCC", "#FFCC99", "#FFFF99", "#FFFFCC",
        "#99FF99", "#99FFFF", "#CCFFFF", "#CCCCFF", "#FFCCFF"],
        ["#CCCCCC", "#FF6666", "#FF9966", "#FFFF66", "#FFFF33",
        "#66FF99", "#33FFFF", "#66FFFF", "#9999FF", "#FF99FF"],
        ["#C0C0C0", "#FF0000", "#FF9900", "#FFCC66", "#FFFF00",
        "#33FF33", "#66CCCC", "#33CCFF", "#6666CC", "#CC66CC"],
        ["#999999", "#CC0000", "#FF6600", "#FFCC33", "#FFCC00",
        "#33CC00", "#00CCCC", "#3366FF", "#6633FF", "#CC33CC"],
        ["#666666", "#990000", "#CC6600", "#CC9933", "#999900",
        "#009900", "#339999", "#3333FF", "#6600CC", "#993399"],
        ["#333333", "#660000", "#993300", "#996633", "#666600",
        "#006600", "#336666", "#000099", "#333399", "#663366"],
        ["#000000", "#330000", "#663300", "#663333", "#333300",
        "#003300", "#003333", "#000066", "#330099", "#330033"]];

    var palette = document.createElement("table");
    palette.className = "shadedTable";
    palette.cellPadding = 0;
    palette.cellSpacing = 1;

    for (var i = 0; i < rowCount; i++) {
        var row = palette.insertRow(i);
        for (var j = 0; j < colors[i].length; j++) {
            var col = colors[i][j];
            var cell = row.insertCell(j);
            cell.id = col;
            cell.bgColor = col;
            cell.style.border = "1px solid gray";
            var fnStr = "this.style.border = '1px solid black';";
            cell.onmouseover = new Function(fnStr);
            fnStr = "this.style.border = '1px solid gray';";
            cell.onmouseout = new Function(fnStr);
            fnStr = this.ctlName + ".setColour(this.id);";
            cell.onclick = new Function(fnStr);
            cell.width = cell.height = 12;
            var img = document.createElement("img");
            img.height = img.width = 1;
            cell.appendChild(img);
        }
    }

    var div = document.createElement("div");
    div.style.visibility = "hidden";
    div.style.position = "absolute";
    div.id = this.name + "palette";
    div.appendChild(palette);

    return div;
}


/**
 * Sets the foreground or background colour in the editor
 * @param colour the colour to set
 */
RTControl.prototype.setColour = function(colour) {
    var parentCommand = parent.command;

    if (browserInfo.isIE) {
        if (parentCommand == "hilitecolor")
            parentCommand = "backcolor";
        this.currentRange.select();
    }

    this.doCommand(parentCommand, colour);
    showHideElement(this.name + "palette", "hidden");
}

/**
 * Creates the content for the link dialog
 */
RTControl.prototype.createLinkDlg = function() {
    var rte = this.name;
    var fName = rte + "linkForm";
    var form = document.createElement("form");
    form.name = fName;
    form.id = fName;
    form.style.visibility = "hidden";
    form.style.position = "absolute";

    var table = document.createElement("table");
    table.className = "shadedTable";
    table.cellPadding = 4;
    table.cellSpacing = 0;
    table.border = 0;

    // URL
    var rowIdx = 0;
    var row = table.insertRow(rowIdx++);
    var cellIdx = 0;
    var cell = row.insertCell(cellIdx++);
    cell.style.fontWeight = "bold";
    cell.appendChild(document.createTextNode(resolveString(rte_st, "url")));
    cell = row.insertCell(cellIdx++);
    cell.appendChild(makeTextField(fName, "url", fName + "Url", "200px"));

    // Link text
    row = table.insertRow(rowIdx++);
    cellIdx = 0;
    cell = row.insertCell(cellIdx++);
    cell.style.fontWeight = "bold";
    cell.appendChild(document.createTextNode(resolveString(rte_st, "text")));
    cell = row.insertCell(cellIdx++);
    cell.appendChild(makeTextField(fName, "linkText", fName + "LinkText", "200px"));

    row = table.insertRow(rowIdx++);
    cellIdx = 0;
    cell = row.insertCell(cellIdx++);
    cell.colSpan = 2;

    // Whether the link should open in a new window
    var input = document.createElement("input");
    input.type = "checkbox";
    input.className = "checkbox";
    input.name = "_blank";
    input.id = fName + "_Blank";
    input.checked = true;
    input.style.marginLeft = "20px";
    cell.appendChild(input);
    cell.appendChild(document.createTextNode(" " + resolveString(rte_st, "newWindow")));

    row = table.insertRow(rowIdx++);
    cellIdx = 0;
    cell = row.insertCell(cellIdx++);
    cell.colSpan = 2;
    cell.align = "center";

    // Submit and cancel
    input = document.createElement("input");
    input.type = "button";
    input.className = "btnSmall";
    input.value = resolveString(rte_st, "insertlink");
    var fnStr = "return " + this.ctlName + ".addLink();"
    input.onclick = new Function(fnStr);
    cell.appendChild(input);
    cell.appendChild(document.createTextNode(" " + String.fromCharCode(NBSP) + " "));
    input = document.createElement("input");
    input.type = "button";
    input.className = "btnSmall";
    input.value = resolveString(rte_st, "cancel");
    var fnStr = "document.getElementById('" + fName + "').style.visibility='hidden';";
    input.onclick = new Function(fnStr);
    cell.appendChild(input);
    form.appendChild(table);

    return form;
}

// Creates onblur function for a text-control
function makeOnBlur(rte, form, field) {
    var formName = rte + form;
    return " onblur=\"rte_vframeworks['" + formName + "'].onchange(this.name);\""
}

/**
 * Adds a link that has been created using the link dialog
 */
RTControl.prototype.addLink = function() {
    var rte = this.name;
    var formName = rte + "linkForm";

    // Validate form
    if (!rte_vframeworks[formName].onsubmit()) {
        return false;
    }

    // We have to use element IDs rather than form names for IE's benefit
    var fElems = ["Url", "LinkText"];
    var linkInfo = {};
    for (var i = 0; i < fElems.length; i++) {
        var eN = fElems[i];
        var e = document.getElementById(formName + eN);
        if (e)
            linkInfo[eN] = e.value;
    }
    var e = document.getElementById(formName + "_Blank");
    if (e && e.checked) {
        linkInfo["target"] = "_blank";
    }

    var linkHtml = '<a href="' + linkInfo["Url"] + '"';
    if (linkInfo["target"]) {
        linkHtml += ' target="_blank"';
    }
    linkHtml += '>' + linkInfo["LinkText"] + '</a>';
    this.insertHTML(linkHtml);

    document.getElementById(formName).style.visibility = 'hidden';
    return false;
}

/**
 * Creates the content for the image dialog
 */
RTControl.prototype.createImageDlg = function() {
    var rte = this.name;
    var fName = rte + "imageForm";
    var form = document.createElement("form");
    form.name = fName;
    form.id = fName;
    form.style.visibility = "hidden";
    form.style.position = "absolute";

    var table = document.createElement("table");
    table.className = "shadedTable";
    table.cellPadding = 4;
    table.cellSpacing = 0;
    table.border = 0;

    // URL
    var rowIdx = 0;
    var row = table.insertRow(rowIdx++);
    var cellIdx = 0;
    var cell = row.insertCell(cellIdx++);
    cell.style.fontWeight = "bold";
    cell.appendChild(document.createTextNode(resolveString(rte_st, "imageurl")));
    cell = row.insertCell(cellIdx++);
    cell.appendChild(makeTextField(fName, "url", fName + "Url", "200px"));

    // Alt-text
    row = table.insertRow(rowIdx++);
    cellIdx = 0;
    cell = row.insertCell(cellIdx++);
    cell.style.fontWeight = "bold";
    cell.appendChild(document.createTextNode(resolveString(rte_st, "altText")));
    cell = row.insertCell(cellIdx++);
    cell.appendChild(makeTextField(fName, "altText", fName + "AltText", "200px"));

    row = table.insertRow(rowIdx++);
    cellIdx = 0;
    cell = row.insertCell(cellIdx++);
    cell.colSpan = 2;

    // Whether the title should be the same as ALT
    var input = document.createElement("input");
    input.type = "checkbox";
    input.className = "checkbox";
    input.name = "setTitle";
    input.id = fName + "SetTitle";
    input.checked = true;
    input.style.marginLeft = "20px";
    cell.appendChild(input);
    cell.appendChild(document.createTextNode(" " + resolveString(rte_st, "altAsTitle")));

    row = table.insertRow(rowIdx++);
    cellIdx = 0;
    cell = row.insertCell(cellIdx++);
    cell.colSpan = 2;
    cell.align = "center";

    // Submit and cancel
    input = document.createElement("input");
    input.type = "button";
    input.className = "btnSmall";
    input.value = resolveString(rte_st, "insertimage");
    var fnStr = this.ctlName + ".addImage();"
    input.onclick = new Function(fnStr);
    cell.appendChild(input);
    cell.appendChild(document.createTextNode(" " + String.fromCharCode(NBSP) + " "));
    input = document.createElement("input");
    input.type = "button";
    input.className = "btnSmall";
    input.value = resolveString(rte_st, "cancel");
    fnStr = "document.getElementById('" + fName + "').style.visibility='hidden';";
    input.onclick = new Function(fnStr);
    cell.appendChild(input);
    form.appendChild(table);
    return form;
}

function makeTextField(fName, name, id, width, maxLength, value) {
    var input = document.createElement("input");
    input.type = "text";
    input.name = name;
    input.id = id;
    input.className = "textField";
    input.style.width = width;
    if (maxLength)
        input.maxLength = maxLength;
    if (null != value)
        input.value = value;

    var fnStr = "rte_vframeworks['" + fName + "'].onchange('" + name + "');";
    input.onblur = new Function(fnStr);
    return input;
}

/**
 * Adds an image that has been specified using the image dialog
 */
RTControl.prototype.addImage = function() {
    var rte = this.name;
    var formName = rte + "imageForm";

    // Validate form
    if (!rte_vframeworks[formName].onsubmit()) {
        return false;
    }

    // We have to use element IDs rather form names for IE's benefit
    var fElems = ["Url", "AltText"];
    var imgInfo = {};
    for (var i = 0; i < fElems.length; i++) {
        var eN = fElems[i];
        var e = document.getElementById(formName + eN);
        if (e)
            imgInfo[eN] = e.value;
    }
    var e = document.getElementById(formName + "SetTitle");
    if (e && e.checked) {
        imgInfo["Title"] = imgInfo["AltText"];
    }

    var imgHtml = '<img src="' + imgInfo["Url"] + '"';
    var alt = imgInfo["AltText"] ? imgInfo["AltText"] : "";
    imgHtml += ' alt="' + alt + '"'
    if (imgInfo["Title"]) {
        imgHtml += ' title="' + imgInfo["Title"] + '"'
    }
    imgHtml += '>';
    this.insertHTML(imgHtml);
    document.getElementById(formName).style.visibility='hidden';
    return false;
}

/**
 * Creates the content for the table dialog
 */
RTControl.prototype.createTableDlg = function() {
    var rte = this.name;
    var fName = rte + "tableForm";
    var form = document.createElement("form");
    form.name = fName;
    form.id = fName;
    form.style.visibility = "hidden";
    form.style.position = "absolute";

    var table = document.createElement("table");
    table.className = "shadedTable";
    table.cellPadding = 4;
    table.cellSpacing = 0;
    table.border = 0;

    // Rows
    var rowIdx = 0;
    var row = table.insertRow(rowIdx++);
    var cellIdx = 0;
    var cell = row.insertCell(cellIdx++);
    cell.style.fontWeight = "bold";
    cell.appendChild(document.createTextNode(resolveString(rte_st, "rows")));
    cell = row.insertCell(cellIdx++);
    cell.appendChild(makeTextField(fName, "rows", fName + "Rows", "40px", 3, 2));
    // Columns
    cell = row.insertCell(cellIdx++);
    cell.style.fontWeight = "bold";
    cell.appendChild(document.createTextNode(resolveString(rte_st, "columns")));
    cell = row.insertCell(cellIdx++);
    cell.appendChild(makeTextField(fName, "columns", fName + "Columns", "40px", 3, 2));

    row = table.insertRow(rowIdx++);
    cellIdx = 0;
    //Table width
    cell = row.insertCell(cellIdx++);
    cell.style.fontWeight = "bold";
    cell.appendChild(document.createTextNode(resolveString(rte_st, "width")));
    cell = row.insertCell(cellIdx++);
    cell.appendChild(makeTextField(fName, "width", fName + "Width", "40px", 4, 100));

    if (browserInfo.isIE) {
        // Percent checkbox
        cell = row.insertCell(cellIdx++);
        cell.colSpan = 2;
        input = document.createElement("input");
        input.type = "checkbox";
        input.className = "checkbox";
        input.id = fName + "WTypepc";
        fnStr = "setWidthValidator('" + rte + "', this.checked);";
        input.onclick = new Function(fnStr);
        cell.appendChild(input);
        cell.appendChild(document.createTextNode(" " + resolveString(rte_st, "usePercent")));
    }
    else {
        // Pixel / percent toggle
        var toggles = [{value:"pc", percent:"true", key:"percent", checked:true},
            {value:"px", percent:"false", key:"pixels"}];
        var input;
        for (var i = 0; i < toggles.length; i++) {
            cell = row.insertCell(cellIdx++);
            input = document.createElement("input");
            input.type = "radio";
            input.name = "wType";
            input.id = fName + "WType" + toggles[i].value;
            if (toggles[i].checked)
                input.checked = true;
            input.value = toggles[i].value;
            input.className = "radio";
            fnStr = "setWidthValidator('" + rte + "', " + toggles[i].percent + ");";
            input.onclick = new Function(fnStr);
            cell.appendChild(input);
            cell.appendChild(document.createTextNode(" " + resolveString(rte_st, toggles[i].key)));
        }
    }

    row = table.insertRow(rowIdx++);
    cellIdx = 0;
    // Border thickness
    cell = row.insertCell(cellIdx++);
    cell.style.fontWeight = "bold";
    cell.appendChild(document.createTextNode(resolveString(rte_st, "thickness")));
    cell = row.insertCell(cellIdx++);
    cell.appendChild(makeTextField(fName, "border", fName + "Border", "40px", 3, 1));
    cell = row.insertCell(cellIdx++);
    cell.colSpan = 2;
    cell.appendChild(document.createTextNode(" " + resolveString(rte_st, "pixels")));

    row = table.insertRow(rowIdx++);
    cellIdx = 0;

    // Padding
    cell = row.insertCell(cellIdx++);
    cell.style.fontWeight = "bold";
    cell.appendChild(document.createTextNode(resolveString(rte_st, "padding")));
    cell = row.insertCell(cellIdx++);
    cell.appendChild(makeTextField(fName, "padding", fName + "Padding", "40px", 3, 4));
    // Spacing
    cell = row.insertCell(cellIdx++);
    cell.style.fontWeight = "bold";
    cell.appendChild(document.createTextNode(resolveString(rte_st, "spacing")));
    cell = row.insertCell(cellIdx++);
    cell.appendChild(makeTextField(fName, "spacing", fName + "Spacing", "40px", 3, 0));

    row = table.insertRow(rowIdx++);
    cellIdx = 0;
    cell = row.insertCell(cellIdx++);
    cell.colSpan = 4;
    cell.align = "center";

    // Submit and cancel
    input = document.createElement("input");
    input.type = "button";
    input.className = "btnSmall";
    input.value = resolveString(rte_st, "inserttable");
    var fnStr = "return " + this.ctlName + ".addTable();"
    input.onclick = new Function(fnStr);
    cell.appendChild(input);
    cell.appendChild(document.createTextNode(" " + String.fromCharCode(NBSP) + " "));
    input = document.createElement("input");
    input.type = "button";
    input.className = "btnSmall";
    input.value = resolveString(rte_st, "cancel");
    var fnStr = "document.getElementById('" + fName + "').style.visibility='hidden';";
    input.onclick = new Function(fnStr);
    cell.appendChild(input);
    form.appendChild(table);
    return form;
}

// The validation for the table width depends on whether or not pixel or
// percentage width is selected
function setWidthValidator(rte, percent) {
    var formName = rte + "tableForm";
    var fieldName = "width";
    var vf = rte_vframeworks[formName];
    // Remove any existing validators
    vf.clearValidators(fieldName);
    vf.addValidator(fieldName, reqV);
    if (percent) {
        vf.addValidator(fieldName, pcV);
    }
    else {
        vf.addValidator(fieldName, mt1V);
    }
}

/**
 * Adds a table that has been specified using the image dialog
 */
RTControl.prototype.addTable = function() {
    var rte = this.name;
    var formName = rte + "tableForm";

    // Validate form
    if (!rte_vframeworks[formName].onsubmit()) {
        return false;
    }

    // We have to use element IDs rather than form names for IE's benefit
    var fElems = ["Border", "Padding", "Spacing", "Width", "Rows", "Columns"];
    var tblInfo = {};
    var e = document.getElementById(formName + "WTypepc");
    if (e && e.checked)
        tblInfo["units"] = "%";
    else
        tblInfo["units"] = "";
    for (var i = 0; i < fElems.length; i++) {
        var eN = fElems[i];
        var e = document.getElementById(formName + eN);
        if (e)
            tblInfo[eN] = e.value;
    }

    var tblHtml = '<table border="' + tblInfo["Border"] + '" cellpadding="' + tblInfo["Padding"] + '" ';
    tblHtml += 'cellspacing="' + tblInfo["Spacing"] + '" width="' + tblInfo["Width"] + tblInfo["units"] + '">\n';
    var rowCount = tblInfo["Rows"];
    var colCount = tblInfo["Columns"];
    for (var i = 0; i < rowCount; i++) {
        tblHtml += '<tr>\n';
        for (var j = 0; j < colCount; j++) {
            tblHtml += '<td>&nbsp;</td>\n';
        }
        tblHtml += '</tr>\n'
    }
    tblHtml += "</table>\n";
    this.insertHTML(tblHtml);

    document.getElementById(formName).style.visibility = 'hidden';
    return false;
}

/**
 * Shows the popup dialog underneath a toolbar button
 * @param command the command being run
 * @param dlg the dialog to show
 * @param showRight whether the dialog should be displayed aligned to the right
 * hand side of the button extending to the left
 */
RTControl.prototype.popDlg = function(command, dlg, showRight) {
    if (null == this.currentRange && browserInfo.isIE) {
        var oRTE = this.getRTE();
        oRTE.focus();
    }
    var rte = this.name;
    if (null == showRight) {
        showRight = false;
    }
    // Get dialog position
    var dlgId = rte + dlg;
    var oDialog = document.getElementById(dlgId);
    var buttonElement = document.getElementById(command + '_' + rte);
    var iLeftPos = getOffsetLeft(buttonElement);
    var iTopPos = getOffsetTop(buttonElement) + (buttonElement.offsetHeight);
    if (showRight) {
        iLeftPos -= (oDialog.offsetWidth - buttonElement.offsetWidth);
    }
    if (browserInfo.isIE) iLeftPos -= 5;
    oDialog.style.left = (iLeftPos) + "px";
    oDialog.style.top = (iTopPos) + "px";

    if (command == parent.command && dlgId == this.currentPopup) {
        // If current command dialog is currently open, close it
        if (oDialog.style.visibility == "visible") {
            showHideElement(oDialog, "hidden");
            dlgId = "";
        }
        else {
            showHideElement(oDialog, 'visible');
        }
    }
    else {
        if (this.currentPopup) {
            showHideElement(this.currentPopup, 'hidden');
        }
        showHideElement(oDialog, 'visible');
    }

    // Save the current insertion position
    this.currentRange = this.getRange();

    // Save current values
    parent.command = command;
    this.currentPopup = dlgId;
}

/**
 * Gets the current insertion point and / or text selection
 */
RTControl.prototype.getRange = function() {
    var rng = null;
    var oRTE = this.getRTE();
    if (document.all) {
        var selection = oRTE.document.selection;
        if (selection != null) rng = selection.createRange();
    } else {
        var selection = oRTE.getSelection();
        rng = selection.getRangeAt(selection.rangeCount - 1).cloneRange();
    }
    return rng;
}

/**
 * Inserts a token at the insertion point
 * @param token the token to insert
 * @param escape true if the token is to be HTML-escaped
 */
RTControl.prototype.insertToken = function(token, escape) {
    if (!token)
        return;
    if (escape)
        token = htmlEscape(token);
    if (this.richText && WYSIWYG_MODE == this.getMode()) {
        this.insertHTML(token);
    }
    else {
        var textArea = this.getRTE();
        if (textArea.setSelectionRange) {
            var currV = textArea.value;
            var selStart = textArea.selectionStart;
            var selEnd = textArea.selectionEnd;
            var selLen = selEnd - selStart;
            var newV = currV.substring(0, selStart) + token +
                currV.substring(selEnd, textArea.value.length);
            textArea.value = newV;
            textArea.selectionStart = textArea.selectionEnd = selEnd + token.length - selLen;
            textArea.focus();
        }
        else {
            textArea.focus();
            var range = document.selection.createRange();
            range.text = token;
        }
    }
}

/**
 * Shows / hides the color palette dialog
 * @param the color command (foreground or background)
 */
RTControl.prototype.dlgColorPalette = function(command) {
    this.popDlg(command, "palette", false);
}

/**
 * Shows / hides the insert link dialog
 * @param command - fixed string "link"
 */
RTControl.prototype.dlgInsertLink = function(command) {
    var rte = this.name;
    var fName = "linkForm";
    this.initForm(fName);
    this.popDlg(command, fName, true);

    // Get currently highlighted text and set link text value
    var range = this.getRange();
    var linkText = '';
    if (browserInfo.isIE) {
        linkText = stripHTML(range.htmlText);
    }
    else {
        linkText = stripHTML(range.toString());
    }

    document.getElementById(rte + fName + "LinkText").value = linkText;

    setTimeout("try {document.forms['" + rte + fName + "'].elements[0].focus()} catch(e){}", 50);
}

/**
 * Shows / hides the insert image dialog
 * @param command - fixed string "image"
 */
RTControl.prototype.dlgInsertImage = function(command) {
    var rte = this.name;
    var fName = "imageForm";
    this.initForm(fName);
    this.popDlg(command, fName, true);
    setTimeout("try {document.forms['" + rte + fName + "'].elements[0].focus()} catch(e){}", 50);
}

/**
 * Shows / hides the insert table dialog
 * @param command - fixed string "table"
 */
RTControl.prototype.dlgInsertTable = function(command) {
    var rte = this.name;
    var fName = "tableForm";
    //this.initForm(fName);
    setWidthValidator(rte, true);
    this.popDlg(command, fName, true);
    setTimeout("try {document.forms['" + rte + fName + "'].elements[0].focus()} catch(e){}", 50);
}

/**
 * Gets the value from the control
 */
RTControl.prototype.getValue = function() {
    var html = "";
    if (WYSIWYG_MODE == this.getMode()) {
        this.sync();
    }
    var rte = this.name;
    if (this.created) {
        var eSfx = this.richText ? "Src" : "";
        var e = document.getElementById(rte + eSfx);
        if (e)
            html = e.value;
    }
    return html;
}

/**
 * Gets the content of the plain-text document
 * @return the text content
 */
RTControl.prototype.getText = function() {
    var text = "";
    if (this.ptOpt) {
        var ptE = document.getElementById(this.name + "Plain");
        if (ptE)
            text = ptE.value;
    }
    return text;
}

/**
 * Syncs the object's source field with the edit control
 */
RTControl.prototype.sync = function() {
    if (!this.richText || this.readOnly || !this.created) {
        return;
    }
    var rte = this.name;
    var e = document.getElementById(rte + 'Src');
    if (!e)
        return;
    var currMode = this.getMode();
    this.mode = WYSIWYG_MODE;
    try {
        var oRTE = this.getRTE();
        if (oRTE) {
            e.value = trim(oRTE.document.body.innerHTML);
        }
    }
    catch(e) {}
    this.mode = currMode;
}

/**
 * Syncs the edit control with the object's source field
 * @param html optional document content to set
 */
RTControl.prototype.sync_r = function(html) {
    if (!this.created) {
        return;
    }
    var rte = this.name;
    if (this.richText) {
        var ptE = document.getElementById(rte + 'Src');
        if (!ptE)
            return;
        if (null == html)
            html = ptE.value;
        else
            ptE.value = html;
        var currMode = this.getMode();
        this.mode = WYSIWYG_MODE;
        try {
            var oRTE = this.getRTE();
            if (oRTE) {
                oRTE.document.body.innerHTML = html;
            }
        }
        catch(e) {}
        this.mode = currMode;
    }
    else {
        var oRTE = this.getRTE();
        if (null == html)
            html = oRTE.value;
        else
            oRTE.value = html;
    }
    this.currentContent = html;
}

/**
 * Forces the control to WYSIWYG display so that it is in a known state
 * for syncing
 */
RTControl.prototype.toWYSIWYG = function() {
    if (PLAINTEXT_MODE == this.getMode()) {
        // Go to HTML mode
        this.togglePT(false);
    }
    if (this.richText && SOURCE_MODE == this.getMode()) {
        this.toggleHTMLSrc(false, false);
    }
}


/**
 * Resets the edit control to its initial content
 */
RTControl.prototype.reset = function() {
    var rte = this.name;
    if (this.created) {
        var csCtl = document.getElementById("chkSrc" + rte);
        if (csCtl && csCtl.checked) {
            var ptE = document.getElementById(rte + 'Src');
            if (ptE)
                ptE.value = this.initialContent;
        }
        else {
            this.setHTML(this.initialContent);
        }
        this.setText(this.initialText);
    }
}

/**
 * Sets the contents of the token list
 * @param tList a list of IToken objects
 */
RTControl.prototype.makeTokenList = function(tList) {
    if (!this.tokens)
        return;
    var sel = document.getElementById("tokens_" + this.name);
    if (!sel)
        return;
    // Empty the current token list
    var opts = sel.options;
    while (sel.length > 1) {
        sel.remove(sel.length - 1);
    }
    if (null == tList || 0 == tList.length) {
        // Disable the control
        sel.disabled = true;
        return;
    }
    sel.disabled = false;
    // Recreate the tokens list
    for (var i = 0; i < tList.length; i++) {
        var opt = tList[i].toOption();
        if (browserInfo.isIE)
            sel.add(opt);
        else
            sel.add(opt, null);
    }
}


/******************************************************************************/
// Utility functions

// Creates an empty document
function emptyDoc() {
    return "<html></html>";
}

/**
 * Shows or hides a popup dialog
 * @param element the element to hide (may be a string)
 * @param showState "visible" or "hidden"
 */
function showHideElement(element, showState) {
    // Element variable can be string or object
    if (document.getElementById(element)) {
        element = document.getElementById(element);
    }
    if (element)
        element.style.visibility = showState;
}


/**
 * Gets y-axis position of an element
 * @param elm the element of interest
 */
function getOffsetTop(elm) {
    var mOffsetTop = elm.offsetTop;
    var mOffsetParent = elm.offsetParent;
    var parents_up = 2; // the positioning div is 2 elements up the tree

    while(parents_up > 0) {
        mOffsetTop += mOffsetParent.offsetTop;
        mOffsetParent = mOffsetParent.offsetParent;
        parents_up--;
    }

    return mOffsetTop;
}

/**
 * Gets x-axis position of an element
 * @param elm the element of interest
 */
function getOffsetLeft(elm) {
    var mOffsetLeft = elm.offsetLeft;
    var mOffsetParent = elm.offsetParent;
    var parents_up = 2;

    while(parents_up > 0) {
        mOffsetLeft += mOffsetParent.offsetLeft;
        mOffsetParent = mOffsetParent.offsetParent;
        parents_up--;
    }

    return mOffsetLeft;
}

/**
 * Removes markup from an input string
 */
function stripHTML(oldString) {
    // Remove markup
    var newString = oldString.replace(/(<([^>]+)>)/ig,"");

    // Replace carriage returns and line feeds
    newString = newString.replace(/\r\n/g," ");
    newString = newString.replace(/\n/g," ");
    newString = newString.replace(/\r/g," ");

    // Trim return value
    return trim(newString);
}

/**
 * Escapes strings for inserting into an HTML stream
 */
function htmlEscape(s) {
    // Escape ampersand
    var ret = s.replace(/&/g, "&amp;");
    // Escape < character
    ret = ret.replace(/</g, "&lt;");
    return ret;
}

/**
 * Escapes quote characters in a string
 * @param s the string to escape
 * @param qCh the quote character to escape (defaults to ')
 */
function jsEscapeQuotes(s, qCh) {
    if (null == qCh)
        qCh = "'";
    var repStr = "\\" + qCh;
    var pattern = new RegExp(qCh, "g");
    return s.replace(pattern, repStr);
}

/**
 * Tokens, delimited by % markers, may be mistakenly interpreted as
 * containing escaped characters (e.g. "%DETECTIONS%" may be interpreted
 * as starting with "%DE", which is a thorn character
 * This function escaped percent signs
 */
function escapePercents(s) {
    var pattern = new RegExp("%", "g");
    var rep = "&#037;";
    return s.replace(pattern, rep);
}

/**
 * Removes leading and trailing spaces from the passed string. Also removes
 * consecutive spaces and replaces it with one space. If something besides
 * a string is passed in (null, custom object, etc.) then return the input.
 */
function trim(inputString) {
    if (typeof inputString != "string")
        return inputString;
    var ret = inputString;

    // Strip leading whitespace
    ret = ret.replace(/^\s+/, "");
    // Strip trailing whitespace
    ret = ret.replace(/\s+$/, "");
    // Collapse multiple internal whitespace characters into a single space
    ret = ret.replace(/\s{2,}/g, " ");
    return ret;
}

/**
 * Determines whether or not a given element is visible; note that for this
 * to work, the element must have had 'style="display:none"' or
 * 'style="visibility:hidden"' explicitly set
 */
function isVisible(element) {
    var ret = true;
    for (var e = element; e != null; e = e.parentNode) {
        ret = isVisible2(e);
        if (!ret)
            break;
    }
    return ret;
}

function isVisible2(e) {
    var ret = true;
    if (e && e.style) {
        if ("none" == e.style.display || "hidden" == e.style.visibility) {
            ret = false;
        }
    }
    return ret;
}

/**
 * Adds bold, italic and underline shortcuts to Gecko browsers
 */
function geckoKeyPress(evt) {
    var rte = evt.target.id;
    if (null == rteMap[rte])
        return;

    if (evt.ctrlKey) {
        var key = String.fromCharCode(evt.charCode).toLowerCase();
        var cmd = '';
        switch (key) {
            case 'b': cmd = "bold"; break;
            case 'i': cmd = "italic"; break;
            case 'u': cmd = "underline"; break;
        };

        if (cmd) {
            rteMap[rte].doCommand(cmd, null);

            // stop the event bubble
            evt.preventDefault();
            evt.stopPropagation();
        }
    }
}

// IE button functions
function raiseButton(e) {
    var el = window.event.srcElement;

    className = el.className;
    if (className == 'rteImage' || className == 'rteImageLowered') {
        el.className = 'rteImageRaised';
    }
}

function normalButton(e) {
    var el = window.event.srcElement;

    className = el.className;
    if (className == 'rteImageRaised' || className == 'rteImageLowered') {
        el.className = 'rteImage';
    }
}

function lowerButton(e) {
    var el = window.event.srcElement;

    className = el.className;
    if (className == 'rteImage' || className == 'rteImageRaised') {
        el.className = 'rteImageLowered';
    }
}
